package de.heinerkuecker;
import java.io.File;

import de.heinerkuecker.asciiartflowparser.FlowConnectionTextArrAndEndNodeValueObj;
import de.heinerkuecker.asciiartflowparser.FlowNet;
import de.heinerkuecker.asciiartflowparser.FlowNetParseException;
import de.heinerkuecker.asciiartflowparser.FlowNodeAndConnectionsValueObj;
import de.heinerkuecker.statetransitionengine.DestinationState;
import de.heinerkuecker.statetransitionengine.SourceState;
import de.heinerkuecker.statetransitionengine.StateTransitEvent;
import de.heinerkuecker.statetransitionengine.StateTransitionEngine;
import de.heinerkuecker.statetransitionengine.StateTransitionEngineIsInvalidException;
import de.heinerkuecker.util.StringUtil;
import de.heinerkuecker.xmlbuilder.XmlDocument;
import de.heinerkuecker.xmlbuilder.XmlDocument.Element;


/**
 * Test-Klasse zum Einparsen eines 
 * Flussdiagrammes in ASCII-Art.
 * 
 * @author Heiner Kcker
 */
public final class TestFlowParser 
{
	/**
	 * Projekt-Folder
	 */
	public static final String HOME_DIR_PATH = System.getProperty( "user.dir" );
	
	/**
	 * Test-File-Folder
	 */
	public static final String DATA_FILE_PATH = HOME_DIR_PATH + File.separator + "test_data";
	
	/**
     * Test Main.
     * 
	 * @param args
	 */
	public static void main(String[] args) 
		throws Throwable
	{
        // Positiv-Test's ohne Verbindungs-Text
        testParse( "flow_01.txt", "check_01.txt", true, 0, 0 , true , true );
        testParse( "flow_02.txt", "check_02.txt", true, 0, 0 , true , true );
        testParse( "flow_03.txt", "check_03.txt", true, 0, 0 , true , true );
        testParse( "flow_04.txt", "check_04.txt", true, 0, 0 , true , true );
        testParse( "flow_05.txt", "check_05.txt", true, 0, 0 , true , false );
        testParse( "flow_06.txt", "check_06.txt", true, 0, 0 , true , false );
        testParse( "flow_07.txt", "check_07.txt", true, 0, 0 , true , false );
        testParse( "flow_08.txt", "check_08.txt", true, 0, 0 , true , false );
        testParse( "flow_09.txt", "check_09.txt", true, 0, 0 , true , false );
        testParse( "flow_10.txt", "check_10.txt", true, 0, 0 , true , false );
        testParse( "flow_11.txt", "check_11.txt", true, 0, 0 , true , true );
        testParse( "flow_12.txt", "check_12.txt", true, 0, 0 , false , false );
        testParse( "flow_13.txt", "check_13.txt", true, 0, 0 , false , false );
        testParse( "flow_14.txt", "check_14.txt", true, 0, 0 , true , false );
        testParse( "flow_15.txt", "check_15.txt", true, 0, 0 , true , true );
        testParse( "flow_16.txt", "check_16.txt", true, 0, 0 , true , false );
        testParse( "flow_17.txt", "check_17.txt", true, 0, 0 , true , false );
        testParse( "flow_18.txt", "check_18.txt", true, 0, 0 , true , false );
        testParse( "flow_19.txt", "check_19.txt", true, 0, 0 , true , false );
        testParse( "flow_20.txt", "check_20.txt", true, 0, 0 , false , false );
        testParse( "flow_21.txt", "check_21.txt", true, 0, 0 , true , false );
        testParse( "flow_22.txt", "check_22.txt", true, 0, 0 , true , true );

        // Positiv-Test's mit Verbindungs-Text
        testParse( "flow_31.txt", "check_31.txt", true, 1, 1 , true , true );
        testParse( "flow_32.txt", "check_32.txt", true, 2, 2 , true , false );
        testParse( "flow_33.txt", "check_33.txt", true, 1, 1 , true , true );
        testParse( "flow_34.txt", "check_34.txt", true, 1, 1 , true , true );
        testParse( "flow_35.txt", "check_35.txt", true, 2, 2 , true , false );
        testParse( "flow_36.txt", "check_36.txt", true, 1, 1 , true , true );
        testParse( "flow_37.txt", "check_37.txt", true, 2, 2 , true , false );
        testParse( "flow_38.txt", "check_38.txt", true, 1, 1 , true , true );
        testParse( "flow_39.txt", "check_39.txt", true, 1, 1 , true , true );
        testParse( "flow_40.txt", "check_40.txt", true, 1, 1 , true , true );
        testParse( "flow_41.txt", "check_41.txt", true, 1, 1 , true , true );
        testParse( "flow_42.txt", "check_42.txt", true, 1, 1 , true , true );
        testParse( "flow_43.txt", "check_43.txt", true, 1, 1 , true , true );
        testParse( "flow_44.txt", "check_44.txt", true, 1, 1 , true , true );
        testParse( "flow_45.txt", "check_45.txt", true, 1, 1 , true , true );
        testParse( "flow_46.txt", "check_46.txt", true, 1, 1 , true , true );
        testParse( "flow_47.txt", "check_47.txt", true, 1, 1 , true , true );
        testParse( "flow_48.txt", "check_48.txt", true, 1, 1 , true , true );
        testParse( "flow_49.txt", "check_49.txt", true, 1, 1 , true , true );
        testParse( "flow_50.txt", "check_50.txt", true, 1, 1 , true , true );
        testParse( "flow_51.txt", "check_51.txt", true, 1, 1 , true , true );
        testParse( "flow_52.txt", "check_52.txt", true, 1, 1 , true , true );
        testParse( "flow_53.txt", "check_53.txt", true, 1, 1 , true , true );
        testParse( "flow_54.txt", "check_54.txt", true, 1, 1 , true , false );
        testParse( "flow_55.txt", "check_55.txt", true, 1, 1 , true , true );
        testParse( "flow_56.txt", "check_56.txt", true, 1, 1 , true , true );
        testParse( "flow_57.txt", "check_57.txt", true, 1, 1 , true , false );
        testParse( "flow_58.txt", "check_58.txt", true, 1, 2 , true , false );
        testParse( "flow_59.txt", "check_59.txt", true, 1, 1 , true , true );
        testParse( "flow_60.txt", "check_60.txt", true, 0, 0 , false , false );
        testParse( "flow_61.txt", "check_61.txt", true, 1, 1 , false , false );
        testParse( "flow_62.txt", "check_62.txt", true, 0, 1 , true , true );
        testParse( "flow_63.txt", "check_63.txt", true, 1, 2 , true , false );
        testParse( "flow_64.txt", "check_64.txt", true, 0, 1 , true , true );
        testParse( "flow_65.txt", "check_65.txt", true, 1, 2 , true , false );
        testParse( "flow_66.txt", "check_66.txt", true, 1, 1 , true , true );
        testParse( "flow_67.txt", "check_67.txt", true, 2, 2 , true , false );
        testParse( "flow_68.txt", "check_68.txt", true, 1, 1 , false , false );
        testParse( "flow_69.txt", "check_69.txt", true, 2, 2 , false , false );
        testParse( "flow_70.txt", "check_70.txt", true, 1, 2 , true , false );
        testParse( "flow_71.txt", "check_71.txt", true, 2, 3 , true , false );
        testParse( "flow_72.txt", "check_72.txt", true, 1, 2 , true , false );
        testParse( "flow_73.txt", "check_73.txt", true, 2, 3 , true , false );
        testParse( "flow_74.txt", "check_74.txt", true, 2, 2 , true , false );
        testParse( "flow_75.txt", "check_75.txt", true, 3, 3 , true , false );
        testParse( "flow_76.txt", "check_76.txt", true, 4, 9 , true , false );
        testParse( "flow_77.txt", "check_77.txt", true, 4, 4 , true , false );
        testParse( "flow_78.txt", "check_78.txt", true, 4, 4 , true , false );
        testParse( "flow_79.txt", "check_79.txt", true, 7, 7 , true , false );
        testParse( "flow_80.txt", "check_80.txt", true, 7, 7 , true , false );
        testParse( "flow_81.txt", "check_81.txt", true, 3, 7 , true , false );
        testParse( "flow_82.txt", "check_82.txt", true, 4, 8 , true , false );
        testParse( "flow_83.txt", "check_83.txt", true, 4, 8 , true , false );
        testParse( "flow_84.txt", "check_84.txt", true, 4, 9 , true , false );
        testParse( "flow_85.txt", "check_85.txt", true, 4, 9 , true , false );
        testParse( "flow_86.txt", "check_86.txt", true, 4, 9 , true , false );
        testParse( "flow_87.txt", "check_87.txt", true, 4, 9 , true , false );
        testParse( "flow_88.txt", "check_88.txt", true, 2, 9 , true , false );
        testParse( "flow_89.txt", "check_89.txt", true, 2, 9 , true , false );
        testParse( "flow_90.txt", "check_90.txt", true, 3, 7 , true , false );
        testParse( "flow_91.txt", "check_91.txt", true, 4, 8 , true , false );
        testParse( "flow_92.txt", "check_92.txt", true, 4, 8 , true , false );
        testParse( "flow_93.txt", "check_93.txt", true, 4, 8, true , false );
        testParse( "flow_94.txt", "check_94.txt", true, 2, 6 , true , false );
        testParse( "flow_95.txt", "check_95.txt", true, 2, 6, true , false );
//   TODO: mache aus der 95 noch ein paar testflle     testParse( "flow_96.txt", "check_96.txt", true, 999, 999 , true , false );
//        testParse( "flow_97.txt", "check_97.txt", true, 999, 999 , true , false );
//        testParse( "flow_98.txt", "check_98.txt", true, 999, 999 , true , false );
//        testParse( "flow_99.txt", "check_99.txt", true, 999, 999 , true , false );

        
        testParse( "flow_search_example.txt", "check_search_example.txt", true, 0, 1 , true , true );

        FlowNet flowNet;

        /*
         * Beispiele zum Erzeugen von XML fr die Java-Server-Faces-Navigation
         */
        // TODO check XML
        flowNet = testParse( "flow_jsf_navigation__xml.txt", "check_jsf_navigation__xml.txt", true, 1, 1 , true , true );
        createJsfNavigationXml( flowNet );
        
        flowNet = testParse( "flow_jsf_navigation_default_xml.txt", "check_jsf_navigation_default_xml.txt", true, 0, 1 , true , true );
        createJsfNavigationXml( flowNet );
        
        
        /*
         * Beipiel fr Anbindung der State-Transition-Engine
         */
        createStateTransitionEngine( flowNet );
        
        
        /*
         * Negativ-Test's
         */
        System.out.println( "" );
        System.out.println( "##################################################################################################" );
        System.out.println( "#                                                                                                #" );
        System.out.println( "#                                 Negativ - Test's                                               #" );
        System.out.println( "#                                                                                                #" );
        System.out.println( "##################################################################################################" );
        System.out.println( "" );


        testParse( "flow_err_01.txt", "err_check_01.txt", false, 999, 999 , true , false );
        testParse( "flow_err_02.txt", "err_check_02.txt", false, 999, 999 , true , false );
        testParse( "flow_err_03.txt", "err_check_03.txt", false, 999, 999 , true , false );
        testParse( "flow_err_04.txt", "err_check_04.txt", false, 999, 999 , true , false );
        testParse( "flow_err_05.txt", "err_check_05.txt", false, 999, 999 , true , false );
        testParse( "flow_err_06.txt", "err_check_06.txt", false, 999, 999 , true , false );
        testParse( "flow_err_07.txt", "err_check_07.txt", false, 999, 999 , true , false );
        testParse( "flow_err_08.txt", "err_check_08.txt", false, 999, 999 , true , false );
        testParse( "flow_err_09.txt", "err_check_09.txt", false, 999, 999 , true , false );
        testParse( "flow_err_10.txt", "err_check_10.txt", false, 999, 999 , true , false );
        testParse( "flow_err_11.txt", "err_check_11.txt", false, 999, 999 , true , false );
        testParse( "flow_err_12.txt", "err_check_12.txt", false, 999, 999 , true , false );
        testParse( "flow_err_13.txt", "err_check_13.txt", false, 999, 999 , true , false );
        testParse( "flow_err_14.txt", "err_check_14.txt", false, 999, 999 , true , false );
        // TODO noch unterschiedliche Varianten mit flow_err_14.txt
        testParse( "flow_err_15.txt", "err_check_15.txt", false, 999, 999 , true , false );
        // Testfall Anzahl Texte per Connection
        testParse( "flow_err_16.txt", "err_check_16.txt", false, 1, 1 , true , false );
        testParse( "flow_err_17.txt", "err_check_17.txt", false, 3, 3 , true , false );

        // TODO Testfall Anzahl Texte per Connection, weitere Kombinationen

        System.out.println( "\n !!! alles ok !!!" );
	}

	/**
     * Test einer einzelnen Diagramm-Datei
     * mit mehreren Formatierungen.
     */
    private static FlowNet testParse(
        final String pDiagramFileName, 
        final String pCheckFileName,
        final boolean pNoException, 
        final int pMinTextCountPerConnection, 
        final int pMaxTextCountPerConnection, 
        final boolean pCheckConnectionUniqueness,
        final boolean pCheckTransitionUniqueness )
    	throws Throwable
    {
        FlowNet flowNet = null;
	    /**
	     * Test mit und ohne Platz links, rechts, oben und unten.
	     * 
	     * @param pFlowStrArr
	     */
    	final int INDENTATION_VALUE_RANGE = 2;
    	final int RIGHT_SPACE_VALUE_RANGE = 2;
    	final int TOP_SPACE_VALUE_RANGE = 3;
    	final int BOTTOM_SPACE_VALUE_RANGE = 1;
    	
        // Einlesen der Diagramm-Datei in ein String-Array
        final String[] originalFlowStrArr = StringUtil.fileToStrArr( DATA_FILE_PATH + File.separator + pDiagramFileName );

        /**
    	 * Schalter fr Test mit und ohne Leerrume an allen Seiten, ein-aus
    	 */
    	final boolean isTestForSpaceAndNoSpace = false;
    	
    	for ( int i1 = 0; ( isTestForSpaceAndNoSpace && i1 <= INDENTATION_VALUE_RANGE ) || i1 == 0 ; i1++ )
    	{
        	for ( int i2 = 0;  ( isTestForSpaceAndNoSpace && i2 <= RIGHT_SPACE_VALUE_RANGE ) || i2 == 0 ; i2++ )
        	{
            	for ( int i3 = 0;  ( isTestForSpaceAndNoSpace && i3 <= TOP_SPACE_VALUE_RANGE ) || i3 == 0 ; i3++ )
            	{
                	for ( int i4 = 0;  ( isTestForSpaceAndNoSpace && i4 <= BOTTOM_SPACE_VALUE_RANGE ) || i4 == 0 ; i4++ )
                	{
						final boolean isUnformatted = i1 == 0 && i2 == 0 && i3 == 0 && i4 == 0;
                		System.out.println( "==================================================================================================" );
                        System.out.println( pDiagramFileName );
                        if ( isUnformatted )
                        {
                            System.out.println();
                        }
                        else
                        {
                            System.out.println( "i1: " + i1 + " i2: " + i2 + " i3: " + i3 + " i4: " + i4 );
                        }

                        // flowStrArr = StringUtil.arrTrim( flowStrArr );
                		// Kopieren zum Schutz des Originals gegen Seiteneffekte
                		String[] flowStrArr = StringUtil.cloneStrArr( originalFlowStrArr );
                		if ( i1 == 1 )
                		{
                			// links straff ranziehen
                			flowStrArr = StringUtil.removeIndentation( flowStrArr );
                			System.out.println( "links straff ranziehen" );
                		}
                		else if ( i1 == 2 )
                		{
                			// links Luft (Leerzeichen) reinbringen
                			flowStrArr = StringUtil.insertForAllLines( flowStrArr, "    " );
                			System.out.println( "links Luft (Leerzeichen) reinbringen" );
                		}
                		if ( i2 == 1 )
                		{
                			// rechts trimmen
                			flowStrArr = StringUtil.rTrimStrArr( flowStrArr );
                			System.out.println( "rechts trimmen" );
                		}
                		else if ( i2 == 2 )
                		{
                			// rechts Luft (Leerzeichen)
                			flowStrArr = StringUtil.arrAllLinesAdd( flowStrArr, "    " );
                			System.out.println( "rechts Luft (Leerzeichen)" );
                		}
                		else if ( i2 == 3 )
                		{
                			// rechts Luft (Leerzeichen) alle Zeilen auf maximale Lnge
                			flowStrArr = StringUtil.arrAllLinesPad( 
                					flowStrArr, 
                					StringUtil.getArrMaxStrLen( flowStrArr ) + 4 );
                			System.out.println( "rechts Luft (Leerzeichen) alle Zeilen auf maximale Lnge" );
                		}
                		if ( i3 == 1 )
                		{
                			// oben trimmen
                			flowStrArr = StringUtil.arrTopTrim( flowStrArr );
                			System.out.println( "oben trimmen" );
                		}
                		else if ( i3 == 2 )
                		{
                			// oben Luft (Leerzeilen)
                			flowStrArr = StringUtil.arrAdd( new String[]{ "" , "" , "" , "" }, flowStrArr );
                			System.out.println( "oben Luft (Leerzeilen)" );
                		}
                		else if ( i3 == 3 )
                		{
                			// oben Luft (Leerzeilen) alle Zeilen auf maximale Lnge
                			flowStrArr = StringUtil.arrAdd( 
                					StringUtil.fillStrArr( 
                							4 , 
                							StringUtil.spaces( 
                									StringUtil.getArrMaxStrLen( flowStrArr ) + 4 ) ) , 
                									flowStrArr );
                			System.out.println( "oben Luft (Leerzeilen) alle Zeilen auf maximale Lnge" );
                		}
                		if ( i4 == 1 )
                		{
                			// unten trimmen
                			flowStrArr = StringUtil.arrTrim( flowStrArr );
                			System.out.println( "unten trimmen" );
                		}
                		else if ( i4 == 2 )
                		{
                			// unten Luft (Leerzeilen)
                			flowStrArr = StringUtil.arrAdd( flowStrArr , new String[]{ "" , "" , "" , "" } );
                			System.out.println( "unten Luft (Leerzeilen)" );
                		}
                		else if ( i4 == 3 )
                		{
                			// unten Luft (Leerzeilen) alle Zeilen auf maximale Lnge
                			flowStrArr = StringUtil.arrAdd( 
                					flowStrArr , 
                					StringUtil.fillStrArr( 4 , StringUtil.spaces( StringUtil.getArrMaxStrLen( flowStrArr ) + 4 ) ) );
                			System.out.println( "unten Luft (Leerzeilen) alle Zeilen auf maximale Lnge" );
                		}
						flowNet = innerTestParse(
                    			pDiagramFileName,
                    			pCheckFileName, 
                    			pNoException, 
                    			pMinTextCountPerConnection, 
                    			pMaxTextCountPerConnection, 
                    			pCheckConnectionUniqueness, 
                    			pCheckTransitionUniqueness, 
                    			// Fehlertexte nur bei 0-Durchlauf (kein Trimmen oder Spacen), weil sonst die Postionen nicht stimmen und es imer zu Fehlermeldungen kommt
                    			isUnformatted,
                    			flowStrArr);
                	}
            	}
        	}
		}
    	return flowNet;
    }

	/**
	 * @param pCheckFileName
	 * @param pNoException
	 * @param pMinTextCountPerConnection
	 * @param pMaxTextCountPerConnection
	 * @param pCheckConnectionUniqueness
	 * @param pCheckTransitionUniqueness
	 * @param flowStrArr
	 * @param flowNet
	 * @throws FlowNetParseException
	 */
	private static FlowNet innerTestParse(
	        final String pDiagramFileName, 
			final String pCheckFileName, 
			final boolean pNoException, 
			final int pMinTextCountPerConnection, 
			final int pMaxTextCountPerConnection, 
			final boolean pCheckConnectionUniqueness, 
			final boolean pCheckTransitionUniqueness, 
			final boolean pCheckErrorMessage, 
			final String[] flowStrArr )
	throws FlowNetParseException 
	{
		FlowNet flowNet = null;
		try 
		{
		    // Ausgabe Source des Diagrammes
		    for (int i = 0; i < flowStrArr.length; i++)
		    {
		        System.out.println( flowStrArr[ i ] );
		    }
		    System.out.println();
		    System.out.println();
		    
		    // das eigentliche Parsen
		    flowNet = FlowNet.parse( flowStrArr );
		    
		    System.out.println( flowNet.getConnectionStrListStr() );

		    System.out.println();

//                            System.out.println( flowNet );

		    System.out.println();
		    
		    /**
		     * Prfungen (Checks)
		     */

		    flowNet.checkNodeIdentifierUniqueness();

		    flowNet.checkForUnparsedCharacters();
		    
		    flowNet.checkForUnconnectedNodes();

		    flowNet.checkMinMaxTextCountPerConnection( //--
		    		pMinTextCountPerConnection , //--
		    		pMaxTextCountPerConnection );
		    
		    /**
		     * Prfen, ob die range min max texts per connection nicht zu gross war, damit die Test's nicht zu lasch sind
		     */ 
		    final int minTextCountPerConnection = flowNet.getMinTextCountPerConnection();
//                            System.out.println( "minTextCountPerConnection: " + minTextCountPerConnection );
			if ( pMinTextCountPerConnection < minTextCountPerConnection )
				// Test war zu lasch
		    {
		    	throw new RuntimeException( "pMinTextCountPerConnection: " + pMinTextCountPerConnection + " lesser tan flowNet.getMinTextCountPerConnection() " + minTextCountPerConnection );
		    }
		    final int maxTextCountPerConnection = flowNet.getMaxTextCountPerConnection();
//                            System.out.println( "maxTextCountPerConnection: " + maxTextCountPerConnection );
			if ( pMaxTextCountPerConnection > maxTextCountPerConnection )
				// Test war zu lasch
		    {
		    	throw new RuntimeException( "pMaxTextCountPerConnection: " + pMaxTextCountPerConnection + " greater tan flowNet.getMaxTextCountPerConnection() " + maxTextCountPerConnection );
		    }
		    

		    if ( pCheckConnectionUniqueness )
		    {
		        flowNet.checkConnectionUniqueness();
		    }
		    else
		    {
		    	/**
		    	 * Test auf non uniqueness, um abzusichern, dass die Bedingungen fr den Test nicht zu lasch waren
		    	 */
		    	boolean toEasy = true;
		    	try 
		    	{
		            flowNet.checkConnectionUniqueness();
				} 
		    	catch (Exception e) 
				{
					toEasy = false;
				}
		    	if ( toEasy )
		    		// Test war zu lasch
		    	{
		        	throw new RuntimeException( "Test failed, Test to easy" );
		    	}
		    }

		    if ( pCheckTransitionUniqueness )
		    {
		        flowNet.checkTransitionUniqueness( "default" );
		    }
		    else
		    {
		    	/**
		    	 * Test auf non uniqueness, um abzusichern, dass die Bedingungen fr den Test nicht zu lasch waren
		    	 */
		    	boolean toEasy = true;
		    	try 
		    	{
		            flowNet.checkTransitionUniqueness( "default" );
				} 
		    	catch (Exception e) 
				{
					toEasy = false;
				}
		    	if ( toEasy )
		    		// Test war zu lasch
		    	{
		        	throw new RuntimeException( "Test failed, Test to easy" );
		    	}
		    }

		    if ( ! flowNet.checkDiagram( StringUtil.fileToStrArr( DATA_FILE_PATH + File.separator + pCheckFileName ) ) )
		    {
		    	throw new RuntimeException( "Test failed" );
		    }
		}
		//catch (Throwable t) 
		catch (FlowNetParseException t) 
		{
			if ( pNoException )
			{
//                        		System.err.println( t.getMessage() );
//                        		t.printStackTrace();
				throw t;
			}
			
			// Exception war gewollt
//                    		System.err.println( t.getMessage() );
//                    		t.printStackTrace();
			System.out.println( t.getMessage() );
			
			// Prfen, ob die Fehlermeldung auch richtig war
			final String expectedErrStr = StringUtil.file2str(DATA_FILE_PATH + File.separator + pCheckFileName ).trim();
			
			if ( pCheckErrorMessage //--
					&& ! expectedErrStr.equals( t.getMessage().trim() ) )
			{
		    	throw new RuntimeException( "Test failed, expected error message was: '" + expectedErrStr + "'" );
			}

		    return flowNet;
		}
		if ( ! pNoException )
		{
			// Exception ist nicht aufgetreten, sollte aber auftreten
			throw new RuntimeException( "Test failed, expected exception not raised" );
		}
		return flowNet;
	}

    
    /**
     * Erzeugen einer XML-Struktur fr
     * die Java-Server-Faces-Navigation.
     * 
	 * @param pFlowNet in navigation-xml umzuwandelndes Fluss-Netz
	 * 
	 * TODO auslagern in eigene Klasse, nachziehen in DOku
	 */
	private static void createJsfNavigationXml(
			final FlowNet pFlowNet) 
	{
		// Anlegen des XmlDocument mit Root-XML-Element <faces-config>
        final Element rootElement = new Element( "faces-config" );
		final XmlDocument xmlDocument = new XmlDocument( rootElement );

		// Value-Objekt-Array mit allen Knoten und den jeweils abgehenden Verbindungen
		final FlowNodeAndConnectionsValueObj[] flowNodeAndConnectionsValObjArr = pFlowNet.getFlowNodeValueObjArr();
		
		// Schleife ber alle Knoten
		for (int i = 0; i < flowNodeAndConnectionsValObjArr.length; i++) 
		{
			// Abarbeiten eines Knoten
			final FlowNodeAndConnectionsValueObj flowNodeAndConnectionsValueObj = flowNodeAndConnectionsValObjArr[ i ];
			
			// Array mit Verbindungs-Texten und den dazugehrenden End-Knoten der vom aktuell bearbeiteten Knoten abgehenden Verbindungen
	        final FlowConnectionTextArrAndEndNodeValueObj[] flowConnectionTextArrAndEndNodeValueObjArr = flowNodeAndConnectionsValueObj.connectionValueObjArr;

	        // Prfen auf abgehende Verbindungen vom aktuell bearbeiteten Knoten
			if ( flowConnectionTextArrAndEndNodeValueObjArr != null && //--
					flowConnectionTextArrAndEndNodeValueObjArr.length > 0 )
				// es gibt abgehende Connections
			{
				// Hinzufgen eines XML-Elementes <navigation-rule> zum Root-XML-Element <faces-config>
				final Element navigationRuleElement = new Element( "navigation-rule" );
		        rootElement.addElement( navigationRuleElement );

		        //Abfragen des Textes des aktuell bearbeiteten Start-Knotens
				final String startNodeTextTrimmed = flowNodeAndConnectionsValueObj.startNode.getNodeTextTrimmed();
				
				// Hinzufgen eines XML-Elementes <from-view-id> zum Parent-XML-Element <navigation-rule> mit dem Text des Start-Knotens
		        final Element fromViewIdElement = new Element( "from-view-id" , startNodeTextTrimmed );
		        navigationRuleElement.addElement( fromViewIdElement );
		        
		        // Schleife ber alle vom aktuell bearbeiteten Knoten abgehenden Verbindungen
		        for (int j = 0; j < flowConnectionTextArrAndEndNodeValueObjArr.length; j++) 
		        {
		        	// Abarbeiten einer Verbindung
					final FlowConnectionTextArrAndEndNodeValueObj flowConnectionTextArrAndEndNodeValueObj = flowConnectionTextArrAndEndNodeValueObjArr[ j ];
					
					// Hinzufgen eines XML-Elementes <navigation-case> zum Parent-XML-Element <navigation-rule>
			        final Element navigationCaseElement = new Element( "navigation-case" );
			        navigationRuleElement.addElement( navigationCaseElement );
					
			        // Ermitteln des Verbindungs-Text oder Benutzung default-Verbindungs-Text
			        final String connectionText;
			        if ( flowConnectionTextArrAndEndNodeValueObj.flowConnectionTextArr != null //--
			        		&& flowConnectionTextArrAndEndNodeValueObj.flowConnectionTextArr.length == 1 )
			        	// es gibt genau einen Verbindungs-Text
			        {
			        	// Benutze den Verbindungs-Text
						connectionText = flowConnectionTextArrAndEndNodeValueObj.flowConnectionTextArr[ 0 ].getTextTrimmed();
			        }
			        else if ( flowConnectionTextArrAndEndNodeValueObj.flowConnectionTextArr == null //--
			        		|| flowConnectionTextArrAndEndNodeValueObj.flowConnectionTextArr.length == 0 )
			        	// es gibt keinen Verbindungs-Text
			        {
			        	// Benutze einen default-Verbindungs-Text
						connectionText = "default";
			        }
			        else
			        	// es gibt mehrere Verbindungs-Texte
			        {
			        	// mehrere Verbindungs-Texte sind nicht erlaubt
			        	throw new RuntimeException( "to many connection text's at " + flowConnectionTextArrAndEndNodeValueObj );
			        }
			        
					// Hinzufgen eines XML-Elementes <from-outcome> zum Parent-XML-Element <navigation-rule> mit dem Verbindungs-Text
					final Element fromOutcomeElement = new Element( "from-outcome" , connectionText );
			        navigationCaseElement.addElement( fromOutcomeElement );

			        //Abfragen des Textes des End-Knotens der aktuell bearbeiteten Verbindung
			        final String endNodeTextTrimmed = flowConnectionTextArrAndEndNodeValueObj.endNode.getNodeTextTrimmed();
			        
					// Hinzufgen eines XML-Elementes <to-view-id> zum Parent-XML-Element <navigation-rule> mit dem Text des End-Knotens der aktuell bearbeiteten Verbindung
					final Element toViewIdElement = new Element( "to-view-id" , endNodeTextTrimmed );
			        navigationCaseElement.addElement( toViewIdElement );
			        
			        // TODO redirect
		        }
			}
		}

		
        //Jedes Element in einer eigenen Zeile anzeigen
        xmlDocument.elementNewline = true;

        //Elemente, die nur eine Textzeile enthalten, nicht in einer eigenen Zeile anzeigen
        xmlDocument.singleTextElementNewline = false;

        //Elemente einrcken mit zwei Leerzeichen
        xmlDocument.elementIndent = "  ";

        // XML Ausgeben
        System.out.println(xmlDocument.build() + "\n\n");
	}

	/**
	 * Erzeugung einer State-Transition-Engine
	 * aus einem geparsten ASCII-Art-Fluss-Diagramm.
	 * 
	 * @param pFlowNet in State-Transition-Engine umzuwandelndes Fluss-Netz
	 * 
	 * TODO auslagern in eigene Klasse, nachziehen in DOku
	 */
	private static StateTransitionEngine createStateTransitionEngine(
			final FlowNet pFlowNet) 
	{
		// Erzeugen einer StateTransitionEngine
		final StateTransitionEngine stateTransitionEngine = new StateTransitionEngine();
		
		// Value-Objekt-Array mit allen Knoten und den jeweils abgehenden Verbindungen
		final FlowNodeAndConnectionsValueObj[] flowNodeAndConnectionsValObjArr = pFlowNet.getFlowNodeValueObjArr();
		
		// Schleife ber alle Knoten
		for (int i = 0; i < flowNodeAndConnectionsValObjArr.length; i++) 
		{
			// Abarbeiten eines Knoten
			final FlowNodeAndConnectionsValueObj flowNodeAndConnectionsValueObj = flowNodeAndConnectionsValObjArr[ i ];
			
			// Array mit Verbindungs-Texten und den dazugehrenden End-Knoten der vom aktuell bearbeiteten Knoten abgehenden Verbindungen
	        final FlowConnectionTextArrAndEndNodeValueObj[] flowConnectionTextArrAndEndNodeValueObjArr = flowNodeAndConnectionsValueObj.connectionValueObjArr;

	        // Prfen auf abgehende Verbindungen vom aktuell bearbeiteten Knoten
			if ( flowConnectionTextArrAndEndNodeValueObjArr != null && //--
					flowConnectionTextArrAndEndNodeValueObjArr.length > 0 )
				// es gibt abgehende Connections
			{
		        //Abfragen des Textes des aktuell bearbeiteten Start-Knotens
				final String startNodeTextTrimmed = flowNodeAndConnectionsValueObj.startNode.getNodeTextTrimmed();
				
				// Hinzufgen eines Source-State zur StateTransitionEngine mit dem Text des Start-Knotens
		        final SourceState sourceState = new SourceState( startNodeTextTrimmed );
		        stateTransitionEngine.add( sourceState );

		        // Schleife ber alle vom aktuell bearbeiteten Knoten abgehenden Verbindungen
		        for (int j = 0; j < flowConnectionTextArrAndEndNodeValueObjArr.length; j++) 
		        {
		        	// Abarbeiten einer Verbindung
					final FlowConnectionTextArrAndEndNodeValueObj flowConnectionTextArrAndEndNodeValueObj = flowConnectionTextArrAndEndNodeValueObjArr[ j ];
					
			        // Ermitteln des Verbindungs-Text oder Benutzung default-Verbindungs-Text
			        final String connectionText;
			        if ( flowConnectionTextArrAndEndNodeValueObj.flowConnectionTextArr != null //--
			        		&& flowConnectionTextArrAndEndNodeValueObj.flowConnectionTextArr.length == 1 )
			        	// es gibt genau einen Verbindungs-Text
			        {
			        	// Benutze den Verbindungs-Text
						connectionText = flowConnectionTextArrAndEndNodeValueObj.flowConnectionTextArr[ 0 ].getTextTrimmed();
			        }
			        else if ( flowConnectionTextArrAndEndNodeValueObj.flowConnectionTextArr == null //--
			        		|| flowConnectionTextArrAndEndNodeValueObj.flowConnectionTextArr.length == 0 )
			        	// es gibt keinen Verbindungs-Text
			        {
			        	// Benutze einen default-Verbindungs-Text
						connectionText = "default";
			        }
			        else
			        	// es gibt mehrere Verbindungs-Texte
			        {
			        	// mehrere Verbindungs-Texte sind nicht erlaubt
			        	throw new RuntimeException( "to many connection text's at " + flowConnectionTextArrAndEndNodeValueObj );
			        }
			        
					// Hinzufgen eines Zustands-bergangs-Ereignisses mit dem Verbindungs-Text zum Start-Zustand
			        StateTransitEvent stateTransitEvent = new StateTransitEvent( connectionText );
			        sourceState.add( stateTransitEvent );

							
			        //Abfragen des Textes des End-Knotens der aktuell bearbeiteten Verbindung
			        final String endNodeTextTrimmed = flowConnectionTextArrAndEndNodeValueObj.endNode.getNodeTextTrimmed();
			        
					// Hinzufgen eines ziel-Zustandes mit dem Text des End-Knotens der aktuell bearbeiteten Verbindung
			        DestinationState destinationState = 
			        	new DestinationState( 
			        		endNodeTextTrimmed , 
			        		""); // TODO Bemerkung
			        stateTransitEvent.add( destinationState );
		        }
			}
		}
		
		// Validierung der StateTransitionEngine
		if ( stateTransitionEngine.check() )
		{
			System.out.println("state map is valid");
		}
		else
		{
			throw new StateTransitionEngineIsInvalidException( "state map is invalid" );
		}
		
		System.out.println( stateTransitionEngine );

		return stateTransitionEngine;
	}

}
