package de.heinerkuecker.asciiartflowparser;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Stack;

import de.heinerkuecker.util.StringUtil;

/**
 * Diese Klasse reprsentiert eine Verbindung
 * (Connection) eines aus einem ASCII-Art-Diagramm
 * eingeparsten Fluss-Diagrammes (FlowNet).
 * Im Flussdiagramm entspricht eine Verbindung
 * einem Zustands-bergang.
 *
 * @author Heiner Kcker
 */
public final class FlowConnection
{
    private FlowNode startFlowNode;

    private FlowNode endFlowNode;

    private final ArrayList partList = new ArrayList();

    /**
     * ob diese Verbindung aus einem Split entstand
     * oder alternativ aus einem Knoten kam
     */
    private final boolean isFromSplit;

    /**
     * Richtungs-Konstante hoch
     */
    public static final int UP = 0;

    /**
     * Richtungs-Konstante runter
     */
    public static final int DOWN = 1;

    /**
     * Richtungs-Konstante links
     */
    public static final int LEFT = 2;

    /**
     * Richtungs-Konstante rechts
     */
    public static final int RIGHT = 3;

    /**
     * Constructor.
     */
    protected FlowConnection(
            final boolean pIsFromSplit)
    {
        super();
        this.isFromSplit = pIsFromSplit;
    }

//  /**
//   * @return the endFlowNode
//   */
//  /*public*/ FlowNode getEndFlowNode()
//    {
//      return this.endFlowNode;
//  }

    /**
     * Zurckgeben der End-FlowNode's mit
     * den auf der Verbindung liegenden
     * Texten.
     *
     * Bei gesplitteten Verbindungen
     * kann es mehrere End-FlowNode
     * geben.
     *
     * Aufruf aus {@link #toConnectionStrArr()},
     * in diesem Fall sollte keine Exception auftreten,
     * weil unmittelbar nach dem Parsen die
     * Struktur aufgelst wird.
     *
     * @return endFlowNode's
     */
    ///*public*/ FlowNode[] getEndFlowNodeArrCatched()
    /*public*/ FlowConnectionTextArrAndEndNodeValueObj[] getFlowConnectionTextArrAndEndNodeValueObjArrCatched()
    {
        try
        {
            //return getEndFlowNodeArr( //--
            final FlowConnectionTextArrAndEndNodeValueObj[] flowConnectionTextArrAndEndNodeValueObjArr = //--
                getFlowConnectionTextArrAndEndNodeValueObjArr( //--
                                new Stack() );

            /**
             * Korrektur aller Start-Knoten wegen Programm-Fehler
             */
            if ( flowConnectionTextArrAndEndNodeValueObjArr != null //--
                    && flowConnectionTextArrAndEndNodeValueObjArr.length > 0 )
            {
                for (int i = 0; i < flowConnectionTextArrAndEndNodeValueObjArr.length; i++)
                {
                    final FlowConnectionTextArrAndEndNodeValueObj valObj = flowConnectionTextArrAndEndNodeValueObjArr[i];
                    valObj.startNode = this.startFlowNode;
                }
            }

            return flowConnectionTextArrAndEndNodeValueObjArr;
            // TODO wie wre es damit, das Array zu cachen ?
            // TODO return clone als Schutz gegen Seiteneffekte
        }
        catch (Throwable t)
        {
            System.err.println( t.getMessage() );
            t.printStackTrace();
            throw new RuntimeException( t );
        }
    }
    /**
     * Zurckgeben der End-FlowNode's mit
     * den auf der Verbindung liegenden
     * Texten.
     *
     * Bei gesplitteten Verbindungen
     * kann es mehrere End-FlowNode
     * geben.
     *
     * @return endFlowNode's
     * @throws FlowNetParseException
     */
    ///*public*/ FlowNode[] getEndFlowNodeArr(
    /*public*/ FlowConnectionTextArrAndEndNodeValueObj[] getFlowConnectionTextArrAndEndNodeValueObjArr(
            final Stack pFlowConnectionStack)
        throws FlowNetParseException
    {
//System.out.println( "FlowConnection#getFlowConnectionTextArrAndEndNodeValueObjArr" );
        return innerGetFlowConnectionTextArrAndEndNodeValueObjArr( //--
                0, //--
                pFlowConnectionStack,
                new FlowConnectionText[ 0 ] );
    }

    /**
     * Zurckgeben der End-FlowNode's mit
     * den auf der Verbindung liegenden
     * Texten.
     *
     * Bei gesplitteten Verbindungen
     * kann es mehrere End-FlowNode
     * geben.
     *
     * @param pFromPartIndex Index des Parts, an dem die Bestimmung erfolgen soll
     *          (zur korrekten Zuordnung bei Joins)
     *
     * @param pFlowConnectionStack Stack zur Absicherung gegen zirkulre Referenzen (StackOverflowError)
     *
     * @param pFlowConnectionTextArr vorherige FlowConnectionText-Objekte, die beim Splitten allen Split-Zweigen mitgegeben werden.
     *
     * @return endFlowNode-Array
     * @throws FlowNetParseException
     */
    //private FlowNode[] innerGetEndFlowNodeArr(
    /*public*/ FlowConnectionTextArrAndEndNodeValueObj[] innerGetFlowConnectionTextArrAndEndNodeValueObjArr(
            final int pFromPartIndex,
            final Stack pFlowConnectionStack,
            final FlowConnectionText[] pFlowConnectionTextArr)
        throws FlowNetParseException
    {
//System.out.println( "FlowConnection#innerGetFlowConnectionTextArrAndEndNodeValueObjArr: " + this.startFlowNode.getNodeTextTrimed() + " " + this.endFlowNode.getNodeTextTrimed() );
//System.out.println( "FlowConnection#innerGetEndFlowNodeArr pFromPartIndex: " + pFromPartIndex );
//( new Exception() ).printStackTrace( System.out );
//if ( ( new Exception() ).getStackTrace().length > 30 ) System.exit( -1 );

//      /**
//       * Absicherung gegen zirkulre Referenzen (StackOverflowError)
//       */
//      pFlowConnectionStack.push( this );

        FlowConnectionText[] flowConnectionTextArr = pFlowConnectionTextArr;

        final ArrayList textAndEndNodeList = new ArrayList();
        for (int i = pFromPartIndex; i < this.partList.size(); i++)
        {
            final FlowConnectionPart part = (FlowConnectionPart) this.partList.get( i );

            if ( part instanceof FlowConnectionText )
            {
                flowConnectionTextArr = add(
                        flowConnectionTextArr, //--
                        (FlowConnectionText) part );
            }
            else if ( part instanceof FlowConnectionSplit )
            {
                final FlowConnectionSplit flowConnectionSplit = (FlowConnectionSplit) part;

                final FlowConnection splitOutFlowConnection = flowConnectionSplit.getSplitOutFlowConnection();

                /**
                 * Abfangen zirkulrer Referenz auf sich selbst.
                 * TODO: eigentlich ist diese Massnahme nur beim Jon erforderlich.
                 */
                if ( pFlowConnectionStack.contains( splitOutFlowConnection ) )
                {
                    throw new FlowNetParseException( "circular connection at" //
                            + " row " + ( flowConnectionSplit.getNextPosRow() + 1 ) //--
                            + " col " + ( flowConnectionSplit.getNextPosCol() + 1 ) );
                }

//              /**
//               * Absicherung gegen zirkulre Referenzen (StackOverflowError)
//               */
//              pFlowConnectionStack.push( this );

                //final FlowNode[] splitEndFlowNodeArr = //-
                final FlowConnectionTextArrAndEndNodeValueObj[] splitFlowConnectionTextArrAndEndNodeValueObjArr = //-
                    //splitOutFlowConnection.getEndFlowNodeArr( //--
                    //splitOutFlowConnection.getFlowConnectionTextArrAndEndNodeValueObjArr( //--
                    //      pFlowConnectionStack );
                    splitOutFlowConnection.innerGetFlowConnectionTextArrAndEndNodeValueObjArr( //--
                            0, //--
                            pFlowConnectionStack,
                            flowConnectionTextArr );

                if ( splitFlowConnectionTextArrAndEndNodeValueObjArr != null )
                {
                    textAndEndNodeList.addAll( Arrays.asList( splitFlowConnectionTextArrAndEndNodeValueObjArr ) );
                }
            }
            else if ( part instanceof FlowConnectionJoin )
            {
                final FlowConnectionJoin flowConnectionJoin = (FlowConnectionJoin) part;
                final FlowConnection joinInFlowConnection = flowConnectionJoin.getJoinInFlowConnection();

                /**
                 * Abfangen zirkulrer Referenz auf sich selbst.
                 *
                 * Nur beim Join, damit ein Wiedereintauchen
                 * nach dem Split in gleicher Richtung erlaubt ist:
                 *    +----------------+
                 *    |                |
                 *    |                V
                 * ---+---------------------->
                 */
                if ( pFlowConnectionStack.contains( joinInFlowConnection ) )
                {
                    throw new FlowNetParseException( "circular connection at" //
                            + " row " + ( flowConnectionJoin.getRow() + 1 ) //--
                            + " col " + ( flowConnectionJoin.getCol() + 1 ) );
                }

                /**
                 * Absicherung gegen zirkulre Referenzen (StackOverflowError)
                 */
                pFlowConnectionStack.push( this );

                /**
                 * Zurckgeben der End-FlowNode's,
                 * ab dem bergebenen Part,
                 * fr Joins, zur korrekten Zuordnung
                 * der End-Knoten.
                 */
                //final FlowNode[] joinEndFlowNodeArr = //--
                final FlowConnectionTextArrAndEndNodeValueObj[] joinEndFlowNodeArr = //--
                    //joinInFlowConnection.getEndFlowNodeArrFromPart( //--
                    joinInFlowConnection.getFlowConnectionTextArrAndEndNodeValueObjArrFromPart( //--
                            flowConnectionJoin.getJoinInFlowConnectionPart(), //--
                            pFlowConnectionStack, //--
                            flowConnectionTextArr );

                pFlowConnectionStack.pop();

                if ( joinEndFlowNodeArr != null )
                {
//System.out.println( "joinEndFlowNodeArr: " + Arrays.asList( joinEndFlowNodeArr ) );
                    textAndEndNodeList.addAll( Arrays.asList( joinEndFlowNodeArr ) );
                }
            }
        }

        //pFlowConnectionStack.pop();

        if ( this.endFlowNode != null )
        {
            final FlowConnectionTextArrAndEndNodeValueObj valObj = new FlowConnectionTextArrAndEndNodeValueObj();
            // wird zentral in anderer Methode gemacht: valObj.startNode = this.startFlowNode;
            valObj.endNode = this.endFlowNode;
            valObj.flowConnectionTextArr = flowConnectionTextArr; //pFlowConnectionTextArr;
            //System.out.println( "valObj: " + valObj );

            textAndEndNodeList.add( valObj );
        }

        //return (FlowNode[]) textAndEndNodeList.toArray( new FlowNode[ textAndEndNodeList.size() ] );
        return (FlowConnectionTextArrAndEndNodeValueObj[]) textAndEndNodeList.toArray( new FlowConnectionTextArrAndEndNodeValueObj[ textAndEndNodeList.size() ] );
    }

    /**
     * Zurckgeben der End-FlowNode's,
     * ab dem bergebenen Part,
     * fr Joins, zur korrekten Zuordnung
     * der End-Knoten.
     *
     * Bei gesplitteten Verbindungen
     * kann es mehrere End-FlowNode
     * geben.
     *
     * @param pFlowConnectionStack Stack zur Absicherung gegen zirkulre Referenzen (StackOverflowError)
     * @return endFlowNode's
     * @throws FlowNetParseException wird geworfen, wenn eine zirkulre Referenz auftaucht
     */
    /*public*/ FlowConnectionTextArrAndEndNodeValueObj[] getFlowConnectionTextArrAndEndNodeValueObjArrFromPart(
            final FlowConnectionPart pJoinFlowConnectionPart,
            final Stack pFlowConnectionStack,
            final FlowConnectionText[] pFlowConnectionTextArr )
        throws FlowNetParseException
    {
//System.out.println( "FlowConnection#getEndFlowNodeArrFromPart pJoinFlowConnectionPart: " + pJoinFlowConnectionPart );
//System.out.println( "pJoinFlowConnectionPart.getFlowConnection(): " + pJoinFlowConnectionPart.getFlowConnection() );
//System.out.println( "this: " + this );

        if ( pJoinFlowConnectionPart.getFlowConnection() != this )
        {
            throw new RuntimeException(//--
                    "\n pJoinFlowConnectionPart.getFlowConnection(): " + pJoinFlowConnectionPart.getFlowConnection() //--
                    + "\n this: " + this );
        }
        /**
         * Suche nach dem Anfangs-Part, ab dem
         * die End-Knoten ermitelt werden sollen.
         */
        int partIndex = 0;
        while ( partIndex < this.partList.size() //--
                // unmittelbarer Objekt-Vergleich
                && pJoinFlowConnectionPart != this.partList.get( partIndex ) )
        {
//System.out.println( "partIndex in while: " + partIndex );
//System.out.println( "part at partIndex: " + this.partList.get( partIndex ) );
            partIndex++;
        }
//System.out.println( "partIndex after while: " + partIndex );
//System.out.println( "part at partIndex: " + this.partList.get( partIndex ) );

        //return innerGetEndFlowNodeArr( //--
        return innerGetFlowConnectionTextArrAndEndNodeValueObjArr( //--
                partIndex , //--
                pFlowConnectionStack, //--
                pFlowConnectionTextArr );
   }

    /**
     * @param pEndFlowNode the endFlowNode to set
     */
    void setEndFlowNode(FlowNode pEndFlowNode)
    {
        if ( this.endFlowNode != null )
        {
            // Verhindern des doppelten Zuordnens, was nur durch einen Programmfehler geschehen kann
            throw new RuntimeException( "ambigous end node: " + this );
        }
        this.endFlowNode = pEndFlowNode;

        // Vermerken dieser Connection an der incomingConnectionList des End-Knotens
        pEndFlowNode.inComingConnectionListAdd( this );
    }

    /**
     * @return the startFlowNode
     */
    /*public*/ FlowNode getStartFlowNode()
    {
        return this.startFlowNode;
    }

    /**
     * @param pStartFlowNode the startFlowNode to set
     */
    void setStartFlowNode(FlowNode pStartFlowNode)
    {
        if ( this.startFlowNode != null )
        {
            // Verhindern des doppelten Zuordnens, was nur durch einen Programmfehler geschehen kann
            throw new RuntimeException( "ambigous start node: " + this );
        }
        this.startFlowNode = pStartFlowNode;

        // Vermerken dieser Connection an der outgoingConnectionList des Start-Knotens
        pStartFlowNode.outGoingConnectionListAdd( this );
    }

    /**
     *
     * @param pFlowConnectionPart
     */
    void addFlowConnectionPart(
            final FlowConnectionPart pFlowConnectionPart)
    {
        this.partList.add( pFlowConnectionPart );
    }

    /**
     * @return
     */
    /*public*/ boolean isFromSplit()
    {
        return this.isFromSplit;
    }

    /**
     * Darstellung als Verbindung:
     *
     * <pre>
     * A -> B
     * B -> C
     * </pre>
     *
     * @return
     */
//  /*public*/ String toConnectionStr()
//  {
//      return //--
//          StringUtil.trim( this.startFlowNode.getNodeText() ) //--
//          + " -> " //--
//          + StringUtil.trim( this.endFlowNode.getNodeText() );
//  }

    /**
     * Darstellung als Verbindung:
     *
     * <pre>
     * A -> B
     * B -> C
     * </pre>
     *
     * @return
     */
    /*public*/ String[] toConnectionStrArr()
    {
//System.out.println( "FlowNet#toConnectionStrArr this: " + this );
        final FlowConnectionTextArrAndEndNodeValueObj[] textAndEndFlowNodeArr = this.getFlowConnectionTextArrAndEndNodeValueObjArrCatched();
//System.out.println( "endFlowNodeArr: " + Arrays.asList( endFlowNodeArr ) );
        if ( textAndEndFlowNodeArr == null )
        {
            return null;
        }
        final String[] connectionStrArr = new String[ textAndEndFlowNodeArr.length ];

        for (int i = 0; i < textAndEndFlowNodeArr.length; i++)
        {
            final FlowConnectionTextArrAndEndNodeValueObj runTextAndEndFlowNode = textAndEndFlowNodeArr[i];

            connectionStrArr[ i ] = //--
                // TODO FlowNode.getNodeTextNormalized()
                //StringUtil.trim( this.startFlowNode.getNodeText() ) ;
                this.startFlowNode.getNodeTextTrimmed();

            final FlowConnectionText[] flowConnectionTextArr = runTextAndEndFlowNode.flowConnectionTextArr;

            if ( flowConnectionTextArr == null //--
                    || flowConnectionTextArr.length < 1 )
            {
                connectionStrArr[ i ] += " -> ";
            }
            else
            {
                connectionStrArr[ i ] += " -- ";
                for (int j = 0; j < flowConnectionTextArr.length; j++)
                {
                    final FlowConnectionText flowConnectionText = flowConnectionTextArr[j];
                    connectionStrArr[ i ] += flowConnectionText.getTextTrimmed();

                    if ( j < flowConnectionTextArr.length - 1 )
                    {
                        connectionStrArr[ i ] += " -- ";
                    }
                    else if ( j < flowConnectionTextArr.length )
                    {
                        connectionStrArr[ i ] += " --> ";
                    }
                }
            }
            connectionStrArr[ i ] += //--
                //StringUtil.trim( runTextAndEndFlowNode.endNode.getNodeText() );
                runTextAndEndFlowNode.endNode.getNodeTextTrimmed();
        }
        return connectionStrArr;
    }

    /**
     * Insert einen FlowConnectionSplit fr
     * einen dreifachen Linien-Split.
     *
     * @param pForkFlowConnectionSplit
     */
    /*public*/ void insertFlowConnectionPart(
            final FlowConnectionSplit pForkFlowConnectionSplit)
    {
        this.partList.add( 0, pForkFlowConnectionSplit );
    }

    /**
     * Suche eine passende FlowConnection fr
     * einen FlowConnectionJoin (keine End-Node).
     *
     * @param pRowNr
     * @param pColNr
     * @return
     */
//  /*public*/ FlowConnection getMatchingFlowConnection(
//          final int pRowNr,
//          final int pColNr)
//  {
//      for (int i = 0; i < this.partList.size(); i++)
//      {
//          final FlowConnectionPart part = (FlowConnectionPart) this.partList.get( i );
//          if ( part.matchPosition( pRowNr, pColNr ) )
//          {
//              // Treffer auf einem eigenen Connection-Part
//              return this;
//          }
//          if ( part instanceof FlowConnectionSplit )
//          {
//              final FlowConnectionSplit flowConnectionSplit = (FlowConnectionSplit) part;
//              final FlowConnection splitFlowConnection = flowConnectionSplit.getEndFlowConnection();
//              final FlowConnection matchingFlowConnection = splitFlowConnection.getMatchingFlowConnection( pRowNr, pColNr );
//              if ( matchingFlowConnection != null )
//              {
//                  return matchingFlowConnection;
//              }
//          }
//          if ( part instanceof FlowConnectionJoin )
//          {
//              final FlowConnectionJoin flowConnectionJoin = (FlowConnectionJoin) part;
//              final FlowConnection joinFlowConnection = flowConnectionJoin.getJoinInFlowConnection();
//              final FlowConnection matchingFlowConnection = joinFlowConnection.getMatchingFlowConnection( pRowNr, pColNr );
//              if ( matchingFlowConnection != null )
//              {
//                  return matchingFlowConnection;
//              }
//          }
//      }
//
//      return null;
//  }

    /**
     * Suche ein passendes FlowConnectionPart fr
     * einen FlowConnectionJoin (keine End-Node).
     *
     * @param pRowNr
     * @param pColNr
     * @return
     */
    /*public*/ FlowConnectionPart getMatchingFlowConnectionPart(
            final int pRowNr,
            final int pColNr)
    {
//System.out.println( "FlowConnection#getMatchingFlowConnectionPart row " + pRowNr + " col " + pColNr );
        for (int i = 0; i < this.partList.size(); i++)
        {
            final FlowConnectionPart part = (FlowConnectionPart) this.partList.get( i );
//System.out.println( "part: " + part );
            if ( part.matchPosition( pRowNr, pColNr ) )
            {
//System.out.println( "passt" );
                // Treffer auf einem eigenen Connection-Part
                return part;
            }
//          if ( part instanceof FlowConnectionSplit )
//          {
//              final FlowConnectionSplit flowConnectionSplit = (FlowConnectionSplit) part;
//              final FlowConnection splitFlowConnection = flowConnectionSplit.getSplitOutFlowConnection();
//              final FlowConnectionPart matchingFlowConnectionPart = splitFlowConnection.getMatchingFlowConnectionPart( pRowNr, pColNr );
//              if ( matchingFlowConnectionPart != null )
//              {
//                  return matchingFlowConnectionPart;
//              }
//          }
//          if ( part instanceof FlowConnectionJoin )
//          {
//              final FlowConnectionJoin flowConnectionJoin = (FlowConnectionJoin) part;
//              final FlowConnection joinFlowConnection = flowConnectionJoin.getJoinInFlowConnection();
//              final FlowConnectionPart matchingFlowConnectionPart = joinFlowConnection.getMatchingFlowConnectionPart( pRowNr, pColNr );
//              //final FlowConnectionPart matchingFlowConnectionPart = this.getMatchingFlowConnectionPart( pRowNr, pColNr );
//              if ( matchingFlowConnectionPart != null )
//              {
//                  return matchingFlowConnectionPart;
//              }
//          }
        }

        return null;
    }

    /**
     * debug output
     */
    public String toString()
    {
        return StringUtil.getShortClassName( this ) + "\n\t" //--
        + "startFlowNode: " + this.startFlowNode + "\n\t" //--
        + "endFlowNode: " + this.endFlowNode + "\n\t" //--
        + "partList: " + this.partList;
    }


    /**
     * static Methoden
     */

    /**
     * Parse alle von der bergebenen FlowNode
     * abgehenden FlowConnection's.
     *
     * @param pFlowNode
     * @return
     * @throws FlowNetParseException
     */
    /*public*/ static FlowConnection[] parseFlowConnectionArr(
            final String[] pFlowStrArr,
            final FlowNode pFlowNode ,
            final FlowNet pFlowNet ,
            final ArrayList pNoEndNodeArrowList,
            final ArrayList pFlowConnectionCrossingList )
        throws FlowNetParseException
    {
        final ArrayList flowConnectionList = new ArrayList();

        /**
         * rund um den Knoten nach abgehenden Connections suchen
         */

        /**
         * oben suchen
         */
        if ( pFlowNode.getRowLeftTop() > 1 )
        {
            final String lineStr = pFlowStrArr[ pFlowNode.getRowLeftTop() - 1 ];
            for ( int colNr = pFlowNode.getColLeftTop() + 1 ; colNr < pFlowNode.getColRightBottom() ; colNr++ )
            {
                if ( colNr < lineStr.length() //--
                        && lineStr.charAt( colNr ) == '|' )
                    // abgehendes '|' nach oben gefunden
                {
                    final FlowConnection flowConnection =
                        FlowConnection.parse(
                                pFlowStrArr,
                                pFlowNode.getRowLeftTop() - 1,
                                colNr ,
                                FlowConnection.UP,
                                pFlowNode,
                                pFlowNet,
                                pNoEndNodeArrowList,
                                pFlowConnectionCrossingList,
                                false );
                    if ( flowConnection != null )
                    {
//System.out.println( "flowConnection: " + flowConnection );
                        flowConnectionList.add( flowConnection );
                    }
                }
            }
        }

        /**
         * unten suchen
         */
        if ( pFlowNode.getRowRightBottom() < pFlowStrArr.length - 2 )
        {
            final String lineStr = pFlowStrArr[ pFlowNode.getRowRightBottom() + 1 ];
            for ( int colNr = pFlowNode.getColLeftTop() + 1 ; colNr < pFlowNode.getColRightBottom() ; colNr++ )
            {
                if ( colNr < lineStr.length() //--
                        && lineStr.charAt( colNr ) == '|' )
                    // abgehendes '|' nach unten gefunden
                {
                    final FlowConnection flowConnection =
                        FlowConnection.parse(
                                pFlowStrArr,
                                pFlowNode.getRowRightBottom() + 1,
                                colNr ,
                                FlowConnection.DOWN,
                                pFlowNode,
                                pFlowNet,
                                pNoEndNodeArrowList,
                                pFlowConnectionCrossingList,
                                false );
                    if ( flowConnection != null )
                    {
//System.out.println( "flowConnection: " + flowConnection );
                        flowConnectionList.add( flowConnection );
                    }
                }
            }
        }

        /**
         * links suchen
         */
        if ( pFlowNode.getColLeftTop() > 1 )
        {
            for ( int rowNr = pFlowNode.getRowLeftTop() + 1 ; rowNr < pFlowNode.getRowRightBottom() ; rowNr++ )
            {
                final int colLeftOuter = pFlowNode.getColLeftTop() - 1;
                if ( colLeftOuter < pFlowStrArr[ rowNr ].length() //--
                        && pFlowStrArr[ rowNr ].charAt( colLeftOuter ) == '-' )
                    // abgehendes '-' nach links gefunden
                {
                    final FlowConnection flowConnection =
                        FlowConnection.parse(
                                pFlowStrArr,
                                rowNr,
                                colLeftOuter ,
                                FlowConnection.LEFT,
                                pFlowNode ,
                                pFlowNet,
                                pNoEndNodeArrowList,
                                pFlowConnectionCrossingList,
                                false );
                    if ( flowConnection != null )
                    {
//System.out.println( "flowConnection: " + flowConnection );
                        flowConnectionList.add( flowConnection );
                    }
                }
            }
        }

        /**
         * rechts suchen
         */
        //rechts keine Bedingung, weil Flatter-Rand rechts mglich
        {
            final int colRightOuter = pFlowNode.getColRightBottom() + 1;
            for ( int rowNr = pFlowNode.getRowLeftTop() + 1 ; rowNr < pFlowNode.getRowRightBottom() ; rowNr++ )
            {
                if ( colRightOuter < pFlowStrArr[ rowNr ].length() - 1 //--
                        && pFlowStrArr[ rowNr ].charAt( colRightOuter ) == '-' )
                    // abgehendes '-' nach rechts gefunden
                {
                    final FlowConnection flowConnection =
                        FlowConnection.parse(
                                pFlowStrArr,
                                rowNr,
                                colRightOuter,
                                FlowConnection.RIGHT,
                                pFlowNode,
                                pFlowNet,
                                pNoEndNodeArrowList,
                                pFlowConnectionCrossingList,
                                false );
                    if ( flowConnection != null )
                    {
//System.out.println( "flowConnection: " + flowConnection );
                        flowConnectionList.add( flowConnection );
                    }
                }
            }
        }
        return ( FlowConnection[] ) flowConnectionList.toArray( new FlowConnection[ flowConnectionList.size() ] );
    }

    /**
     * Parsen mit Richtungsangabe.
     *
     * @param pFlowStrArr
     * @param pRowStart
     * @param pColStart
     * @return
     * @throws FlowNetParseException
     */
    /*public*/ static FlowConnection parse(
            final String[] pFlowStrArr,
            final int pRowStart,
            final int pColStart,
            final int pDirection,
            final FlowNode pStartFlowNode,
            final FlowNet pFlowNet ,
            final ArrayList pNoEndNodeArrowList,
            final ArrayList pFlowConnectionCrossingList,
            final boolean pIsFromSplit )
        throws FlowNetParseException
    {
//System.out.println( "+++" );
        boolean hasArrow = false;
        final FlowConnection flowConnection = new FlowConnection( pIsFromSplit );
        flowConnection.setStartFlowNode(pStartFlowNode);

        int rowNr = pRowStart;
        int colNr = pColStart;
        int direction = pDirection;

        int oldRowNr = -1;
        int oldColNr = -1;

        while ( true )
        {
            if ( rowNr == oldRowNr //--
                    && colNr == oldColNr )
            {
                throw new RuntimeException( "Programm frisst sich fest" );
            }
            oldRowNr = rowNr;
            oldColNr = colNr;
//System.out.println( "1 next rowNr: " + rowNr );
//System.out.println( "2 next colNr: " + colNr );

            /**
             * Prfen, ob die nchste Position in einem Knoten liegt
             */
            if ( pFlowNet.getMatchingFlowNode( rowNr , colNr ) != null )
            {
                throw new FlowNetParseException( "FlowConnection connect a FlowNode without Arrow char at row " + ( rowNr + 1 ) + " col " + ( colNr + 1 ) + " direction " + getDirectionStr( direction ) + " for line" );
            }

            /**
             * Parsen Linie mit Richtungsangabe
             */
            final FlowConnectionLinePart flowConnectionLinePart =
                FlowConnectionLinePart.parse(
                        pFlowStrArr,
                        rowNr,
                        colNr,
                        direction ,
                        flowConnection );

            if ( flowConnectionLinePart == null )
            {
                //System.err.println( "FlowConnection lost at row " + rowNr + " col " + colNr + " for line" );
                //return null;
                //throw new FlowNetParseException( "FlowConnection connect a FlowNode without Arrow char at row " + ( rowNr + 1 ) + " col " + ( colNr + 1 ) + " direction " + getDirectionStr( direction ) + " for line" );
                throw new FlowNetParseException( "FlowConnection ended without Arrow char at row " + ( rowNr + 1 ) + " col " + ( colNr + 1 ) + " direction " + getDirectionStr( direction ) + " for line" );
            }

            flowConnection.addFlowConnectionPart( flowConnectionLinePart );

            // Position weiterschalten
            rowNr = flowConnectionLinePart.getNextPosRow();
            colNr = flowConnectionLinePart.getNextPosCol();
//System.out.println( "1 next rowNr: " + rowNr );
//System.out.println( "2 next colNr: " + colNr );


            /**
             * Prfen, ob die nchste Position in einem Knoten liegt
             */
            if ( pFlowNet.getMatchingFlowNode( rowNr , colNr ) != null )
            {
                throw new FlowNetParseException( "FlowConnection connect a FlowNode without Arrow char at row " + ( rowNr + 1 ) + " col " + ( colNr + 1 ) + " direction " + getDirectionStr( direction ) + " for line" );
            }

            /**
             * Suche nach einem Folge-Part.
             * dabei muss nach dem am weitesten
             * vorausblickenen Part zuerst gesucht
             * werden.
             *
             * Erst nach Split, dann nach Bend (Kurve)
             * suchen.
             *
             * Analogie in Sprache:
             * Suche erst '==', dann '=', weil sonst
             * flschlicherweise ein '=' erkannt
             * werden knnte.
             */

            FlowConnectionSplit flowConnectionSplit =
                FlowConnectionSplit.parse(
                        pFlowStrArr,
                        rowNr,
                        colNr,
                        direction,
                        flowConnection, // pStartFlowNode,
                        pFlowNet,
                        pNoEndNodeArrowList,
                        pFlowConnectionCrossingList );

            if ( flowConnectionSplit != null )
                // Abzweig
            {
                flowConnection.addFlowConnectionPart( flowConnectionSplit );

                // Position weiterschalten
                rowNr = flowConnectionSplit.getNextPosRow();
                colNr = flowConnectionSplit.getNextPosCol();
                direction = flowConnectionSplit.getNextDirection();
//System.out.println( "3 next rowNr: " + rowNr );
//System.out.println( "4 next colNr: " + colNr );
                continue;
            }

            final FlowConnectionBend flowConnectionBend =
                FlowConnectionBend.parse(
                        pFlowStrArr,
                        rowNr,
                        colNr,
                        direction,
                        flowConnection );

            if ( flowConnectionBend != null )
                // Kurve
            {
                flowConnection.addFlowConnectionPart( flowConnectionBend );

                // Position weiterschalten
                rowNr = flowConnectionBend.getNextPosRow();
                colNr = flowConnectionBend.getNextPosCol();
                direction = flowConnectionBend.getNextDirection();
//System.out.println( "3 next rowNr: " + rowNr );
//System.out.println( "4 next colNr: " + colNr );
                continue;
            }

            final FlowConnectionCrossing flowConnectionCrossing =
                FlowConnectionCrossing.parse(
                        pFlowStrArr,
                        rowNr,
                        colNr,
                        direction ,
                        flowConnection,
                        pFlowConnectionCrossingList );

            if ( flowConnectionCrossing != null )
            {
                flowConnection.addFlowConnectionPart( flowConnectionCrossing );

                // Position weiterschalten
                rowNr = flowConnectionCrossing.getNextPosRow();
                colNr = flowConnectionCrossing.getNextPosCol();
//System.out.println( "3 next rowNr: " + rowNr );
//System.out.println( "4 next colNr: " + colNr );
                continue;
            }

            final FlowConnectionArrow flowConnectionArrow =
                FlowConnectionArrow.parse(
                        pFlowStrArr,
                        rowNr,
                        colNr,
                        direction,
                        flowConnection );

            if ( flowConnectionArrow != null )
            {
                hasArrow = true;
                flowConnection.addFlowConnectionPart( flowConnectionArrow );

                // Position weiterschalten
                rowNr = flowConnectionArrow.getNextPosRow();
                colNr = flowConnectionArrow.getNextPosCol();
//System.out.println( "3 next rowNr: " + rowNr );
//System.out.println( "4 next colNr: " + colNr );


                final FlowNode destinationFlowNode =
                    pFlowNet.getFlowNodeForPosition(
                            rowNr,
                            colNr );
                if ( destinationFlowNode != null )
                {
                    flowConnection.setEndFlowNode( destinationFlowNode );
                    break;
                }
                else
                {
                    //throw new FlowNetParseException( "has no end node: " + flowConnectionArrow );

                    /**
                     * Vermerken zum spteren Auflsen als eventuelle FlowConnectionJoin's
                     */
                    pNoEndNodeArrowList.add( flowConnectionArrow );
                    break;
                }
            }

            // ganz wichtig: Text erst nach Kreuzung parsen, weil in FlowNet#parse alle Kreuzungen noch mal darauf geprft werden, ob sie Texte sind
            final FlowConnectionText flowConnectionText =
                FlowConnectionText.parse(
                        pFlowStrArr,
                        rowNr,
                        colNr,
                        direction,
                        flowConnection );

            if ( flowConnectionText != null )
            {
                flowConnection.addFlowConnectionPart( flowConnectionText );

                // Position weiterschalten
                rowNr = flowConnectionText.getNextPosRow();
                colNr = flowConnectionText.getNextPosCol();
//System.out.println( "3 next rowNr: " + rowNr );
//System.out.println( "4 next colNr: " + colNr );
                continue;
            }


        }

        return flowConnection;
    }

    /**
     * Richtungs int-Wert in String umwandeln.
     *
     * TODO Enum(Pattern bei < Java 5)
     * @param pDirection Richtungs-Code
     * @return Richtungs-Name
     */
    public static String getDirectionStr(
            final int pDirection )
    {
        switch ( pDirection )
        {
            case UP:
                return "UP";
            case DOWN:
                return "DOWN";
            case LEFT:
                return "LEFT";
            case RIGHT:
                return "RIGHT";
            default:
                return "???";
        }
    }

    /**
     * Feststellen, ob die bergebene Richtung horizontal ist.
     *
     * TODO Enum(Pattern bei < Java 5)
     * @param pDirection Richtungs-Code
     * @return is horizontal
     */
    public static boolean isHorizontal(
            final int pDirection )
    {
        switch ( pDirection )
        {
            case UP:
                return false;
            case DOWN:
                return false;
            case LEFT:
                return true;
            case RIGHT:
                return true;
            default:
                throw new IllegalArgumentException( "unknown direction: " + pDirection );
        }
    }

    /**
     * Feststellen, ob die bergebene Richtung vertikal ist.
     *
     * TODO Enum(Pattern bei < Java 5)
     * @param pDirection Richtungs-Code
     * @return is vertikal
     */
    /*public*/ static boolean isVertical(
            final int pDirection )
    {
        return ! isHorizontal(pDirection);
    }

    /**
     * Array add fr FlowConnectionText.
     *
     * @param pFlowConnectionTextArr
     * @param pFlowConnectionText
     * @return
     */
    static FlowConnectionText[] add(
            final FlowConnectionText[] pFlowConnectionTextArr,
            final FlowConnectionText pFlowConnectionText)
    {
        if ( pFlowConnectionText == null )
        {
            throw new NullPointerException();
        }
        final FlowConnectionText[] retArr = new FlowConnectionText[ pFlowConnectionTextArr.length + 1 ];
        retArr[ retArr.length - 1 ] = pFlowConnectionText;
        System.arraycopy(pFlowConnectionTextArr, 0, retArr, 0, pFlowConnectionTextArr.length );
        return retArr;
    }

    /**
     * Fr eine eingeparste Kreuzung {@link FlowConnectionCrossing}
     * wurde keine zu kreuzende Linie {@link FlowConnectionLinePart}
     * gefunden.
     * Statt dessen wurde ein {@link FlowConnectionText}
     * eingeparst.
     * Mit dieser Methode wird die Kreuzung durch
     * den Text ersetzt.
     *
     * @param pFlowConnectionText
     */
    /*public*/ void replaceCrossingWithText(
            final FlowConnectionCrossing pFlowConnectionCrossing,
            final FlowConnectionText pFlowConnectionText)
    {
        final int pos = this.partList.indexOf( pFlowConnectionCrossing );
        // eine eventuelle Exception bei nicht gefundener Kreuzung einfach werfen
        this.partList.set( pos , pFlowConnectionText );
    }

}
