package de.heinerkuecker.asciiartflowparser;
import de.heinerkuecker.util.StringUtil;

/**
 * Diese Klasse reprsentiert einen Pfeil (Arrow)
 * am Ende einer Verbindung (Connection) eines
 * aus einem ASCII-Art-Diagramm eingeparsten
 * Fluss-Diagrammes (FlowNet).
 *
 * Ein solcher Pfeil muss immer in einen
 * Knoten (Node) oder in eine andere Verbindung
 * (FlowConnection) eintauchen.
 *
 * @author Heiner Kcker
 */
public final class FlowConnectionArrow
extends FlowConnectionPart
{
    private final int row;
    private final int col;
    private final char arrowChar;
    private final int direction;

    /**
     * Start-FlowConnection fr die
     * Auflsung von FlowConnectionJoin.
     */
    //private final FlowConnection startFlowConnection;

    /**
     * @return the row
     */
    /*public*/ int getRow()
    {
        return this.row;
    }

    /**
     * @return the col
     */
    /*public*/ int getCol()
    {
        return this.col;
    }

    /**
     * @return the direction
     */
    /*public*/ int getDirection()
    {
        return this.direction;
    }

    /**
     * @return the startFlowConnection
     */
//  /*public*/ FlowConnection getStartFlowConnection()
//  {
//      return this.startFlowConnection;
//  }

    /**
     * Constructor.
     *
     * @param pRow
     * @param pCol
     * @param pArrowChar
     */
    private FlowConnectionArrow(
            final int pRow,
            final int pCol,
            final char pArrowChar,
            final int pDirection,
            final FlowConnection pFlowConnection )
    {
        super( pFlowConnection );
        this.row = pRow;
        this.col = pCol;
        this.arrowChar = pArrowChar;
        this.direction = pDirection;
        //this.startFlowConnection = pFlowConnection;
    }

    /**
     * Zurckgeben der Zeile der
     * nchsten Position, wenn die Linie
     * weiter laufen wrde.
     *
     * @return
     */
    /*public*/ int getNextPosRow()
    {
        // TODO Umstellen auf direction member
        if ( this.arrowChar == 'v' //--
            || this.arrowChar == 'V' )
            // nach unten
        {
            // eine Zeile nach unten
            return this.row + 1;
        }
        else if ( this.arrowChar == '^' //--
            || this.arrowChar == 'A' )
            // nach oben
        {
            // eine Zeile nach oben
            return this.row - 1;
        }
        return this.row;
    }

    /**
     * Zurckgeben der Spalte der
     * nchsten Position, wenn die Linie
     * weiter laufen wrde.
     *
     * @return
     */
    /*public*/ int getNextPosCol()
    {
        if ( this.arrowChar == '>' )
            // nach rechts
        {
            // eine Spalte nach rechts
            return this.col + 1;
        }
        else if ( this.arrowChar == '<' )
        {
            // eine Spalte nach links
            return this.col - 1;
        }
        return this.col;
    }

    /**
     * @see FlowConnectionPart#matchPosition(int, int)
     */
    /*public*/ boolean matchPosition(
            final int pRowNr,
            final int pColNr)
    {
        return //--
            pRowNr == this.row
            && pColNr == this.col;
    }

    /**
     * debug output
     */
    public String toString()
    {
        return "\n\t\t" + StringUtil.getShortClassName( this ) + " " //--
            + "row: " + this.row + " " //--
            + "col: " + this.col + " " //--
            + "direction: " + FlowConnection.getDirectionStr( this.direction ) + " " //--
            + "arrowChar: " + this.arrowChar;
    }

    /**
     * static methods
     */

    /**
     * parse
     *
     * @param pFlowStrArr
     * @param pRow
     * @param pCol
     * @return
     * @throws FlowNetParseException
     */
    /*public*/ static FlowConnectionArrow parse(
            final String[] pFlowStrArr,
            final int pRow,
            final int pCol,
            final int pDirection,
            final FlowConnection pStartFlowConnection )
        throws FlowNetParseException
    {
        final String lineStr = pFlowStrArr[ pRow ];
        if ( pCol >= lineStr.length() )
        {
            return null;
        }
        final char arrowChar = lineStr.charAt( pCol );
        switch ( pDirection )
        {
            case FlowConnection.UP:
                // TODO Konstante fr arrowChar
                if ( ! ( arrowChar == 'A' || arrowChar == '^' ) )
                {
                    return null;
                }
                if ( ! ( pRow > 0 //--
                        && pCol < pFlowStrArr[ pRow - 1 ].length() //--
                        && ! Character.isWhitespace( pFlowStrArr[ pRow - 1 ].charAt( pCol ) ) ) )
                {
                    throw new FlowNetParseException( "Arrow has no destination at row " + ( pRow + 1 ) + " col " + ( pCol + 1 ) + " direction " + FlowConnection.getDirectionStr( pDirection ) );
                }
                if ( ! isSpecialChar( pFlowStrArr[ pRow - 1 ].charAt( pCol ) ) )
                {
                    throw new FlowNetParseException( "Arrow has wrong destination at row " + ( pRow + 1 ) + " col " + ( pCol + 1 ) + " direction " + FlowConnection.getDirectionStr( pDirection ) );
                }
                if ( pFlowStrArr[ pRow - 1 ].charAt( pCol ) == '|' )
                    // Prfen auf ein '|'-Zeichen eine Zeile hher, wenn ein '|'-Zeichen da ist, handelt es sichwahrscheinlich um einen Text
                    // TODO: funktioniert diese Sonderlsung auch in Zukunft bei mehrzeiligen Texten ???
                {
                    return null;
                }
                break;
            case FlowConnection.DOWN:
                if ( ! ( arrowChar == 'V' || arrowChar == 'v' ) )
                {
                    return null;
                }
                if ( ! ( pRow < pFlowStrArr.length - 1 //--
                        && pCol < pFlowStrArr[ pRow + 1 ].length() //--
                        && ! Character.isWhitespace( pFlowStrArr[ pRow + 1 ].charAt( pCol ) ) ) )
                {
                    throw new FlowNetParseException( "Arrow has no destination at row " + ( pRow + 1 ) + " col " + ( pCol + 1 ) + " direction " + FlowConnection.getDirectionStr( pDirection ) );
                }
                if ( ! isSpecialChar( pFlowStrArr[ pRow + 1 ].charAt( pCol ) ) )
                {
                    throw new FlowNetParseException( "Arrow has wrong destination at row " + ( pRow + 1 ) + " col " + ( pCol + 1 ) + " direction " + FlowConnection.getDirectionStr( pDirection ) );
                }
                if ( pFlowStrArr[ pRow + 1 ].charAt( pCol ) == '|' )
                    // Prfen auf ein '|'-Zeichen eine Zeile tiefer, wenn ein '|'-Zeichen da ist, handelt es sichwahrscheinlich um einen Text
                    // TODO: funktioniert diese Sonderlsung auch in Zukunft bei mehrzeiligen Texten ???
                {
                    return null;
                }
                break;
            case FlowConnection.LEFT:
                if ( ! ( arrowChar == '<' ) )
                {
                    return null;
                }
                if ( ! ( pCol > 0 //--
                        // && pCol < pFlowStrArr[ pRow ].length() //--
                        && ! Character.isWhitespace( pFlowStrArr[ pRow ].charAt( pCol - 1 ) ) ) )
                {
                    throw new FlowNetParseException( "Arrow has no destination at row " + ( pRow + 1 ) + " col " + ( pCol + 1 ) + " direction " + FlowConnection.getDirectionStr( pDirection ) );
                }
                if ( ! isSpecialChar( pFlowStrArr[ pRow ].charAt( pCol - 1 ) ) )
                {
                    throw new FlowNetParseException( "Arrow has wrong destination at row " + ( pRow + 1 ) + " col " + ( pCol + 1 ) + " direction " + FlowConnection.getDirectionStr( pDirection ) );
                }
                if ( lineStr.charAt( pCol - 1 ) == '-' )
                    // Prfen auf ein '-'-Zeichen eine Spalte links, wenn ein '-'-Zeichen da ist, handelt es sichwahrscheinlich um einen Text
                {
                    return null;
                }
                break;
            case FlowConnection.RIGHT:
                if ( ! ( arrowChar == '>' ) )
                {
                    return null;
                }
                if ( ! ( pCol > 0 //--
                        && pCol < lineStr.length() - 1 //--
                        && ! Character.isWhitespace( pFlowStrArr[ pRow ].charAt( pCol + 1 ) ) ) )
                {
                    throw new FlowNetParseException( "Arrow has no destination at row " + ( pRow + 1 ) + " col " + ( pCol + 1 ) + " direction " + FlowConnection.getDirectionStr( pDirection ) );
                }
                if ( ! isSpecialChar( pFlowStrArr[ pRow ].charAt( pCol - 1 ) ) )
                {
                    throw new FlowNetParseException( "Arrow has wrong destination at row " + ( pRow + 1 ) + " col " + ( pCol + 1 ) + " direction " + FlowConnection.getDirectionStr( pDirection ) );
                }
                if ( lineStr.charAt( pCol + 1 ) == '-' )
                    // Prfen auf ein '-'-Zeichen eine Spalte rechts, wenn ein '-'-Zeichen da ist, handelt es sichwahrscheinlich um einen Text
                {
                    return null;
                }
                break;
            default:
                throw new RuntimeException( "unknown direction: " + pDirection );
        }
        return new FlowConnectionArrow(
                pRow,
                pCol,
                arrowChar,
                pDirection,
                pStartFlowConnection );
    }

    /**
     * Prfen, ob das bergebene Zeichen ein Sonderzeichen ist.
     *
     * @param pChar
     * @return
     */
    private static boolean isSpecialChar(
            final char pChar)
    {
        return ! ( //--
                Character.isLetter( pChar ) //--
                || Character.isDigit( pChar ) //--
                || Character.isWhitespace( pChar ) );
    }

}
