package de.heinerkuecker.asciiartflowparser;
import de.heinerkuecker.util.StringUtil;

/**
 *  Richtungsnderung einer Verbindungs-Linie
 *  mit einem Plus-Zeichen (nicht Split oder Join).
 *
 * FlowConnectionBend (Krmme, Kurve).
 *
 * @author Heiner Kcker
 */
public final class FlowConnectionBend
extends FlowConnectionPart
{
    private final int row;
    private final int col;
    private final int inDirection;
    private final int outDirection;


    /**
     * @return the row
     */
    /*public*/ int getRow()
    {
        return this.row;
    }

    /**
     * @return the col
     */
    /*public*/ int getCol()
    {
        return this.col;
    }

    /**
     * Constructor.
     *
     * @param pRow
     * @param pCol
     * @param pArrowChar
     */
    private FlowConnectionBend(
            final int pRow,
            final int pCol,
            final int pInDirection,
            final int pOutDirection,
            final FlowConnection pFlowConnection )
    {
        super( pFlowConnection );
        this.row = pRow;
        this.col = pCol;
        this.inDirection = pInDirection;
        this.outDirection = pOutDirection;
    }

    /**
     * Zurckgeben der Zeile der
     * nchsten Position nach der
     * Abbiegung.
     *
     * @return
     */
    /*public*/ int getNextPosRow()
    {
        if ( this.outDirection == FlowConnection.DOWN )
            // nach unten
        {
            // eine Zeile nach unten
            return this.row + 1;
        }
        else if ( this.outDirection == FlowConnection.UP )
            // nach oben
        {
            // eine Zeile nach oben
            return this.row - 1;
        }
        return this.row;
    }

    /**
     * Zurckgeben der Spalte der
     * nchsten Position nach der
     * Abbiegung.
     *
     * @return
     */
    /*public*/ int getNextPosCol()
    {
        if ( this.outDirection == FlowConnection.RIGHT )
            // nach rechts
        {
            // eine Spalte nach rechts
            return this.col + 1;
        }
        else if ( this.outDirection == FlowConnection.LEFT )
        {
            // eine Spalte nach links
            return this.col - 1;
        }
        return this.col;
    }

    /**
     * @return the outDirection
     */
    /*public*/ int getNextDirection() {
        return this.outDirection;
    }

    /**
     * @see FlowConnectionPart#matchPosition(int, int)
     */
    /*public*/ boolean matchPosition(
            final int pRowNr,
            final int pColNr)
    {
        return //--
            pRowNr == this.row
            && pColNr == this.col;
    }

    /**
     * debug output
     */
    public String toString()
    {
        return "\n\t\t" + StringUtil.getShortClassName( this ) + " " //--
            + "row: " + this.row + " " //--
            + "col: " + this.col + " " //--
            + "inDirection: " + FlowConnection.getDirectionStr( this.inDirection ) + " " //--
            + "outDirection: " + FlowConnection.getDirectionStr( this.outDirection );
    }

    /**
     * static methods
     */

    /**
     * parse
     *
     * @param pFlowStrArr
     * @param pRow
     * @param pCol
     * @param pDirection
     * @return
     */
    /*public*/ static FlowConnectionBend parse(
            final String[] pFlowStrArr,
            final int pRow,
            final int pCol,
            final int pDirection,
            final FlowConnection pFlowConnection )
    {
        final String lineStr = pFlowStrArr[ pRow ];
        if ( pCol >= lineStr.length() )
        {
            return null;
        }
        final char arrowChar = lineStr.charAt( pCol );
        if ( arrowChar != '+' )
            // TODO Konstante
        {
            return null;
        }
        final int outDirection;

        // Suchen nach Richtung der Fortsetzung
        if ( pDirection == FlowConnection.UP //--
                || pDirection == FlowConnection.DOWN )
            // alte Richtung war hoch oder runter
        {
            // suche links
            if ( pCol > 1 //--
                    && pFlowStrArr[ pRow ].charAt( pCol - 1 ) == '-' )
            {
                outDirection = FlowConnection.LEFT;
            }
            // suche rechts
            else if ( pCol < pFlowStrArr[ pRow ].length() - 2 //--
                    && pFlowStrArr[ pRow ].charAt( pCol + 1 ) == '-' )
            {
                outDirection = FlowConnection.RIGHT;
            }
            else
            {
                System.err.println( "FlowConnection lost at row " + ( pRow + 1 ) + " col " + ( pCol + 1 ) + " for plus sign" );
                return null;
                //throw new FlowNetParseException( "FlowConnection lost at row " + pRow + " col " + pCol + " for plus sign" );
            }
        }
        else
            // alte Richtung war links oder rechts
        {
            // suche hoch
            if ( pRow > 1 //--
                    && pCol < pFlowStrArr[ pRow - 1 ].length() //--
                    && pFlowStrArr[ pRow - 1 ].charAt( pCol ) == '|' )
            {
                outDirection = FlowConnection.UP;
            }
            // suche runter
            else if ( pRow < pFlowStrArr.length - 2 //--
                    && pCol < pFlowStrArr[ pRow + 1 ].length() //--
                    && pFlowStrArr[ pRow + 1 ].charAt( pCol ) == '|' )
            {
                outDirection = FlowConnection.DOWN;
            }
            else
            {
                System.err.println( "FlowConnection lost at row " + ( pRow + 1 ) + " col " + ( pCol + 1 ) + " for plus sign" );
                return null;
                //throw new FlowNetParseException( "FlowConnection lost at row " + pRow + " col " + pCol + " for plus sign" );
            }

        }

        final FlowConnectionBend flowConnectionBend = new FlowConnectionBend(
                            pRow,
                            pCol,
                            pDirection,
                            outDirection,
                            pFlowConnection );
//System.out.println( flowConnectionBend );
        return flowConnectionBend;
    }

}
