package de.heinerkuecker.asciiartflowparser;
import de.heinerkuecker.util.StringUtil;

/**
 * Einfliessen/Vereinigen (Join) einer
 * Verbindung (FlowConnection) in eine
 * andere.
 *
 * Beide ge-jointe FlowConnection haben die
 * gleiche End-FlowNode.
 *
 * @author Heiner Kcker
 */
public final class FlowConnectionJoin
extends FlowConnectionPart
{
    /**
     * Merker fr die FlowConnection, von der
     * dieser Join kommt.
     * Wird nur fr das Auflsen (Zuordnen
     * joinInFlowConnection) nach dem Parsen
     * bentigt
     */
    //private final FlowConnection startFlowConnection;

    /**
     * In diese FlowConnection fliesst dieser
     * Join ein.
     */
    private final FlowConnection joinInFlowConnection;

    /**
     * In diesen FlowConnectionPart fliesst dieser
     * Join ein.
     */
    private final FlowConnectionPart joinInFlowConnectionPart;

    private final int row;
    private final int col;
    private final int inDirection;
    //private final int outDirection;

    /**
     * Constructor.
     */
//    private FlowConnectionJoin()
//    {
//        super();
//    }

    /**
     * @return the joinInFlowConnectionPart
     */
    /*public*/ FlowConnectionPart getJoinInFlowConnectionPart()
    {
        return this.joinInFlowConnectionPart;
    }

    /**
     * @return the col
     */
    /*public*/ int getCol() {
        return this.col;
    }

    /**
     * @return the inDirection
     */
    /*public*/ int getInDirection()
    {
        return this.inDirection;
    }

    /**
     * @return the joinInFlowConnection
     */
    /*public*/ FlowConnection getJoinInFlowConnection()
    {
        return this.joinInFlowConnection;
    }

    /**
     * @return the row
     */
    /*public*/ int getRow() {
        return this.row;
    }

    /**
     * @return the startFlowConnection
     */
//  /*public*/ FlowConnection getStartFlowConnection() {
//      return this.startFlowConnection;
//  }

    /**
     * Constructor.
     */
    /*private*/ FlowConnectionJoin(
            final FlowConnection pStartFlowConnection,
            final FlowConnection pJoinFlowConnection,
            final FlowConnectionPart pJoinFlowConnectionPart,
            final int pRow,
            final int pCol,
            final int pInDirection
            //final int pOutDirection
            )
    {
        super( pStartFlowConnection );
        //this.startFlowConnection = pStartFlowConnection;
        this.joinInFlowConnection = pJoinFlowConnection;
        this.joinInFlowConnectionPart = pJoinFlowConnectionPart;
        this.row = pRow;
        this.col = pCol;
        this.inDirection = pInDirection;
        //this.outDirection = pOutDirection;

        if ( pStartFlowConnection == pJoinFlowConnection )
        {
            throw new RuntimeException();
        }

    }

    /**
     * @see FlowConnectionPart#matchPosition(int, int)
     */
    /*public*/ boolean matchPosition(
            final int pRowNr,
            final int pColNr)
    {
        return //--
            pRowNr == this.row
            && pColNr == this.col;
    }

    /**
     * debug output
     */
    public String toString()
    {
        return "\n\t\t" + StringUtil.getShortClassName( this ) + " " //--
            + "row: " + this.row + " " //--
            + "col: " + this.col + " " //--
            + "inDirection: " + FlowConnection.getDirectionStr( this.inDirection );
    }

}
