package de.heinerkuecker.asciiartflowparser;
import de.heinerkuecker.util.StringUtil;

/**
 * Diese Klasse reprsentiert einen horizontalen
 * oder vertikalen Verbindungslinien-Teil ohne
 * Abzweige, Kreuzungen oder anderes.
 *
 * @author Heiner Kcker
 */
public final class FlowConnectionLinePart
extends FlowConnectionPart
{
    private final int rowLeftTop;

    private final int colLeftTop;

    private final int rowRightBottom;

    private final int colRightBottom;

    private final int direction;

    /**
     * Constructor.
     *
     * @param pRowLeftTop
     * @param pColLeftTop
     * @param pRowRightBottom
     * @param pColRightBottom
     * @param pDirection
     */
    private FlowConnectionLinePart(
            final int pRowLeftTop,
            final int pColLeftTop,
            final int pRowRightBottom,
            final int pColRightBottom,
            final int pDirection,
            final FlowConnection pFlowConnection )
    {
        super( pFlowConnection );
        this.rowLeftTop = pRowLeftTop;
        this.colLeftTop = pColLeftTop;
        this.rowRightBottom = pRowRightBottom;
        this.colRightBottom = pColRightBottom;
        this.direction = pDirection;

//      if ( ! ( pRowLeftTop != pRowRightBottom //--
//              || pColLeftTop != pColRightBottom ) )
//      {
//          throw new IllegalArgumentException( "no horicontal or vertical line: " + this );
//      }
    }

    /**
     * debug output.
     */
    public String toString()
    {
        return "\n\t\t" + StringUtil.getShortClassName( this ) + " " //--
            + "rowLeftTop: " + this.rowLeftTop + " " //--
            + "colLeftTop: " + this.colLeftTop + " " //--
            + "rowRightBottom: " + this.rowRightBottom + " " //--
            + "colRightBottom: " + this.colRightBottom + " " //--
            + "direction: " + FlowConnection.getDirectionStr( this.direction );
    }

    /**
     * @return the colLeftTop
     */
    /*public*/ int getColLeftTop() {
        return this.colLeftTop;
    }

    /**
     * @return the colRightBottom
     */
    /*public*/ int getColRightBottom() {
        return this.colRightBottom;
    }

    /**
     * @return the rowLeftTop
     */
    /*public*/ int getRowLeftTop() {
        return this.rowLeftTop;
    }

    /**
     * @return the rowRightBottom
     */
    /*public*/ int getRowRightBottom() {
        return this.rowRightBottom;
    }

    /**
     * Zurckgeben der Zeile der
     * nchsten Position, wenn die Linie
     * weiter laufen wrde.
     *
     * @return
     */
    /*public*/ int getNextPosRow()
    {
//      if ( this.rowLeftTop == this.rowRightBottom )
//          // horizontal
//      {
//          // Zeile bleibt gleich
//          return this.rowLeftTop;
//      }
//      if ( this.rowLeftTop < this.rowRightBottom )
//          // nach unten
//      {
//          // eine Zeile nach unten
//          return this.rowRightBottom + 1;
//      }
//      else
//      {
//          // eine Zeile nach oben
//          return this.rowRightBottom - 1;
//      }
        switch ( this.direction )
        {
            case FlowConnection.UP:
                // eine Zeile nach oben
                return this.rowRightBottom - 1;
            case FlowConnection.DOWN:
                // eine Zeile nach unten
                return this.rowRightBottom + 1;
            default:
                // Zeile bleibt gleich
                return this.rowLeftTop;
        }
    }

    /**
     * Zurckgeben der Spalte der
     * nchsten Position, wenn die Linie
     * weiter laufen wrde.
     *
     * @return
     */
    /*public*/ int getNextPosCol()
    {
//      if ( this.colLeftTop == this.colRightBottom )
//          // vertikal
//      {
//          // Spalte bleibt gleich
//          return this.colLeftTop;
//      }
//      if ( this.colLeftTop < this.colRightBottom )
//          // nach rechts
//      {
//          // eine Spalte nach rechts
//          return this.colRightBottom + 1;
//      }
//      else
//      {
//          // eine Spalte nach links
//          return this.colRightBottom - 1;
//      }
        switch ( this.direction )
        {
            case FlowConnection.LEFT:
                // eine Spalte nach links
                return this.colRightBottom - 1;
            case FlowConnection.RIGHT:
                // eine Spalte nach rechts
                return this.colRightBottom + 1;
            default:
                // Spalte bleibt gleich
                return this.colLeftTop;
        }
    }

    /**
     * @see FlowConnectionPart#matchPosition(int, int)
     */
    /*public*/ boolean matchPosition(
            final int pRowNr,
            final int pColNr)
    {
        return //--
        ( this.direction == FlowConnection.RIGHT //--
                && pRowNr == this.rowLeftTop //--
                && pRowNr == this.rowRightBottom //--
                && pColNr >= this.colLeftTop
                && pColNr <= this.colRightBottom ) //--
            ||( this.direction == FlowConnection.LEFT //--
                    && pRowNr == this.rowLeftTop //--
                    && pRowNr == this.rowRightBottom //--
                    && pColNr <= this.colLeftTop
                    && pColNr >= this.colRightBottom ) //--
            || ( this.direction == FlowConnection.DOWN //--
                    && pColNr == this.colLeftTop //--
                    && pColNr == this.colRightBottom //--
                    && pRowNr >= this.rowLeftTop //--
                    && pRowNr <= this.rowRightBottom ) //--
            || ( this.direction == FlowConnection.UP //--
                && pColNr == this.colLeftTop //--
                && pColNr == this.colRightBottom //--
                && pRowNr <= this.rowLeftTop //--
                && pRowNr >= this.rowRightBottom ) ;
    }
    /**
     * static methods
     */

    /**
     * Parse FlowConnectionLinePart.
     *
     * @param pFlowStrArr
     * @param pRowStart
     * @param pColStart
     * @param pDirection
     * @return
     */
    /*public*/ static FlowConnectionLinePart parse(
            final String[] pFlowStrArr,
            final int pRowStart,
            final int pColStart,
            final int pDirection,
            final FlowConnection pFlowConnection )
    {
        final FlowConnectionLinePart flowConnectionLinePart;
        if ( pDirection == FlowConnection.LEFT )
            // nach links
        {
            int colNr = pColStart;
            final String lineStr = pFlowStrArr[ pRowStart ];
            while ( colNr > 0 //--
                    && lineStr.charAt( colNr ) == '-' )
            {
                colNr--;
            }
            if ( colNr >= pColStart )
                // es gab kein '-'
            {
                // kein FlowConnectionLinePart
                return null;
            }
            flowConnectionLinePart = new FlowConnectionLinePart(
                    pRowStart,
                    pColStart,
                    pRowStart,
                    colNr + 1 ,
                    pDirection,
                    pFlowConnection );
        }
        else if ( pDirection == FlowConnection.RIGHT )
            // nach rechts
        {
            int colNr = pColStart;
            final String lineStr = pFlowStrArr[ pRowStart ];
            while ( colNr < lineStr.length() //--
                    && lineStr.charAt( colNr ) == '-' )
            {
                colNr++;
            }
            if ( colNr <= pColStart )
                // es gab kein '-'
            {
                // kein FlowConnectionLinePart
                return null;
            }
            flowConnectionLinePart = new FlowConnectionLinePart(
                    pRowStart,
                    pColStart,
                    pRowStart,
                    colNr - 1 ,
                    pDirection,
                    pFlowConnection );
        }
        else if ( pDirection == FlowConnection.DOWN )
            // nach unten
        {
            int rowNr = pRowStart;
            while ( rowNr < pFlowStrArr.length //--
                    && pColStart < pFlowStrArr[ rowNr ].length() //--
                    && pFlowStrArr[ rowNr ].charAt( pColStart ) == '|' )
            {
                rowNr++;
            }
            if ( rowNr <= pRowStart )
                // es gab kein '|'
            {
                // kein FlowConnectionLinePart
                return null;
            }
            flowConnectionLinePart = new FlowConnectionLinePart(
                    pRowStart,
                    pColStart,
                    rowNr - 1,
                    pColStart ,
                    pDirection,
                    pFlowConnection );
        }
        else if ( pDirection == FlowConnection.UP )
            // nach oben
        {
            int rowNr = pRowStart;
            while ( rowNr > 0 //--
                    && pColStart < pFlowStrArr[ rowNr ].length() //--
                    && pFlowStrArr[ rowNr ].charAt( pColStart ) == '|' )
            {
                rowNr--;
            }
            if ( rowNr >= pRowStart )
                // es gab kein '|'
            {
                // kein FlowConnectionLinePart
                return null;
            }
            flowConnectionLinePart = new FlowConnectionLinePart(
                    pRowStart,
                    pColStart,
                    rowNr + 1,
                    pColStart ,
                    pDirection,
                    pFlowConnection );
        }
        else
        {
            throw new IllegalArgumentException( "unsupported direction " + pDirection );
        }

//System.out.println( flowConnectionLinePart );
        return flowConnectionLinePart;
    }

}
