package de.heinerkuecker.asciiartflowparser;

import java.util.ArrayList;
import java.util.Stack;

import de.heinerkuecker.util.StringUtil;

/**
 * Aufteilen einer Verbindung
 * zu zwei Verbindungen.
 *
 * Aufgeteilte Verbindungen
 * werden als zwei Verbindungen
 * gespeichert.
 *
 * @author Heiner Kcker
 */
public final class FlowConnectionSplit
extends FlowConnectionPart
//extends FlowConnection
{
    //private final FlowConnection startFlowConnection;

    private final FlowConnection splitOutFlowConnection;

    private final int row;
    private final int col;
    private final int inDirection;
    private final int outDirection;
    private final int splitDirection;


    /**
     * @return the splitOutFlowConnection
     */
    /*public*/ FlowConnection getSplitOutFlowConnection()
    {
        return this.splitOutFlowConnection;
    }

    /**
     * @return the startFlowConnection
     */
//  /*public*/ FlowConnection getStartFlowConnection()
//  {
//      return this.startFlowConnection;
//  }

    /**
     * @return the row
     */
    /*public*/ int getRow() {
        return this.row;
    }

    /**
     * @return the col
     */
    /*public*/ int getCol() {
        return this.col;
    }

    /**
     * Constructor.
     *
     * @param pStartFlowConnection
     * @param pSplitOutFlowConnection
     */
    private FlowConnectionSplit(
            final FlowConnection pStartFlowConnection,
            final FlowConnection pSplitOutFlowConnection,
            final int pRow,
            final int pCol,
            final int pInDirection,
            final int pOutDirection,
            final int pSplitDirection )
    {
        //this.startFlowConnection = pStartFlowConnection;
        super( pStartFlowConnection );
        this.splitOutFlowConnection = pSplitOutFlowConnection;
        this.row = pRow;
        this.col = pCol;
        this.inDirection = pInDirection;
        this.outDirection = pOutDirection;
        this.splitDirection = pSplitDirection;
    }

    /**
     * Zurckgeben der Zeile der
     * nchsten Position nach der
     * Abbiegung.
     *
     * @return
     */
    /*public*/ int getNextPosRow()
    {
        if ( this.outDirection == FlowConnection.DOWN )
            // nach unten
        {
            // eine Zeile nach unten
            return this.row + 1;
        }
        else if ( this.outDirection == FlowConnection.UP )
            // nach oben
        {
            // eine Zeile nach oben
            return this.row - 1;
        }
        return this.row;
    }

    /**
     * Zurckgeben der Spalte der
     * nchsten Position nach der
     * Abbiegung.
     *
     * @return
     */
    /*public*/ int getNextPosCol()
    {
        if ( this.outDirection == FlowConnection.RIGHT )
            // nach rechts
        {
            // eine Spalte nach rechts
            return this.col + 1;
        }
        else if ( this.outDirection == FlowConnection.LEFT )
        {
            // eine Spalte nach links
            return this.col - 1;
        }
        return this.col;
    }

    /**
     * @return the outDirection
     */
    /*public*/ int getNextDirection() {
        return this.outDirection;
    }

    /**
     * @see FlowConnectionPart#matchPosition(int, int)
     */
    /*public*/ boolean matchPosition(
            final int pRowNr,
            final int pColNr)
    {
        return //--
            pRowNr == this.row
            && pColNr == this.col;
    }

    /**
     * debug output
     */
    public String toString()
    {
        return "\n\t\t" + StringUtil.getShortClassName( this ) + " " //--
            + "row: " + this.row + " " //--
            + "col: " + this.col + " " //--
            + "inDirection: " + FlowConnection.getDirectionStr( this.inDirection ) + " " //--
            + "outDirection: " + FlowConnection.getDirectionStr( this.outDirection ) + " " //--
            + "splitDirection: " + FlowConnection.getDirectionStr( this.splitDirection );
    }

    /**
     * Zurckgeben der End-FlowNode's.
     * Bei gesplitteten Verbindungen
     * kann es mehrere End-FlowNode
     * geben.
     *
     * @param pFlowConnectionStack Stack zur Absicherung gegen zirkulre Referenzen (StackOverflowError)
     * @return endFlowNode's
     * @throws FlowNetParseException wird geworfen, wenn eine zirkulre Referenz auftaucht
     */
    ///*public*/ FlowNode[] getEndFlowNodeArr(
    /*public*/ FlowConnectionTextArrAndEndNodeValueObj[] getFlowConnectionTextArrAndEndNodeValueObjArr(
            final Stack pFlowConnectionStack)
        throws FlowNetParseException
    {
        //return this.splitOutFlowConnection.getEndFlowNodeArr( //--
        return this.splitOutFlowConnection.getFlowConnectionTextArrAndEndNodeValueObjArr( //--
                pFlowConnectionStack );
    }

    /**
     * static Methoden
     */

    /**
     * Parsen mit Richtungsangabe.
     *
     * @param pFlowStrArr
     * @param pRowStart
     * @param pColStart
     * @return
     * @throws FlowNetParseException
     */
    /*public*/ static FlowConnectionSplit parse(
            final String[] pFlowStrArr,
            final int pRow,
            final int pCol,
            final int pDirection,
            final FlowConnection pStartFlowConnection,
            final FlowNet pFlowNet ,
            final ArrayList pNoEndNodeArrowList,
            final ArrayList pFlowConnectionCrossingList )
        throws FlowNetParseException
    {
        final String lineStr = pFlowStrArr[ pRow ];
        if ( pCol >= lineStr.length() )
        {
            return null;
        }
        final char arrowChar = lineStr.charAt( pCol );
        if ( arrowChar != '+' )
            // TODO Konstante
        {
            return null;
        }
        final int outDirection;
        final int splitDirection;
        final int forkSplitDirection;

        int splitRow = pRow;
        int splitCol = pCol;

        int forkSplitRow = pRow;
        int forkSplitCol = pCol;

        // Suchen nach Richtung der Fortsetzung
        if ( pDirection == FlowConnection.UP //--
                || pDirection == FlowConnection.DOWN )
            // alte Richtung war hoch oder runter
        {
            // suche links
            if ( pCol > 1 //--
                    && pFlowStrArr[ pRow ].charAt( pCol - 1 ) == '-' )
            {
                splitCol--;
                splitDirection = FlowConnection.LEFT;

                if ( pDirection == FlowConnection.UP //--
                        && pRow > 1 //--
                        && pCol < pFlowStrArr[ pRow - 1 ].length() //--
                        && pFlowStrArr[ pRow - 1 ].charAt( pCol ) == '|' )
                {
                    outDirection = FlowConnection.UP;
                    if ( pCol < pFlowStrArr[ pRow ].length() - 1 //--
                            && pFlowStrArr[ pRow ].charAt( pCol + 1 ) == '-' )
                        // 2. Abzweigung nach rechts
                    {
                        forkSplitCol++;
                        forkSplitDirection = FlowConnection.RIGHT;
                    }
                    else
                    {
                        forkSplitDirection = -1;
                    }
                }
                else if ( pDirection == FlowConnection.DOWN //--
                        && pRow < pFlowStrArr.length - 1 //--
                        && pCol < pFlowStrArr[ pRow + 1 ].length() //--
                        && pFlowStrArr[ pRow + 1 ].charAt( pCol ) == '|' )
                {
                    outDirection = FlowConnection.DOWN;
                    if ( pCol < pFlowStrArr[ pRow ].length() - 1 //--
                            && pFlowStrArr[ pRow ].charAt( pCol + 1 ) == '-' )
                        // 2. Abzweigung nach rechts
                    {
                        forkSplitCol++;
                        forkSplitDirection = FlowConnection.RIGHT;
                    }
                    else
                    {
                        forkSplitDirection = -1;
                    }
                }
                else if ( pCol < pFlowStrArr[ pRow ].length() - 1 //--
                        && pFlowStrArr[ pRow ].charAt( pCol + 1 ) == '-' )
                    // T-frmig nach rechts
                {
                    outDirection = FlowConnection.RIGHT;
                    forkSplitDirection = -1;
                }
                else
                {
                    // kein split
                    return null;
                }
            }
            // suche rechts
            else if ( pCol < pFlowStrArr[ pRow ].length() - 2 //--
                    && pFlowStrArr[ pRow ].charAt( pCol + 1 ) == '-' )
            {
                splitCol++;
                splitDirection = FlowConnection.RIGHT;

                if ( pDirection == FlowConnection.UP //--
                        && pRow > 1 //--
                        && pCol < pFlowStrArr[ pRow - 1 ].length() //--
                        && pFlowStrArr[ pRow - 1 ].charAt( pCol ) == '|' )
                {
                    outDirection = FlowConnection.UP;
                    if ( pCol > 1 //--
                            && pFlowStrArr[ pRow ].charAt( pCol - 1 ) == '-' )
                        // 2. Abzweigung nach links
                    {
                        forkSplitCol--;
                        forkSplitDirection = FlowConnection.LEFT;
                    }
                    else
                    {
                        forkSplitDirection = -1;
                    }
                }
                else if ( pDirection == FlowConnection.DOWN //--
                        && pRow < pFlowStrArr.length - 1 //--
                        && pCol < pFlowStrArr[ pRow + 1 ].length() //--
                        && pFlowStrArr[ pRow + 1 ].charAt( pCol ) == '|' )
                {
                    outDirection = FlowConnection.DOWN;
                    if ( pCol > 1 //--
                            && pFlowStrArr[ pRow ].charAt( pCol - 1 ) == '-' )
                        // 2. Abzweigung nach links
                    {
                        forkSplitCol--;
                        forkSplitDirection = FlowConnection.LEFT;
                    }
                    else
                    {
                        forkSplitDirection = -1;
                    }
                }
                else if ( pCol > 1 //--
                        && pFlowStrArr[ pRow ].charAt( pCol - 1 ) == '-' )
                    // T-frmig nach links
                {
                    outDirection = FlowConnection.LEFT;
                    forkSplitDirection = -1;
                }
                else
                {
                    // kein split
                    return null;
                }
            }
            else
            {
                System.err.println( "FlowConnection lost at row " + ( pRow + 1 ) + " col " + ( pCol + 1 ) + " for plus sign" );
                return null;
                //throw new FlowNetParseException( "FlowConnection lost at row " + pRow + " col " + pCol + " for plus sign" );
            }
        }
        else
            // alte Richtung war links oder rechts
        {
            // suche hoch
            if ( pRow > 1 //--
                    && pCol < pFlowStrArr[ pRow - 1 ].length() //--
                    && pFlowStrArr[ pRow - 1 ].charAt( pCol ) == '|' )
            {
                splitRow--;
                splitDirection = FlowConnection.UP;

                if ( pDirection == FlowConnection.LEFT //--
                        && pCol > 1 //--
                        //&& pCol < pFlowStrArr[ pRow ].length() //--
                        && pFlowStrArr[ pRow ].charAt( pCol - 1 ) == '-' )
                {
                    outDirection = FlowConnection.LEFT;
                    if ( pRow < pFlowStrArr.length - 1 //--
                            && pCol < pFlowStrArr[ pRow + 1 ].length() //--
                            && pFlowStrArr[ pRow + 1 ].charAt( pCol ) == '|' )
                        // 2. Abzweigung nach unten
                    {
                        forkSplitRow++;
                        forkSplitDirection = FlowConnection.DOWN;
                    }
                    else
                    {
                        forkSplitDirection = -1;
                    }
                }
                else if ( pDirection == FlowConnection.RIGHT //--
                        //&& pCol > 1 //--
                        && pCol < pFlowStrArr[ pRow ].length() - 1 //--
                        && pFlowStrArr[ pRow ].charAt( pCol + 1 ) == '-' )
                {
                    outDirection = FlowConnection.RIGHT;
                    if ( pRow < pFlowStrArr.length - 1 //--
                            && pCol < pFlowStrArr[ pRow + 1 ].length() //--
                            && pFlowStrArr[ pRow + 1 ].charAt( pCol ) == '|' )
                        // 2. Abzweigung nach unten
                    {
                        forkSplitRow++;
                        forkSplitDirection = FlowConnection.DOWN;
                    }
                    else
                    {
                        forkSplitDirection = -1;
                    }
                }
                else if ( pRow < pFlowStrArr.length - 1 //--
                        && pCol < pFlowStrArr[ pRow + 1 ].length() //--
                        && pFlowStrArr[ pRow + 1 ].charAt( pCol ) == '|' )
                    // T-frmig nach unten
                {
                    outDirection = FlowConnection.DOWN;
                    forkSplitDirection = -1;
                }
                else
                {
                    // kein split
                    return null;
                }
            }
            // suche runter
            else if ( pRow < pFlowStrArr.length - 2 //--
                    && pCol < pFlowStrArr[ pRow + 1 ].length() //--
                    && pFlowStrArr[ pRow + 1 ].charAt( pCol ) == '|' )
            {
                splitRow++;
                splitDirection = FlowConnection.DOWN;

                if ( pDirection == FlowConnection.LEFT //--
                        && pCol > 1 //--
                        //&& pCol < pFlowStrArr[ pRow ].length() //--
                        && pFlowStrArr[ pRow ].charAt( pCol - 1 ) == '-' )
                {
                    outDirection = FlowConnection.LEFT;
                    if ( pRow > 1 //--
                            && pCol < pFlowStrArr[ pRow - 1 ].length() //--
                            && pFlowStrArr[ pRow - 1 ].charAt( pCol ) == '|' )
                        // 2. Abzweigung nach oben
                    {
                        forkSplitRow--;
                        forkSplitDirection = FlowConnection.UP;
                    }
                    else
                    {
                        forkSplitDirection = -1;
                    }
                }
                else if ( pDirection == FlowConnection.RIGHT //--
                        //&& pCol > 1 //--
                        && pCol < pFlowStrArr[ pRow ].length() - 1 //--
                        && pFlowStrArr[ pRow ].charAt( pCol + 1 ) == '-' )
                {
                    outDirection = FlowConnection.RIGHT;
                    if ( pRow > 1 //--
                            && pCol < pFlowStrArr[ pRow - 1 ].length() //--
                            && pFlowStrArr[ pRow - 1 ].charAt( pCol ) == '|' )
                        // 2. Abzweigung nach oben
                    {
                        forkSplitRow--;
                        forkSplitDirection = FlowConnection.UP;
                    }
                    else
                    {
                        forkSplitDirection = -1;
                    }
                }
                else if ( pRow > 1 //--
                        && pCol < pFlowStrArr[ pRow - 1 ].length() //--
                        && pFlowStrArr[ pRow - 1 ].charAt( pCol ) == '|' )
                    // T-frmig nach oben
                {
                    outDirection = FlowConnection.UP;
                    forkSplitDirection = -1;
                }
                else
                {
                    // kein split
                    return null;
                }
            }
            else
            {
                System.err.println( "FlowConnection lost at row " + ( pRow + 1 ) + " col " + ( pCol + 1 ) + " for plus sign" );
                return null;
                //throw new FlowNetParseException( "FlowConnection lost at row " + ( pRow + 1 ) + " col " + ( pCol + 1 ) + " for plus sign" );
            }

        }

        final FlowConnection splitFlowConnection =
            FlowConnection.parse(
                    pFlowStrArr,
                    splitRow,
                    splitCol,
                    splitDirection,
                    pStartFlowConnection.getStartFlowNode(),
                    pFlowNet,
                    pNoEndNodeArrowList,
                    pFlowConnectionCrossingList,
                    true );

        pFlowNet.addFlowConnection( splitFlowConnection );

        final FlowConnectionSplit flowConnectionSplit =
            new FlowConnectionSplit(
                    pStartFlowConnection,
                    splitFlowConnection,
                    pRow,
                    pCol,
                    pDirection,
                    outDirection,
                    splitDirection );

        if ( forkSplitDirection >= 0 )
        {
            /**
             * Ein Fork-FlowConnectionSplit
             *
             *  +-->
             *  |
             * -+->
             *  |
             *  +->
             *
             *  wird als zweiter FlowConnectionSplit
             *  gespeichert.
             */
            final FlowConnection forkFlowConnection =
                FlowConnection.parse(
                        pFlowStrArr,
                        forkSplitRow,
                        forkSplitCol,
                        forkSplitDirection,
                        pStartFlowConnection.getStartFlowNode(),
                        pFlowNet,
                        pNoEndNodeArrowList,
                        pFlowConnectionCrossingList,
                        true );

            pFlowNet.addFlowConnection( forkFlowConnection );

            final FlowConnectionSplit forkFlowConnectionSplit =
                new FlowConnectionSplit(
                        pStartFlowConnection,
                        forkFlowConnection,
                        pRow,
                        pCol,
                        pDirection,
                        outDirection,
                        forkSplitDirection );

            splitFlowConnection.insertFlowConnectionPart(
                    forkFlowConnectionSplit);

        }

        return flowConnectionSplit;
    }

}
