package de.heinerkuecker.asciiartflowparser;

import java.util.Arrays;

import de.heinerkuecker.util.StringUtil;

/**
 * Value Object fr mehrere FlowConnectionTexte
 * und einen End-Knoten-Text.
 *
 * Dieses Value Object ist notwendig, um gesplittete
 * und gejointe Verbindungen zu organisieren.
 *
 * @author Heiner Kcker
 */
public final class FlowConnectionTextArrAndEndNodeValueObj
implements Comparable
{
    /**
     * Start-Knoten.
     */
    public FlowNode startNode;

    /**
     * Erster Verbindungs-Text.
     * (zur Bequemlichkeit, weil meist nur ein Text exisiteren sollte)
     */
    ///*public*/ FlowConnectionText firstConnectionText;

    /**
     * Texte an der Verbindung.
     */
    public FlowConnectionText[] flowConnectionTextArr;

    /**
     * End-Knoten.
     */
    public FlowNode endNode;

    /**
     * debug output.
     * @see java.lang.Object#toString()
     */
    public String toString()
    {
        return StringUtil.getShortClassName( this ) + " " + //--
            "startNode: " + this.startNode.getNodeTextTrimmed() + " " + //--
            "flowConnectionTextArr: " + Arrays.asList( this.flowConnectionTextArr ) + " " + //--
            "endNode: " + this.endNode.getNodeTextTrimmed() + " ";
    }

    /**
     * Vergleichs-Methode zum Sortieren nach
     * startNodeTextTrimmed (Haupt-Sortier-Kriterium),
     * flowConnectionText(en) (2. Sortier-Kriterium) und
     * endNodeTextTrimmed (3. Sortier-Kriterium).
     *
     * @see java.lang.Comparable#compareTo(java.lang.Object)
     */
    public int compareTo(Object pOther)
    {
        final FlowConnectionTextArrAndEndNodeValueObj other = (FlowConnectionTextArrAndEndNodeValueObj) pOther;

        int retVal;

        // Vergleich startNodeTextTrimmed
        retVal = this.startNode.getNodeTextTrimmed().compareTo( other.startNode.getNodeTextTrimmed() ) ;
        if ( retVal != 0 )
        {
            return retVal;
        }

        // Vergleich flowConnectionText(e)
        for (int i = 0; i < Math.min( this.flowConnectionTextArr.length, other.flowConnectionTextArr.length ); i++)
        {
            FlowConnectionText thisConnText = this.flowConnectionTextArr[i];
            FlowConnectionText otherConnText = other.flowConnectionTextArr[i];
            retVal = thisConnText.compareTo( otherConnText );
            if ( retVal != 0 )
            {
                return retVal;
            }
        }
        // unterschiedliche Anzahl Texte
        retVal = this.flowConnectionTextArr.length - other.flowConnectionTextArr.length;
        if ( retVal != 0 )
        {
            return retVal;
        }

        // Vergleich endNodeTextTrimmed
        retVal = this.endNode.getNodeTextTrimmed().compareTo( other.endNode.getNodeTextTrimmed() ) ;
        return 0;
    }
}
