package de.heinerkuecker.asciiartflowparser;
/**
 * Value Object fr FlowNode (Konten) mit
 * den dazugehrigen FlowConnections
 * (Verbindungen).
 *
 * Dieses Value Object ist notwendig, um
 * die Struktur des FlowNet zur Erzeugung
 * von XML oder anderen Dateien/Objekten
 * sowie zum Prfen der Struktur.
 *
 * @author Heiner Kcker
 */
public final class FlowNodeAndConnectionsValueObj
implements Comparable
{
    /**
     * Start-Knoten.
     */
    public FlowNode startNode;

    /**
     * alle Connections, die von {@link #startNode} ausgehen.
     */
    public FlowConnectionTextArrAndEndNodeValueObj[] connectionValueObjArr;

    /**
     * @see java.lang.Comparable#compareTo(java.lang.Object)
     */
    public int compareTo(Object pOther)
    {
        final FlowNodeAndConnectionsValueObj other = (FlowNodeAndConnectionsValueObj) pOther;

        int retVal;

        retVal = this.startNode.getNodeTextTrimmed().compareTo( other.startNode.getNodeTextTrimmed() ) ;
        if ( retVal != 0 )
        {
            return retVal;
        }

        for (int i = 0; i < Math.min( this.connectionValueObjArr.length, other.connectionValueObjArr.length ); i++)
        {
            FlowConnectionTextArrAndEndNodeValueObj thisValObj = this.connectionValueObjArr[i];
            FlowConnectionTextArrAndEndNodeValueObj otherValObj = other.connectionValueObjArr[i];
            retVal = thisValObj.compareTo( otherValObj );
            if ( retVal != 0 )
            {
                return retVal;
            }
        }
        // unterschiedliche Anzahl Connections
        return this.connectionValueObjArr.length - other.connectionValueObjArr.length;
    }
}
