package de.heinerkuecker.statetransitionengine;


import java.util.*;


/**
 * Helper Class to use many states (session, user, document and so on) on one
 * {@link StateTransitionEngine}
 *
 * <br/>
 * <b>Author:</b> <a href="http://www.heinerkuecker.de" target="_blank">Heiner Kcker</a>
 * <br/>
 *
 * @author Heiner Kcker
 * @version $Id$
 */
public class ApplicationState {

  /**
   * currently state
   */
  // private SourceState currentState;
  private String currentStateName;

  /**
   * gemeinsame StateTransitionEngine fr mehrere ApplicationStates
   */
  private StateTransitionEngine stateTranEng;

  /**
   * constructor with {@link StateTransitionEngine} to bind/using . <br>
   * set the state to initial state if an initial state in the state
   * transition engine is defined.
   */
  public ApplicationState(StateTransitionEngine newStateTransitionEngine) {
    if (newStateTransitionEngine == null) {
      throw new NullPointerException();
    }
    this.stateTranEng = newStateTransitionEngine;
    // setze den initial state, der in der StateTransitionEngine festgelegt ist
    String strInitialStateName = newStateTransitionEngine.getInitialStateName();

    if (strInitialStateName != null) {
      setState(strInitialStateName);
    }
  }// end constructor

  // noch Konstruktor mit StartStateName machen

  /**
   * set the current state
   */
  public void setState(String paStrNewStateName) {
    // SourceState newCurrentState = stateTranEng.getStateForName(paStrNewState);
    //
    // if (newCurrentState == null) {
    // throw new UnknownStateException(paStrNewState);
    // }
    // currentState = newCurrentState;

    if (paStrNewStateName == null) {
      throw new NullPointerException();
    }
    if (!stateTranEng.hasState(paStrNewStateName)) {
      throw new UnknownStateException(paStrNewStateName);
    }

    currentStateName = paStrNewStateName;
  }// end method

  /**
   * set the current state
   */
  // void setState(SourceState paSourceState) {
  // if (newCurrentState == null) {
  // throw new UnknownStateException(paSourceState);
  // }
  // //hier knnte man noch eine Prfung einbauen, ob die engine den state berhaupt kennt
  // currentState = paSourceState;
  // }// end method



  /**
   * get the name of the current state
   */
  public String getCurrentStateName() {
    // if (currentState == null) {
    // throw new InvalidStateException("no current state setted");
    // }
    // return currentState.getName();

    if (currentStateName == null) {
      throw new InvalidStateException("no current state setted");
    }

    return currentStateName;
  }// end method

  /**
   * Get a Collection with the names of the possible states to change from the current state . <br>
   * This method is planned for showing the destination states in a select box or other choice gui element
   */
  public Collection getPossibleStateToChangeNames(Object paObjStateContext, String paStrUserRole) {
    return stateTranEng.getPossibleStateToChangeNames(this, paObjStateContext, paStrUserRole);
  }// end method

  /**
   * Get a Collection with the names of the possible states to change from the current state . <br>
   * This method is planned for showing the destination states in a select box or other choice gui element
   */
  public Collection getPossibleStateToChangeNames(Object paObjStateContext) {
    return stateTranEng.getPossibleStateToChangeNames(this, paObjStateContext);
  }// end method

  /**
   * Get a Collection with the names of the possible states to change from the current state
   * and the corresponding events. <br>
   * This method is planned for showing the destination states in a valuable select box or other choice gui element
   */
  public Collection getPossibleStateToChangeNamesAndEventNames(Object paObjStateContext, String paStrUserRole) {
    return stateTranEng.getPossibleStateToChangeNamesAndEventNames(this, paObjStateContext, paStrUserRole);
  }// end method

  /**
   * Get a Collection with the names of the possible states to change from the current state
   * and the corresponding events. <br>
   * This method is planned for showing the destination states in a valuable select box or other choice gui element
   */
  // Struktur noch richtig beschreiben
  public Collection getPossibleStateToChangeNamesAndEventNames(Object paObjStateContext) {
    return stateTranEng.getPossibleStateToChangeNamesAndEventNames(this, paObjStateContext);
  }// end method

  /**
   * check whether the state is final (has no StateTransitEvents and DestinationStates) or not.
   */
  public boolean isFinal() {
    return stateTranEng.isFinal(this);

  }// end method

  /**
   * check whether the state is final (has no StateTransitEvents and DestinationStates)
   * for the overgiven context or not.
   */
  public boolean isFinal(Object paObjStateContext) {
    return stateTranEng.isFinal(this, paObjStateContext);
  }// end method

  /**
   * check whether the state is final (has no StateTransitEvents and DestinationStates)
   * for the overgiven context and user role or not.
   */
  public boolean isFinal(Object paObjStateContext, String paStrUserRole) {
    return stateTranEng.isFinal(this, paObjStateContext, paStrUserRole);
  }// end method

  /**
   * perform the event on the state map
   *
   * @param paStrEventName name of the event to performed
   * @param paObjStateContext Object with context to perform the event
   *        (information for conditions to perform the event)
   * @throws InvalidStateException, UnknownEventException
   * @throws NoMatchedDestinationStateException if no matched destination state was found
   */
  public void perform(String paStrEventName, Object paObjStateContext, String paStrUserRole)
      throws NoMatchedDestinationStateException {
    stateTranEng.perform(this, paStrEventName, paObjStateContext, paStrUserRole);
  }// end method

  /**
   * perform the event on the state map
   *
   * @param paStrEventName name of the event to performed
   * @param paObjStateContext Object with context to perform the event
   *        (information for conditions to perform the event)
   * @throws InvalidStateException, UnknownEventException
   * @throws NoMatchedDestinationStateException if no matched destination state was found
   */
  public void perform(String paStrEventName, Object paObjStateContext)
      throws NoMatchedDestinationStateException {
    stateTranEng.perform(this, paStrEventName, paObjStateContext);
  }// end method

  /**
   * get the name of the initial state . <br>
   * returns null if no initial state is defined.
   */
  public String getInitialStateName() {
    return stateTranEng.getInitialStateName();
  }// end method

  /**
   * returns if this state equals the initial state of the state transition engine
   */
  public boolean isInitial() {
    if (currentStateName == null) {
      return false;
    }
    return currentStateName.equals(stateTranEng.getInitialStateName());
  }// end method

  /**
   * debug output
   */
  public String toString() {
    return getClass().getName() + "\n" + "currentStateName: " + currentStateName;
  }// end method

}// end class

