package de.heinerkuecker.statetransitionengine;


import java.util.*;


/**
 * an source state of the state transition engine with an map
 * of events to transit the current state to an destination state . <br>
 *
 * <br/>
 * <b>Author:</b> <a href="http://www.heinerkuecker.de" target="_blank">Heiner Kcker</a>
 * <br/>
 *
 * @author Heiner Kcker
 * @version $Id$
 */
public class SourceState {

  /**
   *
   */
  private String name;

  /**
   * the map with the events to transit from this state to another state
   */
  Map eventMap = new TreeMap();

  /**
   * Konstruktor
   */
  public SourceState(String paStrName) {
    if (paStrName == null) {
      throw new NullPointerException();
    }
    name = paStrName ;
  }// end Konstruktor

  /**
   * get Name
   */
  public String getName() {
    return name;
  }// end method

  /**
   *
   */
  public void add(StateTransitEvent paEvent) {
    if (eventMap.get(paEvent.getName()) != null) {
      throw new EventAlreadyDefinedException(paEvent.getName()) ;
    }
    eventMap.put(paEvent.getName() , paEvent);
  }// end method

  /**
   * get an list with all DestinationStates for the overgiven Event
   */
  public List getDestinationStateListForEvent(String paStrEventName) {
    StateTransitEvent tmpEvent = (StateTransitEvent) eventMap.get(paStrEventName) ;

    if (tmpEvent == null) {
      throw new UnknownEventException(paStrEventName) ;
    }
    List retList = tmpEvent.getDestinationStateList() ;

    return retList ;
  }// end method

  /**
   * get an list with all DestinationStates
   */
  public List getDestinationStateList() {
    ArrayList retList = new ArrayList();

    Iterator eventIterator = eventMap.entrySet().iterator();

    while (eventIterator.hasNext()) {
      StateTransitEvent runEvent = (StateTransitEvent) ((Map.Entry) eventIterator.next()).getValue();

      List runList = runEvent.getDestinationStateList() ;

      concat(retList , runList);

    }

    return retList ;
  }// end method

  /**
   * concatenate second parameter list to first parameter list
   */
  private void concat(List paListFirst , List paListSecond) {
    Iterator iter = paListSecond.iterator();

    while (iter.hasNext()) {
      paListFirst.add(iter.next()) ;
    }// while
  }// end method

  /**
   *
   * Notice: be care with the returned object by side effects
   */
  public Map getEventMap() {
    return eventMap ;
  }// end method

  /**
   * debug output
   */
  public String toString() {
    StringBuffer strBuff = new StringBuffer();

    strBuff.append(getClass().getName() + "\n");
    strBuff.append("name: " + name + "\n");
    strBuff.append("eventMap:\n" + eventMap + "\n");
    return strBuff.toString();
  }// end method

  /**
   * debug output
   */
  public String toFormattedString() {
    StringBuffer strBuff = new StringBuffer();

    strBuff.append("-SourceState: " + name + "\n");

    Iterator eventIterator = eventMap.entrySet().iterator();

    while (eventIterator.hasNext()) {
      StateTransitEvent trnsEvent = (StateTransitEvent) ((Map.Entry) eventIterator.next()).getValue();

      strBuff.append(trnsEvent.toFormattedString());
    }

    return strBuff.toString();
  }// end method

}// end class

