package de.heinerkuecker.statetransitionengine;


import java.util.*;


/**
 * event to transit from source state to destination state
 *
 * <br/>
 * <b>Author:</b> <a href="http://www.heinerkuecker.de" target="_blank">Heiner Kcker</a>
 * <br/>
 *
 * @author Heiner Kcker
 * @version $Id$
 */
public class StateTransitEvent {

  /**
   * name
   */
  private String name;

  /**
   * list of destination states
   */
  private List destStateList = new ArrayList();

  /**
   * constructor with name of the StateTransitEvent
   */
  public StateTransitEvent(String paStrName) {
    if (paStrName == null) {
      throw new NullPointerException();
    }
    name = paStrName ;
  }

  /**
   *
   */
  public String getName() {
    return name;
  }

  /**
   * add an destination state
   */
  public void add(DestinationState paDestState) {

    /*
     if (destStateMap.get(paDestState.getOrderPositionNumber()) != null) {
     throw new DestinationStateAlreadyDefinedException(paDestState.getDestStateName() + " in event " + name) ;
     }
     destStateMap.put(paDestState.getOrderPositionNumber() , paDestState);
     */
    destStateList.add(paDestState);
  }// end method

  /**
   * get the list of the destination states of this transit event . <br>
   * Notice: be care with the returned object by side effects
   */
  public List getDestinationStateList() {
    return destStateList ;
  }// end method

  /**
   * debug output
   */
  public String toString() {
    StringBuffer strBuff = new StringBuffer();

    strBuff.append(getClass().getName() + "\n");
    strBuff.append("name: " + name + "\n");
    strBuff.append("destStateList:\n" + destStateList + "\n");
    return strBuff.toString();
  }// end method

  /**
   * debug output
   */
  public String toFormattedString() {
    StringBuffer strBuff = new StringBuffer();

    strBuff.append("    -StateTransitEvent: " + name + "\n");
    Iterator dstStateIterator = destStateList.iterator();

    while (dstStateIterator.hasNext()) {
      DestinationState dstState = (DestinationState) dstStateIterator.next();

      strBuff.append(dstState.toFormattedString());
    }

    return strBuff.toString();
  }// end method

}// end class

