package de.heinerkuecker.util;
import java.io.*;
import java.math.BigDecimal;
import java.sql.Timestamp;
import java.text.*;
import java.util.*;

/**
 * Alle mglichen Methoden, hauptschlich fr Strings . <br/>
 *
 * <br/>
 * <b>Author:</b> <a href="http://www.heinerkuecker.de" target="_blank">Heiner Kcker</a>
 * <br/>
 *
 * @author Heiner Kcker
 * @version $Id: StringUtil.java,v 1.4 2005/07/19 16:42:17 hku Exp $
 */
public final class StringUtil {

  /**
   * Datumsformat fr {@link #date2str} und {@link #parseDate}
   */
  private static SimpleDateFormat formatDate = new SimpleDateFormat("dd.MM.yyyy");

  /**
   * Timestamp-Datumsformat fr {@link #getTimeStamp}
   */
  private static SimpleDateFormat formatTimestamp = new SimpleDateFormat("dd.MM.yyyy HH:mm:ss", Locale.GERMAN);

  /**
   * Constructor declaration<br/>
   * Instanziieren nicht erlaubt
   */
  private StringUtil()
  {
    super();
  }

  /**
   * String auf null, Lnge 0 oder nur Spaces prfen
   *
   * @param strValue zu prfender String
   * @return ob String empty (ja/nein)
   */
  public static final boolean empty(String strValue) {
    boolean bReturn = true;

    if (strValue != null) {
      if (strValue.trim().length() > 0) {
        bReturn = false;
      }
    }

    return bReturn;
  } // end method empty

  /**
   * StringBuffer auf leer prfen
   *
   * @param strBuffValue zu prfender StringBuffer
   * @return ob StringBuffer empty (ja/nein)
   */
  public static final boolean empty(StringBuffer strBuffValue) {
    boolean bReturn = true;

    if (strBuffValue != null) {
      if (strBuffValue.toString().trim().length() > 0) {
        bReturn = false;
      }
    }

    return bReturn;
  } // end method empty

  /**
   * Verlngern eines Strings auf die gewnschte Lnge mit Leerzeichen. <br/>
   * Der String wird nicht abgeschnitten, falls er lnger ist als verlangt.
   *
   * @param pStr zu verlngernder String
   * @param pLen gewnschte Lnge
   * @return verlngerter String
   */
  public static final String pad(
          final String pStr,
          final int pLen)
  {
    String str = pStr;
    if (pStr == null)
    {
      str = "";
    }// end if

    if (str.length() >= pLen)
    {
      return str;
    }// end if

    StringBuffer strBuff = new StringBuffer(pLen);
    /**
     * Beschleunigung durch Bndelung der Aufrufe in einem eigenen synchronisierten Block
     * @see http://www.rz.uni-hohenheim.de/anw/programme/prg/java/tutorials/javainsel4/javainsel_09_007.htm#t2t312
     */
    synchronized ( strBuff )
    {
        strBuff.append(str);
        while (strBuff.length() < pLen)
        {
          strBuff.append(' ');
        }// end while
        return strBuff.toString();
    }
  }// end method pad

  /**
   * String in Grossbuchtaben umwandeln, mit null-Abfangung
   *
   * @param strValue umzuwandlender String
   * @return umgewandelter String oder null, wenn Parameter-String null war
   */
  public static final String upper(String strValue) {
    if (strValue != null) {
      strValue = strValue.toUpperCase();
    }
    return strValue;
  } // end method upper

  /**
   * String.toLowerCase() mit Abfangung Null-Pointer. <br/>
   *
   * @param pStr umzuwandelnder String
   * @return in Kleinbuchstaben umgewandelter String
   */
  public static final String lower(
          final String pStr)
  {
    if (pStr == null) {
      return null;
    }// end if

    return pStr.toLowerCase();
  }// end method lower

  /**
   * Verlngern eines Strings auf die gewnschte Lnge durch
   * Einfgen von Leerzeichen am Anfang. <br/>
   * Der String wird nicht abgeschnitten, falls er lnger ist als verlangt.
   *
   * @param pStr zu verlngernder String
   * @param pLen gewnschte Lnge
   * @return verlngerter String
   */
  public static final String leftPad(
          String pStr,
          final int pLen)
  {
    if (pStr == null) {
      pStr = "";
    }// end if

    if (pStr.length() >= pLen) {
      return pStr;
    }// end if

    StringBuffer sb = new StringBuffer(pLen);

    sb.append(pStr);
    while (sb.length() < pLen) {
      sb.insert(0, ' ');
    }// end while
    return sb.toString();
  }// end method leftPad

  /**
   * Filter the specified string for characters that are senstive to
   * HTML interpreters, returning the string with these characters replaced
   * by the corresponding character entities.
   *
   * @param value The string to be filtered and returned
   * @return the filtered String
   */
  public static final String encodeHtml(String value) {
    if (value == null) {
      return (null);
    }

    char  content[] = new char[value.length()];

    value.getChars(0, value.length(), content, 0);

    StringBuffer  result = new StringBuffer(content.length + 50);

    for (int i = 0; i < content.length; i++) {
      switch (content[i]) {

      case '<':
        result.append("&lt;");

        break;

      case '>':
        result.append("&gt;");

        break;

      case '&':
        result.append("&amp;");

        break;

      case '"':
        result.append("&quot;");

        break;

      default:
        result.append(content[i]);
      }
    }

    return (result.toString());
  } // end method encodeHtml

  /**
   * String in int umwandeln
   *
   * @param numStr String numerisch
   * @return Zahl
   */
  public static final int str2int(String numStr) {
    if (empty(numStr)) {
      return 0;
    }

    numStr = numStr.trim();

    int retInt = 0;

    try {
      retInt = (new Integer(numStr)).intValue();
    } catch (Exception e) {
      ;
    }

    return retInt;
  } // end method str2int

  /**
   * String in Integer-Object umwandeln
   *
   * @param numStr String numerisch
   * @return Intger-Object
   */
  public static final Integer str2Integer(String numStr) {
    Integer retInteger = null;

    retInteger = new Integer(str2int(numStr));
    return retInteger;
  } // end method str2Integer

  /**
   * String in Integer-Object umwandeln mit Rckgabe null wen nicht lesbar
   *
   * @param numStr String numerisch
   * @return Integer-Object
   */
  public static final Integer str2IntegerStrict(String numStr) {
    Integer retInteger = null;

    try {
      retInteger = new Integer(numStr);
    }
    catch (Exception e)
    {
      ;// nicht lesbar , return null
    }

    return retInteger;
  } // end method str2IntegerStrict

  /**
   * String in Long-Object umwandeln mit Rckgabe null wen nicht lesbar
   *
   * @param numStr String numerisch
   * @return Long-Object
   */
  public static final Long str2LongStrict(String numStr)
  {
    Long retLong = null;

    try {
       retLong = new Long( numStr.trim() );
    }
    catch (Exception e)
    {
      ;// nicht lesbar , return null
    }

    return retLong;
  } // end method str2LongStrict

  /**
   * Parsen eines Strings in ein Integer,
   * unter Nachsicht auf fhrende Nullen
   * und nachfolgende Buchstaben oder
   * Sonderzeichen
   *
   * @author Heiner Kcker
   * @since 19.05.2004
   */
  public static Integer str2IntegerLenient(String pStr)
  {
      if (pStr == null || pStr.length() == 0 )
      {
          return null; // Garbage in, Garbage out
      }
      final String trimmedStr = pStr.trim();
      StringBuffer tmpStrBuff = new StringBuffer();
      int iPos = 0;
      //laufen ber fhrende Nullen
      for(; iPos < trimmedStr.length() - 1 && trimmedStr.charAt(iPos) == '0'; iPos++);
      //sammeln aller Digits
      for(; iPos < trimmedStr.length() && Character.isDigit(trimmedStr.charAt(iPos)); iPos++)
      {
          tmpStrBuff.append(trimmedStr.charAt(iPos));
      }
      Integer retInteger = null;
      try
      {
          retInteger = new Integer( tmpStrBuff.toString() );
      }
      catch (Exception e)
      {
          ;
      }
      return retInteger;
  }

  /**
   * String in BigDecimal-Object umwandeln mit Rckgabe null wen nicht lesbar
   *
   * @param numStr String numerisch
   * @return BigDecimal-Object
   */
  public static final BigDecimal str2BigDecimalStrict(final String numStr)
  {
     if (numStr == null)
     {
        return null;
     }

    BigDecimal retBigDecimal = null;

    try
    {
       // Komma in Punkt tauschen
       final String tNumStr = numStr.replace( ',' , '.' ).trim();
       retBigDecimal = new BigDecimal(tNumStr);
    }
    catch (Exception e)
    {
      ;// nicht lesbar , return null
    }

    return retBigDecimal;
  } // end method str2BigDecimalStrict

  /**
   * String in int umwandeln mit Tolerierung enthaltener nichtnumerischer Zeichen . <br/>
   * geparst wird bis zum Auftreten nichtnumerischer Zeichen.<br/>
   * Entfernung fhrender Nullen lt sich zuschalten (auskommentierter Code).<br/>
   *
   * @param numStr String numerisch
   * @return int Zahl
   */
  public static final int str2intTolerant(String numStr) {
    if (empty(numStr)) {
      return 0;
    }

    numStr = numStr.trim();

    // Entfernen fhrende Nullen
    // while ( numStr.length() > 1 && numStr.startsWith( "0" ) )
    // {
    // numStr = numStr.substring(1) ;
    // }

    // Entfernen evtl. nichtnumerische Zeichen
    for (int i = 0; i < numStr.length(); i++) {
      if (numStr.charAt(i) < '0' || numStr.charAt(i) > '9') {

        // nicht numerisches Zeichen
        numStr = numStr.substring(0, i);

        break;
      }
    }

    int retInt = 0;

    try {
      retInt = (new Integer(numStr)).intValue();
    } catch (Exception e) {
      ;
    }

    return retInt;
  } // end method str2intTolerant

  /**
   * berprfung einer Whrungs-Eingabe und Umwandlung in ein double-Wert
   *
   * @param inBetrag Betrag als String, der in ein double umgewandelt werden soll
   * @return double, der aus der Eingabe erzeugt wurde<br>
   * ist inBetrag ein NULL-String, wird Double.NaN zurckgegeben
   * @exception NumberFormatException, falls bei der Umwandlung ein Fehler auftritt
   * @see Double#NaN Double.NaN
   */
  public static double parseDouble(String inBetrag)
      throws NumberFormatException {
    double  retDouble;
    String  strTemp;

    if (inBetrag == null) {
      retDouble = Double.NaN;
    }
    else {
      strTemp = inBetrag.replace(',', '.');
      retDouble = Double.parseDouble(strTemp);
    }

    return retDouble;
  } // end parseDouble

  /**
   * berprfung einer Whrungs-Eingabe und Umwandlung in ein double, aber ohne Werfen NumberFormatException
   *
   * @param inBetrag Betrag als String, der in ein double umgewandelt werden soll
   * @return double, der aus der Eingabe erzeugt wurde<br>
   * ist inBetrag ein NULL-String, wird Double.NaN zurckgegeben
   * @see Double#NaN Double.NaN
   */
  public static double parseDoubleCatched(String inBetrag) {
    double  retDouble = 0;

    try {
      retDouble = parseDouble(inBetrag);
    } catch (NumberFormatException ne) {
      ;
    }

    return retDouble;
  } // end parseDouble

  /**
   * Umwandeln String in double-Wert
   *
   * @param parStr String numerisch
   * @return double-Zahl
   */
  public static double str2double(String parStr) {
    return parseDoubleCatched(parStr);
  }

  /**
   * Umwandeln String in double-Objekt
   *
   * @param parStr String numerisch
   * @return double-Objekt
   */
  public static Double str2Double(String parStr) {
    return new Double(parseDoubleCatched(parStr));
  }

  /**
   * null-String in Leerstring umwandeln
   *
   * @param str eventueller null-String
   * @return nicht null-String
   */
  public static final String notNull(String str) {
    if (str == null) {
      return "";
    }

    return str;
  } // end method notNull

  /**
   * String in boolean umwandeln. <br/>
   * true: true, TRUE, yes, YES, ja, JA, ok, OK, on, ON, 1 <br/>
   * false: alles andere                                <br/>
   *
   * @param str String mit logischem Wert
   * @return boolean-Wert
   */
  public static final boolean str2bool(String str) {
    boolean bReturn = false;

    if (!empty(str)) {
      str = str.toUpperCase().trim();

      if ("TRUE".equals(str) || "YES".equals(str) || "Y".equals(str) || "JA".equals(str) || "J".equals(str)
          || "OK".equals(str) || "ON".equals(str) || "1".equals(str)) {
        bReturn = true;
      }
    }

    return bReturn;
  } // end method str2bool

  /**
   * String in Boolean-Object umwandeln. <br/>
   * true: true, TRUE, yes, YES, ja, JA, ok, OK, on, ON, 1 <br/>
   * false: alles andere                                <br/>
   *
   * @param str String mit logischem Wert
   * @return boolean-Object
   */
  public static final Boolean str2boolObject(String str) {
    return str2bool(str) ? Boolean.TRUE : Boolean.FALSE ;
  } // end method str2boolObject

  /**
   * numerischen String in boolean umwandeln. <br/>
   * true:  x >= 1  <br/>
   * false: x <  1  <br/>
   *
   * @param str String numerisch
   * @return boolean-Wert
   */
  public static final boolean numStr2bool(String str) {
    boolean bReturn = false;

    if (!empty(str)) {
      int iTmp = -1;

      try {
        iTmp = str2int(str);
      } catch (Exception e) {// Log.fatal (e);
      }

      bReturn = (iTmp > 0);
    }

    return bReturn;
  } // end method str2bool

  /**
   * numerischen String in Boolean-Object umwandeln. <br/>
   * true: true, TRUE, yes, YES, ja, JA, ok, OK, on, ON <br/>
   * false: alles andere                                <br/>
   *
   * @param str String numerisch
   * @return Boolean-Object
   */
  public static final Boolean numStr2boolObject(String str) {
    return (numStr2bool(str)) ? Boolean.TRUE : Boolean.FALSE;
  } // end method str2bool

  /**
   * Entferne Hoch-Kommas
   *
   * @param value String mit Hochkommas
   * @return String ohne Hochkommas
   */
  public static final String stripHochKomma(String value) {
    if (value == null) {
      return (null);
    }

    char[]  content = new char[value.length()];

    value.getChars(0, value.length(), content, 0);

    StringBuffer  result = new StringBuffer(content.length);

    for (int i = 0; i < content.length; i++) {
      switch (content[i]) {

      case (char) 39:
        break;

      default:
        result.append(content[i]);
      }
    }

    return (result.toString());
  } // end method stripHochKomma

  /**
   * Entfernen Newline (char(10)) und Carriage Return (char(13))
   *
   * @param value String mit Newline (char(10)) und Carriage Return (char(13))
   * @return String ohne Newline (char(10)) und Carriage Return (char(13))
   */
  public static final String stripBreak(String value) {
    if (value == null) {
      return (null);
    }

    char[]  content = new char[value.length()];

    value.getChars(0, value.length(), content, 0);

    StringBuffer  result = new StringBuffer(content.length);

    for (int i = 0; i < content.length; i++) {
      switch (content[i]) {

      case (char) 10:
        break;

      case (char) 13:
        break;

      default:
        result.append(content[i]);
      }
    }

    return (result.toString());
  } // end method stripBreak

  /**
   * Entferne Carriage Return (char(13)) und encode Newline (char(10)) zu "\n"
   *
   * @param value String vorher
   * @return String encoded
   */
  public static final String encodeBreak(String value) {
    if (value == null) {
      return (null);
    }

    char[]  content = new char[value.length()];

    value.getChars(0, value.length(), content, 0);

    StringBuffer  result = new StringBuffer(content.length);

    for (int i = 0; i < content.length; i++) {
      switch (content[i]) {

      case (char) 10:
        result.append((char) 92); // steht fr Backslash
        result.append("n");       // n von \n

        break;

      case (char) 13:
        break;

      default:
        result.append(content[i]);
      }
    }

    return (result.toString());
  } // end method stripBreak

  /**
   * Entferne Carriage Return (char(13)) und encode Newline (char(10)) zu &lt;br/&gt;
   *
   * @param value String vorher
   * @return String verarbeitet
   */
  public static final String encodeBreakToTag(String value) {
    if (value == null) {
      return (null);
    }

    char[]  content = new char[value.length()];

    value.getChars(0, value.length(), content, 0);

    StringBuffer  result = new StringBuffer(content.length);

    for (int i = 0; i < content.length; i++) {
      switch (content[i]) {

      case (char) 10:
        result.append("<br/>");

        break;

      case (char) 13:
        break;

      default:
        result.append(content[i]);
      }
    }

    return (result.toString());
  } // end method stripBreak

  /**
   * Entfernen Space (32) und Tab (9)
   *
   * @param value String vorher
   * @return String verarbeitet
   */
  public static final String stripTabSpace(String value) {
    if (value == null) {
      return (null);
    }

    value = value.trim();

    char[]  content = new char[value.length()];

    value.getChars(0, value.length(), content, 0);

    StringBuffer  result = new StringBuffer(content.length);

    for (int i = 0; i < content.length; i++) {
      switch (content[i]) {

      case (char) 32:
        break;

      case (char) 9:
        break;

      default:
        result.append(content[i]);
      }
    }

    return (result.toString());
  } // end method stripTabSpace

  /**
   * ersten Buchstaben im String gro machen, Capitalize
   *
   * @param strPaLower zu bearbeitender String
   * @return kapitalisierter String
   */
  public static final String upper1(String strPaLower) {
    String  retString = null;

    if (strPaLower != null) {
      if (strPaLower.length() < 2) {
        retString = strPaLower.toUpperCase();
      }
      else {
        retString = strPaLower.substring(0, 1).toUpperCase();
        retString += strPaLower.substring(1);
      }
    }

    return retString;
  } // end method upper1

  /**
   * aktuelles Datum als String
   *
   * @return aktuelles Datum als String
   */
  public static final String currentDate() {
    return date2str(new Date());
  } // end method currentDate

  /**
   * Timestamp fr aktuellen Zeitpunkt erstellen
   *
   * @return current Timestamp as String
   */
  public static final String getTimeStamp() {
    return formatTimestamp.format(new Date(System.currentTimeMillis()));
  }

  /**
   * Datum in String umwandeln
   *
   * @param dateValue Datum
   * @return Datums-String
   */
  public static final String date2str(Date dateValue) {
    return formatDate.format(dateValue);
  } // end method date2str

  /**
   * String im Format dd.MM.yyyy in Datum umwandeln
   *
   * @param strValue String mit Datum im Format dd.MM.yyyy
   * @return Datum oder null, wenn Datum ungltig
   */
  public static final Date str2date(String strValue) {
    return parseDateCatched(strValue);
  } // end method str2date

  /**
   * String im Format dd.MM.yyyy in SQL-Datum umwandeln
   *
   * @param strValue String mit Datum im Format dd.MM.yyyy
   * @return SQL-Datum
   */
  public static final java.sql.Date str2sqldate(String strValue) {
    java.util.Date  dateTmp = str2date(strValue);

    if (dateTmp != null) {
      return new java.sql.Date(dateTmp.getTime());
    }

    return null;
  } // end method str2sqldate

  /**
   * Vergleich zweier Strings mit Nullpointer-Behandlung
   *
   * @param obj1 erster String
   * @param obj2 zweiter String
   * @return ob gleich
   */
  public static final boolean stringEqual(String obj1, String obj2) {
    if (obj1 == null && obj2 == null) {
      return true;
    }

    if (obj1 != null) {
      if (obj1.equals(obj2)) {
        return true;
      }
    }

    return false;
  } // end method stringEqual

  /**
   * Vergleich zweier Double mit Nullpointer-Behandlung
   *
   * @param obj1 erstes Double
   * @param obj2 zweites Double
   * @return ob gleich
   */
  public static final boolean doubleEqual(Double obj1, Double obj2) {
    if (obj1 == null && obj2 == null) {
      return true;
    }

    if (obj1 != null) {
      if (obj1.equals(obj2)) {
        return true;
      }
    }

    return false;
  } // end method doubleEqual

  /**
   * Vergleich zweier Objekte mit Nullpointer-Behandlung
   *
   * @param obj1 erstes Objekt
   * @param obj2 zweites Objekt
   * @return ob gleich
   */
  public static final boolean objectEqual(Object obj1, Object obj2) {
    if (obj1 == null && obj2 == null) {
      return true;
    }

    if (obj1 != null && obj2 != null) {
      if (obj1.getClass().isInstance(obj2)) {
        if (obj1.equals(obj2)) {
          return true;
        }
      }
    }

    return false;
  } // end method objectEqual

  /**
   * berprfung einer Datums-Eingabe und ggf. Umwandlung in ein Date-Objekt.
   *
   * @param inDatum Datum als String, das in ein Date-Objekt umgewandelt werden soll
   *
   * @return retDatum Datum-Objekt, das aus der Eingabe erzeugt wurde
   *
   * @exception DomainParseException: Falls der Eingabestring nicht in ein Datum
   * umgewandelt werden kann, wird eine DomainParseException geworfen
   */
  public static Date parseDate(String inDatum)
      throws ParseException {
    Date              retDatum;
    String            strDatum = "";
    String            strDay;
    String            strMonth;
    String            strYear;
    int               indexDay;
    int               indexMonth;
    GregorianCalendar cal;
    int               intError = -1;
    ParseException    e = new ParseException("Ecis: Not a valid Date", intError);

    ParsePosition     pos = new ParsePosition(0);

    if (inDatum == null) {
      retDatum = null;
    }
    else {
      strDatum = inDatum.trim();

      try {
        indexDay = strDatum.indexOf(".");
        indexMonth = strDatum.indexOf(".", indexDay + 1);
        strDay = strDatum.substring(0, indexDay);
        strMonth = strDatum.substring(indexDay + 1, indexMonth);
        strYear = strDatum.substring(indexMonth + 1);

        if (strYear.length() != 4) {
          throw e;
        }

        if ((str2int(strMonth) > 0) && (str2int(strMonth) < 13)) {
          cal = new GregorianCalendar(str2int(strYear), (str2int(strMonth) - 1), 1);

          if ((str2int(strDay) < 1) || (str2int(strDay) > cal.getActualMaximum(Calendar.DAY_OF_MONTH))) {
            throw e;
          }
        }
        else {
          throw e;
        }
      } catch (Exception ex) {
        throw e;
      }

      formatDate.setLenient(false);

      retDatum = formatDate.parse(strDatum, pos);

      if (retDatum == null) {
        throw e;
      }
    }

    return retDatum;
  } // end parseDate

  /**
   * berprfung einer Datums-Eingabe und ggf. Umwandlung in ein Date-Objekt.
   *
   * @param inDatum Datum als String, das in ein Date-Objekt umgewandelt werden soll
   * @return retDatum Datum-Objekt, das aus der Eingabe erzeugt wurde
   */
  public static Date parseDateCatched(String inDatum) {
    Date  retDatum = null;

    try {
      retDatum = parseDate(inDatum);
    } catch (ParseException pe) {// pe.printStackTrace() ;
    }
    return retDatum;
  } // end parseDateCatched

  /**
   * Datum auf Gltigkeit prfen
   *
   * @param testString
   * @return gltig (ja/nein)
   */
  public static boolean testDateString(String testString) {
    Date  testDate = parseDateCatched(testString);

    if (testDate == null) {
      return false;
    }
    return true;
  } // end method testDateString

  /**
   * Zurckgeben der iLength Zeichen eines Strings ab dem linken Rand. <br/>
   * Mit Behandlung Nullpointer und Grenberlauf. <br/>
   * Wenn String krzer als iLength, werden so viele Zeichen wie vorhanden zurckgegeben. <br/>
   *
   * @param parString Original-String
   * @param iLength Anzahl Zeichen
   * @return String links
   */
  public static String left(String parString, int iLength) {
    String  retString = null;

    if (parString != null) {
      if (parString.length() > 0) {
        iLength = Math.min(iLength, parString.length());
        retString = parString.substring(0, iLength);
      }
      else {
        retString = parString;
      }
    }

    return retString;
  } // end method left

  /**
   * list(PrintWriter)-Methode eines Properties-Objekts auf einen String umgeleitet
   *
   * @param prop Properties
   * @return list-String
   */
  public static String listToString(Properties prop) {
    String  retString = "";

    if (prop != null) {
      StringWriter  strWriter = new StringWriter();
      PrintWriter   pWriter = new PrintWriter(strWriter);

      prop.list(pWriter);

      retString = strWriter.toString();
    }

    return retString;
  } // end method listToString

  /**
   * Textdatei zeilenweise in ein String-Array einziehen . <br/>
   * Jede Zeile bildet einen String im String-Array.<br/>
   *
   * @param fileName Dateiname
   * @return String-Array
   */
  public static String[] fileToStrArr(String fileName /*,String pEncoding*/) {
    if (fileName == null) {
      return null;
    }

    return fileToStrArr(new File(fileName) /*,pEncoding*/);
  } // end method fileToStrArr

  /**
   * Textdatei zeilenweise in ein String-Array einziehen . <br/>
   * Jede Zeile bildet einen String im String-Array.<br/>
   *
   * @param inFile File-Object
   * @param pEncoding  The name of a supported
   *                   <a href="../lang/package-summary.html#charenc">character
   *                    encoding</a>
   *
   * @return String-Array
   */
  public static String[] fileToStrArr(File inFile/*, String pEncoding*/) {
    if (inFile == null) {
      return null;
    }

    try {
      // BufferedReader zum zeilenweisen Lesen
      //FileReader      defFileReader = new FileReader(inFile);
      //BufferedReader  defBufferedReader = new BufferedReader(defFileReader);

      //Feststellen, ob die Applikation auf dem HOST OS/390 luft, nur provisorisch implementiert, nicht korrekt
      boolean isHost = isHost();
      FileInputStream   fin = new FileInputStream(inFile);
      InputStreamReader isr;

      if (isHost)
      {
         //Encoding fr HOST EBCDIC
         isr = new InputStreamReader( fin , "8859_1" /*pEncoding*/ );
      }
      else
      {
         isr = new InputStreamReader( fin );
      }

      BufferedReader    defBufferedReader = new BufferedReader( isr );

      ArrayList       strArrayList = new ArrayList();
      String          readLine = null;

      while ((readLine = defBufferedReader.readLine()) != null) // bis zum Ende der Datei
      {
//Log.debug( "readLine: " +readLine );
        // readLine = readLine.trim();
        strArrayList.add(readLine);
      } // end while

      String[]  retArr = arrayListToStrArr(strArrayList);

      return retArr;
    } catch (Exception e) {
        System.err.println( e.getMessage() );
        e.printStackTrace();
      throw new RuntimeException(ExceptionUtil.getMessageAndStackTrace(e));
    }
    // return null ;
} // end method fileToStrArr

  /**
   * ArrayList mit Strings in String[] umwandeln . <br/>
   * Verwendet in {@link #fileToStrArr}.<br/>
   *
   * @param strArrayList ArrayList mit Strings
   * @return String[]
   * @exception ClassCastException wenn Element kein String
   */
  public static String[] arrayListToStrArr(ArrayList strArrayList) {
    if (strArrayList == null || strArrayList.isEmpty()) {
      return new String[0];
    }

    String[]  retArr = new String[strArrayList.size()];

    for (int i = 0; i < strArrayList.size(); i++) {
      retArr[i] = (String) strArrayList.get(i);
    }

    return retArr;
  } // end method arrayListToStrArr

  /**
   * Teilstring von String links abschneiden, wenn mglich
   *
   * @param srcStr Original-String
   * @param cutStr abzuschneidender String
   * @return abgeschnittener String
   */
  public static String cutLeft(String srcStr, String cutStr) {
    if (srcStr == null || cutStr == null) {
      return srcStr;
    }

    String  retStr = srcStr;

    if (retStr.startsWith(cutStr)) {
      retStr = retStr.substring(cutStr.length()); // cutStr abschneiden
    }

    return retStr;
  }// end method cutLeft

  /**
   * Teilstring von String rechts abschneiden, wenn mglich
   *
   * @param srcStr Original-String
   * @param cutStr abzuschneidender String
   * @return abgeschnittener String
   */
  public static String cutRight(String srcStr, String cutStr) {
    if (srcStr == null || cutStr == null) {
      return srcStr;
    }

    String  retStr = srcStr;

    if (retStr.endsWith(cutStr)) {
      retStr = retStr.substring(0, retStr.length() - cutStr.length());  // cutStr abschneiden
    }

    return retStr;
  }// end method cutRight

  /**
   * Teilstring von String links und rechts abschneiden, wenn mglich
   *
   * @param srcStr Original-String
   * @param cutLeftStr  links  abzuschneidender String
   * @param cutRightStr rechts abzuschneidender String
   * @return abgeschnittener String
   */
  public static String cutLeftRight(//--
          final String srcStr, //--
          final String cutLeftStr, //
          final String cutRightStr)
  {
    String retStr = cutLeft( srcStr , cutLeftStr );
    retStr = cutRight( retStr , cutRightStr );
    return retStr;
  }// end method cutLeftRight

  /**
   * Speichern eines String in einer Datei ohne weitere Informationen
   *
   * @param outStr   auszugebender String
   * @param fileName Name der Datei, in welcher der String gespeichert werden soll
   * @return         ob erfolgreich
   */
  public static boolean str2file(String outStr, String fileName) {
    return str2file(outStr, new File(fileName));
  } // end method str2file

  /**
   * Speichern eines String in einer Datei ohne weitere Informationen
   *
   * @param outStr   auszugebender String
   * @param outFile  Datei, in welcher der String gespeichert werden soll
   * @return         ob erfolgreich
   */
  public static boolean str2file(String outStr, File outFile) {
    boolean bReturn = false;

    try {
      if (outFile.exists()) {
        outFile.delete();
      }

      outFile.createNewFile();

      FileWriter  outFileWriter = new FileWriter(outFile);

      outFileWriter.write(outStr);
      outFileWriter.close();

      bReturn = true;
    } catch (Exception e) {
      System.err.println( e.getMessage() ); e.printStackTrace();
    }

    return bReturn;

    // TODO in StringUtil.str2file verlagern
    //File writeDir = new File(writeDirName);
    //writeDir.mkdirs();

  } // end method str2file

  /**
   * Einlesen einer Datei in einen String
   *
   * @param  fileName Name der einzulesenden Datei
   * @return String mit Datei-Inhalt
   */
  public static String file2str(String fileName) {
//System.out.println( "fileName: " + fileName );
    return file2str(new File(fileName));
  } // end method file2str

  /**
   * Einlesen einer Datei in einen String
   *
   * @param inFile einzulesende Datei
   * @return String mit Datei-Inhalt
   */
  public static String file2str(File inFile) {
//System.out.println( "inFile: " + inFile );
    String  strReturn = null;

    try {
      StringBuffer    strBuff = new StringBuffer(1024 * 16);

      // BufferedReader zum zeilenweisen Lesen
      //FileReader      defFileReader = new FileReader(inFile);
      //BufferedReader  defBufferedReader = new BufferedReader(defFileReader);

      //Feststellen, ob die Applikation auf dem HOST OS/390 luft, nur provisorisch implementiert, nicht korrekt
      boolean isHost = isHost();
      FileInputStream   fin = new FileInputStream(inFile);
      InputStreamReader isr;

      if (isHost)
      {
         //Encoding fr HOST EBCDIC
         isr = new InputStreamReader( fin , "8859_1" /*pEncoding*/ );
      }
      else
      {
         isr = new InputStreamReader( fin );
      }

      BufferedReader    defBufferedReader = new BufferedReader( isr );

      String          readLine = null;

      while ((readLine = defBufferedReader.readLine()) != null) { // bis zum Ende der Datei
//Log.debug( "readLine: " + readLine );
        strBuff.append(readLine + "\n");
      } // end while

      strReturn = strBuff.toString();
    } catch (Exception e) {
      System.err.println( e.getMessage() ); e.printStackTrace();
    }
    return strReturn;
  }// end method file2str

  /**
   * Umwandeln int-Wert in String mit Angabe Anzahl Stellen
   *
   * @param int-Wert
   * @param Anzahl Stellen
   *
   * @return numerischer String, rechtsbndig
   */
  public static String str(int iPaValue, int iPaDigits) {
    return leftPad("" + iPaValue, iPaDigits);
  }// end method str(int)

  /**
   * Umwandeln Integer-Objekt in String mit Angabe Anzahl Stellen. <br/>
   * Bei bergabe null-Pointer wird der Wert 0 angenommen
   *
   * @param iPaValue Integer-Objekt
   * @param iPaDigits Anzahl Stellen
   *
   * @return numerischer String, rechtsbndig
   */
  public static String str(Integer iPaValue, int iPaDigits) {
    int iTmp = 0;

    if (iPaValue != null) {
      iTmp = iPaValue.intValue();
    }// end if

    return leftPad("" + iTmp, iPaDigits);
  }// end method str(Integer)

  /**
   * int-Wert in String mit fhrenden Nullen umwandeln
   */
  public static String strzero(//--
    final int iPaValue, //--
    final int iPaDigits)
  {
    return strzero(String.valueOf(iPaValue), iPaDigits);
  }// end method strzero

  public static final String ZERO_20_STR = "00000000000000000000";

  /**
   * String mit fhrenden Nullen auffllen.
   */
  public static String strzero(String pStr, int pDigits)
  {
      if (pStr.length() >= pDigits)
      {
          return pStr;
      }

      final int zeroLen = pDigits - pStr.length();
      if ( zeroLen < 21 )
      {
          return ZERO_20_STR.substring(0, zeroLen).concat( pStr );
      }

      StringBuffer sb = new StringBuffer(pDigits);

      // Laut Buch ist Test auf Null schneller, TODO Test
      for (int i = zeroLen; i > 0 ; i--)
      {
          sb.append( '0' );
      }
      sb.append(pStr);
      return sb.toString();
  }// end method strzero

  public static void main(String[] args)
  {
      System.out.println( strzero(2, 2 ) );
  }

  /**
   * Zurckliefern String-Teil vor dem Bindestrich aus einem String der einen Bindestrich enthlt. <br/>
   *
   * @param strPaHyphen String mit Bindestrich
   *
   * @return String-Teil vor Bindestrich
   *
   * @deprecated umbenennen in preNumberSign und beschleunigen
   */
  public static String preHyphen(String strPaHyphen) {
    // Log.debug( strPaHyphen ) ;
    StringTokenizer tok = new StringTokenizer(strPaHyphen, "-");

    return tok.nextToken();
  }// end method preHyphen

  /**
   * Zurckliefern String-Teil nach dem Bindestrich aus einem String der einen Bindestrich enthlt. <br/>
   *
   * @param strPaHyphen String mit Bindestrich
   *
   * @return String-Teil nach Bindestrich
   * @deprecated umbenennen in postNumberSign und beschleunigen
   */
  public static String postHyphen(String strPaHyphen) {
    // Log.debug( strPaHyphen ) ;
    StringTokenizer tok = new StringTokenizer(strPaHyphen, "-");

    tok.nextToken();
    return tok.nextToken();
  }// end method preHyphen

  /**
   * Zurckliefern String-Teil vor dem Punkt aus einem String der einen Punkt enthlt. <br/>
   *
   * @param strPaHyphen String mit Punkt
   *
   * @return String-Teil vor Punkt
   */
  public static String prePoint(String strPaPoint) {
    // Log.debug( strPaPoint ) ;
    StringTokenizer tok = new StringTokenizer(strPaPoint, ".");

    return tok.nextToken();
  }// end method prePoint

  /**
   * Zurckliefern String-Teil nach dem Punkt aus einem String der einen Punkt enthlt. <br/>
   *
   * @param strPaPoint String mit Punkt
   *
   * @return String-Teil nach Punkt
   */
  public static String postPoint(String strPaPoint) {
    // Log.debug( strPaPoint ) ;
    StringTokenizer tok = new StringTokenizer(strPaPoint, ".");

    tok.nextToken();
    return tok.nextToken();
  }// end method prePoint

  /**
   * Zurckliefern String-Teil vor dem Nummern-Symbol # aus einem String der einen Nummern-Symbol # enthlt. <br/>
   *
   * @param strPaHyphen String mit Nummern-Symbol #
   *
   * @return String-Teil vor Nummern-Symbol #
   */
  public static String preNumSign(String strPaNumSign) {
    // Log.debug( strPaNumSign ) ;
    StringTokenizer tok = new StringTokenizer(strPaNumSign, "#");

    return tok.nextToken();
  }// end method preNumSign

  /**
   * Zurckliefern String-Teil nach dem Nummern-Symbol # aus einem String der einen Nummern-Symbol # enthlt. <br/>
   *
   * @param strPaNumSign String mit Nummern-Symbol #
   *
   * @return String-Teil nach Nummern-Symbol #
   */
  public static String postNumSign(String strPaNumSign) {
    // Log.debug( strPaNumSign ) ;
    StringTokenizer tok = new StringTokenizer(strPaNumSign, "#");

    tok.nextToken();
    return tok.nextToken();
  }// end method preNumSign

  /**
   * Wiederholt einen String n-Mal
   *
   * @param strPaSingle zu wiederholender String
   * @param iPaCount    Anzahl Wiederholungen
   */
  public static String repeat(String strPaSingle, int iPaCount) {
    StringBuffer sb = new StringBuffer(iPaCount);

    for (int i = 0; i < iPaCount; i++) {
      sb.append(strPaSingle);
    }
    return sb.toString();
  }// end method repeat

  /**
   * Prfen, ob ein String in einem anderen String enthalten ist mit Nullpointer-Abfangung
   *
   * @param testStr zu durchsuchender String
   * @param searchStr gesuchter String
   *
   * @return ob enthalten
   */
  public static boolean contents(String testStr, String searchStr) {
    boolean bRet = false;

    if (testStr != null) {
      bRet = (testStr.indexOf(searchStr) > -1);
    }// end if
    return bRet;
  }// end method contents

  /**
   * Prfen, ob ein String in einem anderen String enthalten ist mit Nullpointer-Abfangung
   * unter ignorieren Gross-/Kleinschreibung
   *
   * @param testStr zu durchsuchender String
   * @param searchStr gesuchter String
   *
   * @return ob enthalten
   */
  public static boolean contentsIgnoreCase(String testStr, String searchStr) {
    boolean bRet = false;

    if (testStr != null && searchStr != null) {
      bRet = (testStr.toUpperCase().indexOf(searchStr.toLowerCase()) > -1);
    }// end if
    return bRet;
  }// end method contentsIgnoreCase

  /**
   * Prfen, ob ein Zeichen in einem anderen String enthalten ist mit Nullpointer-Abfangung
   *
   * @param testStr zu durchsuchender String
   * @param searchChr gesuchtes Zeichen
   *
   * @return ob enthalten
   */
  public static boolean contents(String testStr, char searchChr) {
    boolean bRet = false;

    if (testStr != null) {
      bRet = (testStr.indexOf(searchChr) > -1);
    }// end if
    return bRet;
  }// end method contents

  /**
   * numerischen String (dezimal) in Hexadezimal-String umwandeln
   *
   * @param paStr umzuwandelnder String
   *
   * @return Hex-String
   */
  public static String toHex(String paStr) {
    int iTmp = str2int(paStr);

    return Integer.toHexString(iTmp);
  }// end method toHex

  /**
   * numerischen String (dezimal) in Hexadezimal-String mit zwei Stellen umwandeln
   *
   * @param paStr umzuwandelnder String
   *
   * @return Hex-String, 2 Digits
   */
  public static String toHexTwoDigits(String paStr) {
    String retStr = toHex(paStr);

    if (retStr.length() < 2) {
      return "0" + retStr;
    }
    return retStr;
  }// end method toHex

  /**
   * String auf Start mit anderem String pr&uuml;fen
   *
   * @param paStr       zu pr&uuml;fender String
   * @param paStrStart  Start-String
   *
   * @return ja, wenn paStr mit paStrStart beginnt
   */
  public static boolean startsWith(String paStr, String paStrStart) {
    if (paStr != null && paStrStart != null) {
      return paStr.startsWith(paStrStart);

    }
    return false;
  } // end method startsWith

  /**
   * String auf Start mit anderem String pr&uuml;fen
   *
   * @param paStr       zu pr&uuml;fender String
   * @param paStrStart  Start-String
   *
   * @return ja, wenn paStr mit paStrStart beginnt
   */
  public static boolean startsWithIgnoreCase(String paStr, String paStrStart) {
    if (paStr != null && paStrStart != null) {
      return paStr.toLowerCase().startsWith(paStrStart.toLowerCase());

    }
    return false;
  } // end method startsWithIgnoreCase

  /**
   * String auf Enden mit anderem String pr&uuml;fen
   *
   * @param paStr zu pr&uuml;fender String
   * @param paStrEnd Ende-String
   *
   * @return ja, wenn paStr mit paStrEnd endet
   */
  public static boolean endsWith(String paStr, String paStrEnd) {
    if (paStr != null && paStrEnd != null) {
      return paStr.endsWith(paStrEnd);

    }
    return false;
  } // end method endsWith

  /**
   * String auf Enden mit anderem String pr&uuml;fen
   *
   * @param paStr zu pr&uuml;fender String
   * @param paStrEnd Ende-String
   *
   * @return ja, wenn paStr mit paStrEnd endet
   */
  public static boolean endsWithIgnoreCase(String paStr, String paStrEnd) {
    if (paStr != null && paStrEnd != null) {
      return paStr.toLowerCase().endsWith(paStrEnd.toLowerCase());

    }
    return false;
  } // end method endsWithIgnoreCase

  /**
   * Zurckgeben des Klassen-Namens ohne Package
   * des bergebenen Objektes.
   */
  public static String getShortClassName(Object pObj)
  {
      Class clazz = pObj.getClass();
      String str = clazz.getName();

      if (str.indexOf('.') < 0)
      {
          return str;
      }
      return str.substring(str.lastIndexOf('.') + 1);
  }

  /**
   * verlangte Anzahl Leerzeichen liefern
   *
   * @param pCount
   * @return
   */
  public static String spaces(int pCount)
  {
      StringBuffer strBuff = new StringBuffer(pCount);

      for (int i = 0; i < pCount; i++)
      {
          strBuff.append(' ');
      }
      return strBuff.toString();
  }

  /**
   * Prfen, ob der String suchStr im String-Array strArr enthalten ist
   *
   * @param suchStr String gesuchter
   * @param strArr String[] zu durchsuchendes
   * @return enthalten (ja/nein)
   */
  public static boolean isInArray(String suchStr, String[] strArr)
  {
    for (int i = 0; i < strArr.length; i++)
    {
      if (stringEqual(suchStr, strArr[i]))
      {
        return true;
      }
    }

    return false;
  } // end method isInArray

  /**
   * Hinzufgen eines Objektes zu einem Objekt-Array
   *
   * @param originalArr vorheriges Array
   * @param addTopic hinzuzufgende Methode
   * @return vergrssertes Array
   */
  public static Object[] arrAdd(final Object[] originalArr, final Object addTopic)
  {
    Object[] newArr;
    if (originalArr == null)
    {
       newArr =
         (Object[]) java.lang.reflect.Array.newInstance(
                         addTopic.getClass(), 1);

       newArr[0] = addTopic;
    }
    else
    {
       newArr =
         (Object[]) java.lang.reflect.Array.newInstance(
                         addTopic.getClass(), originalArr.length + 1);

       //for (int i = 0; i < originalArr.length; i++)
       //{
       //  newArr[i] = originalArr[i];
       //}
       System.arraycopy( originalArr , 0 , newArr, 0 , originalArr.length );

       newArr[newArr.length - 1] = addTopic;
    }

    return newArr;
  }

  /**
   * Hinzufgen eines Objektes zu einem Objekt-Array
   * an der gewnschten Position.
   *
   * @param originalArr vorheriges Array
   * @param addTopic hinzuzufgende Methode
   * @return vergrssertes Array
   */
  public static Object[] arrAdd( //-
      final Object[] originalArr, //-
      final int pIndex, //-
      final Object addTopic)
  {
     //@todo: mal auf Algorithmus ohne List umstellen
     List list = Arrays.asList( originalArr );
     list.add( pIndex , addTopic );
     return list.toArray( originalArr );
  }

  /**
   * Lschen eines Objektes aus einem Objekt-Array
   * an der gewnschten Position.
   *
   * @param originalArr vorheriges Array
   * @param addTopic hinzuzufgende Methode
   * @return vergrssertes Array
   */
  public static Object[] arrRemove( //-
      final Object[] originalArr, //-
      final int pIndex)
  {
     //@todo: mal auf Algorithmus ohne List umstellen
     List list = Arrays.asList( originalArr );
     list.remove( pIndex );
     return list.toArray( originalArr );
  }

  /**
   * Hinzufgen einer Methode zu einem Methoden-Array
   *
   * @param originalArr vorheriges Array
   * @param addTopic hinzuzufgende Methode
   * @return vergrssertes Array
   */
//  public static Method[] arrAdd(Method[] originalArr, Method addTopic)
//  {
//    if (originalArr == null)
//    {
//      originalArr = new Method[0];
//    }
//
//    Method[]  newArr = new Method[originalArr.length + 1];
//
//    for (int i = 0; i < originalArr.length; i++)
//    {
//      newArr[i] = originalArr[i];
//    }
//
//    newArr[newArr.length - 1] = addTopic;
//
//    return newArr;
//  }

  /**
   * Anhngen eines String-Arrays an ein zweidimensionales String-Array
   *
   * @param originalArr Original-String[][]
   * @param addStrArr anzufgendendes String[]
   * @return Ergebnis-String[][]
   */
  public static String[][] arrAdd(String[][] originalArr, String[] addStrArr)
  {
    if (originalArr == null)
    {
      originalArr = new String[0][0];
    }

    String[][]  newArr = new String[originalArr.length + 1][0];

    for (int i = 0; i < originalArr.length; i++)
    {
      newArr[i] = originalArr[i];
    }

    newArr[newArr.length - 1] = addStrArr;

    return newArr;
  } // end method arrAdd

  /**
   * Entfernen aller leeren Zeilen (Eintrge) am
   * String-Array-Ende.
   *
   * @param pStrArr
   * @return trimmed String-Array
   */
  public static String[] arrTrim(
          final String[] pStrArr)
  {
    if ( pStrArr == null )
    {
      return null;
    }
    int endPos = pStrArr.length - 1;
    while ( endPos > 0 //--
            && empty( pStrArr[ endPos ] ) )
    {
      endPos--;
    }
    final String[] retStrArr = new String[ endPos + 1 ];
    System.arraycopy( pStrArr , 0, retStrArr, 0, endPos + 1 );
    return retStrArr;
  }

  /**
   * Entfernen aller leeren Zeilen (Eintrge) am
   * String-Array-Ende.
   *
   * @param pStrArr
   * @return trimmed String-Array
   */
  public static String[] arrTopTrim(
          final String[] pStrArr)
  {
    if ( pStrArr == null )
    {
      return null;
    }
    int anfPos = 0;
    while ( anfPos < pStrArr.length -1 //--
            && empty( pStrArr[ anfPos ] ) )
    {
      anfPos++;
    }
    final int retStrSize = pStrArr.length - anfPos;
    final String[] retStrArr = new String[ retStrSize ];
    System.arraycopy( pStrArr , anfPos, retStrArr, 0, retStrSize );
    return retStrArr;
  }

  /**
   * Anfgen eines Strings an ein String-Array
   *
   * @param originalArr Original-String[]
   * @param addStr anzuhngener String
   * @return Ergebnis-String[]
   */
  public static String[] arrAdd(String[] originalArr, String addStr)
  {
    if (originalArr == null)
    {
      return new String[]{ addStr };
    }

    String[]  newArr = new String[originalArr.length + 1];

    for (int i = 0; i < originalArr.length; i++)
    {
      newArr[i] = originalArr[i];
    }

    newArr[newArr.length - 1] = addStr;

    return newArr;
  } // end method arrAdd

  /**
   * Anfgen eines String-Array an ein String-Array
   *
   * @param originalArr Original-String[]
   * @param addStr anzuhngener String
   * @return Ergebnis-String[]
   */
  public static String[] arrAdd(String[] originalArr, String[] addStrArr)
  {
    if (originalArr == null)
    {
      originalArr = new String[0];
    }
    if ( addStrArr == null )
    {
        return originalArr; // evtl. clonen
    }

    String[]  newArr = new String[originalArr.length + addStrArr.length];

    for (int i = 0; i < originalArr.length; i++)
    {
      newArr[i] = originalArr[i];
    }
    int j = originalArr.length;
    for (int i = 0; i < addStrArr.length; i++)
    {
      newArr[j] = addStrArr[i];
      j++;
    }
    return newArr;
  } // end method arrAdd

  /**
   * Einfgen eines Strings am Anfang eines String-Arrays . <br/>
   * verwendet in {@link de.cnc.singletons.BerechtigungenSingleton}.<br/>
   *
   * @param addStr einzufgender String
   * @param originalArr Original-String[]
   * @return Ergebnis-String[]
   */
  public static String[] arrAdd(String addStr, String[] originalArr)
  {
    if (originalArr == null)
    {
      originalArr = new String[0];
    }

    String[]  newArr = new String[originalArr.length + 1];

    newArr[0] = addStr;

    for (int i = 0; i < originalArr.length; i++)
    {
      newArr[i + 1] = originalArr[i];
    }

    return newArr;
  } // end method arrAdd

  /**
   * Durchsucht ein zweidimensionales String-Array nach dem Auftreten des Suchstrings in Spalte suchIndex
   * und gibt dafr den String in Spalte getIndex zurck. <br/>
   * Bei nicht Finden wird null zurckgegeben.<br/>
   * verwendet in {@link de.cnc.singletons.BerechtigungenSingleton}.<br/>
   *
   * @param suchArr      abzusuchendes String-Array
   * @param suchIndex    Spalte (2. Index) in der gesucht werden soll
   * @param suchString   gesuchter String
   * @param getIndex     Spalte (2. Index) aus welcher der Ergebnis-String entnommen werden soll
   * @return             Ergebnis-String oder null, wenn erfolglos
   */
  public static String findForKey(String[][] suchArr, int suchIndex, String suchString, int getIndex)
  {
    String  retString = null;

    for (int i = 0; i < suchArr.length; i++)
    {
      if (StringUtil.stringEqual(suchArr[i][suchIndex], suchString))
      {
        retString = suchArr[i][getIndex];

        break;
      }
    }

    return retString;
  } // end method findForKey

  /**
   * Formatieren des bergebenen Objektes entsperechend der bergebenen Maske.
   * Zur Zeit untersttzte Maskierungen:
   *
   * upper          - Umwandeln in Grossbuchstaben
   * int            - Umwandeln in Integer
   * stripHochKomma - Entfernen HochKomma
   *
   * Kombination mehrerer Optionen ber kommaseparierte Liste.
   *
   * @param pUnmaskedObj unmaskiertes Objekt
   * @param pMask Maske (Formatangabe)
   * @return maskiertes Objekt
   */
  public static Object mask(
      final Object pUnmaskedObj,
      final String pMask)
  {
     if ( empty( pMask ) )
     {
        return pUnmaskedObj;
     }

     if (pUnmaskedObj == null)
     {
        //dieser Block muss evtl. weg , zum Beispiel fr nullToEmpty
        return null;
     }

     Object retObj = pUnmaskedObj;

     StringTokenizer toknzr = new StringTokenizer( pMask , "," );
     while ( toknzr.hasMoreTokens() )
     {
        String maskTok = toknzr.nextToken();
        if ("upper".equals( maskTok ))
        {
           retObj = upper( "" + retObj );
        }
        else if ("stripHochKomma".equals( maskTok ))
        {
           retObj = stripHochKomma( "" + retObj );
        }
        else if ("int".equals( maskTok ))
        {
           retObj = str2IntegerStrict( "" + retObj );
        }
        else if ("long".equals( maskTok ))
        {
           retObj = str2LongStrict( "" + retObj );
        }
        else
        {
           throw new IllegalArgumentException( "" + maskTok );
        }
     }
     return retObj;
  }

  /**
   * String trimmen mit NullPointer-Abfangung
   */
  public static String trim(String pStr)
  {
      if (pStr == null)
      {
          return null;
      }
      return pStr.trim();
  }

  /**
   * String rechts trimmen mit NullPointer-Abfangung
   */
  public static String rTrim(
          final String pStr)
  {
      if (pStr == null)
      {
          return null;
      }

      /**
       * rckwrts ber alle WhiteSpace laufen.
       */
      int lastWhiteSpacePos = pStr.length();
      while ( lastWhiteSpacePos > 0 //--
              && Character.isWhitespace( pStr.charAt( lastWhiteSpacePos - 1 ) ) )
      {
          lastWhiteSpacePos--;
      }
      return pStr.substring(0, lastWhiteSpacePos );
  }

  /**
   * Clonen eines String-Array.
   * @param pStrArr
   * @return
   */
  public static String[] cloneStrArr(
          final String[] pStrArr)
  {
      if ( pStrArr == null )
      {
          return null;
      }
      final String[] retStrArr = new String[ pStrArr.length ];
      for (int i = 0; i < pStrArr.length; i++)
      {
        final String str = pStrArr[i];
        retStrArr[ i ] = str;
      }
      return retStrArr;
  }

  /**
   * Clonen und Rechts Trimmen eines String-Array.
   * Alle Zeilen des bergebenen Arrays rechts trimmen.
   *
   * @param pStrArr
     * @return clone zum Vermeiden von ungewollten Seiten-Effekten
   */
  public static String[] rTrimStrArr(
          final String[] pStrArr)
  {
      if ( pStrArr == null )
      {
          return null;
      }
      final String[] retStrArr = new String[ pStrArr.length ];
      for (int i = 0; i < pStrArr.length; i++)
      {
        final String str = pStrArr[i];
        retStrArr[ i ] = rTrim( str );
      }
      return retStrArr;
  }

  /**
   * Zerlegen eines Strings in Token . <br/>
   * Ersatz fr StringTokenizer, da dieser Delimiter, die aus mehreren Zeichen
   * bestehen, als Liste alternativer Zeichen versteht.
   * Diese Methode sieht den Mehrzeichen-Delimiter als Trennstring an.
   *
   * @param strPaValue zu zerteilender String
   * @param strPaDelim Trenn-String
   * @param bPaInclDelim ob Trenn-Strings mit in die Ausgabe aufgenommen werden sollen
   * @param pInclEmptyToken ob leere Tokens mit zurckgegeben werden sollen
   * @return Arraylist mit den String-Teilen (Token) oder null
   */
  public static String[] tokenize(
     String strPaValue ,
     String strPaDelim ,
     boolean bPaInclDelim ,
     boolean pInclEmptyToken )
  {
    if ( strPaValue == null ) {
      return null ;
    }

    ArrayList arrList = new ArrayList() ;
    if ( strPaDelim == null ) {
      arrList.add( strPaValue ) ;
      return (String[]) arrList.toArray( new String[1] ) ;
    }

    int iPos = 0 ;

    if ( strPaValue.startsWith( strPaDelim , iPos ) )
    {
       //Beginn mit Delimiter
       if ( bPaInclDelim ) {
         arrList.add( strPaDelim ) ;
       }
       iPos += strPaDelim.length() ;
       if (pInclEmptyToken)
       {
          //empty Token, wenn der String mit dem Trennzeichen begonnen hat
          arrList.add( "" ) ;
       }
    }

    while ( iPos < strPaValue.length() )
    {
      int iDelimPos = strPaValue.indexOf( strPaDelim , iPos ) ; // evtl. - 1 ;
      if ( iDelimPos < 0 ) {
        //Delimiter nicht mehr enthalten
        //Rckgabe des Restes vom String
        arrList.add( strPaValue.substring( iPos ) ) ;
        iPos = strPaValue.length() ; //ans Ende setzen fr evtl. weitere Auswertungen
        break ;
      }
      else
      {
         //Beginn mit Token (Teilstck)
         String strTmp = strPaValue.substring( iPos , iDelimPos ) ;
         arrList.add( strTmp ) ;
         iPos += strTmp.length() ; // = iEndPos ; // oder += iEndPos

         //Delimiter berspringen
         while ( iPos < strPaValue.length()
              && strPaValue.startsWith( strPaDelim , iPos ) ) //ab Position
         {
           //Beginn mit Delimiter
           if ( bPaInclDelim )
           {
             arrList.add( strPaDelim ) ;
           }
           iPos += strPaDelim.length() ;
           if ( pInclEmptyToken
             && strPaValue.startsWith( strPaDelim , iPos ) ) //ab Position
           {
              //es folgt sofort wieder ein Trenn-String
              //empty Token
              arrList.add( "" ) ;
           }
         }
      }//end if
    }//end while
    return (String[]) arrList.toArray( new String[ arrList.size() ] ) ;
  }//end method tokenize

   /**
    * toString(), null-sicher
    *
    * @param pObj
    * @return
    */
   public static String toStr(Object pObj)
   {
      if (pObj == null)
      {
         return null;
      }
      return pObj.toString();
   }

   /**
    * Vergrert oder verkleinert das bergebene String-Array auf
    * die gewnschte Gre iPaLength. Bei zu wenig Elementen
    * wird das String-Array mit Leer-Strings aufgefllt. Bei
    * zu viel Elementen wird das String-Array ohne Rcksicht
    * auf verlorenene Elemente entsprechend gekrzt . <br>
    *
    * @param strArrPa auszurichtendes String-Array
    * @param iPaLength gewnschte Lnge
    * @return ausgerichtetes String-Array
    */
   public static String[] padStrArr(String[] strArrPa, int iPaLength)
   {
      String[] retStrArr = new String[iPaLength];

      for (int i = 0; i < iPaLength; i++)
      {
         if (strArrPa.length > i)
         {
            retStrArr[i] = strArrPa[i];
         }
         else
         {
            retStrArr[i] = "";
         }
      }
      return retStrArr;
   } // end method padStrArr

   /**
    * Erzeuge DB2-Timestamp-String
    *
    * @param pTimestamp
    * @return
    */
//   public static String toDB2TimestampStr(Timestamp pTimestamp)
//   {
//      return strzero( pTimestamp.getYear() + 1900, 4) + "-"
//             + strzero( pTimestamp.getMonth() + 1 , 2 ) + "-"
//             + strzero( pTimestamp.getDate() , 2 ) + "-"
//             + strzero( pTimestamp.getHours() , 2 ) + "."
//             + strzero( pTimestamp.getMinutes() , 2 ) + "."
//             + strzero( pTimestamp.getSeconds() , 2 ) + "."
//             + strzero( pTimestamp.getNanos() / 1000 , 6 )
//             ;
//   }

   public static boolean isHost()
   {
      //Feststellen, ob die Applikation auf dem HOST OS/390 luft, nur provisorisch implementiert, nicht korrekt
      return ! System.getProperty( "os.name" ).toUpperCase().startsWith( "WIN" );
   }

   /**
    * Umwandlung Byte-Array in String unter Beachtung der Plattform OS/390
    *
    * @param pByteArr
    * @return
    */
   public static String toStr( byte[] pByteArr)
   {
      //Feststellen, ob die Applikation auf dem HOST OS/390 luft, nur provisorisch implementiert, nicht korrekt
      boolean isHost = isHost();
      String retStr;
      if (isHost)
      {
         //Encoding fr HOST EBCDIC
         try
         {
            retStr = new String( pByteArr , "8859_1" /*pEncoding*/ );
         }
         catch (UnsupportedEncodingException e)
         {
            System.err.println( e.getMessage() ); e.printStackTrace();
            throw new RuntimeException( e.getMessage() );
         }
      }
      else
      {
         retStr = new String( pByteArr );
      }
      return retStr;
   }

   /**
    * Rckgabe String oder & n b s p ; wenn null oder Leerstring
    * @param pObj
    * @return
    */
   public static String emptyToNbsp( final Object pObj )
   {
      if (pObj == null)
      {
         return "&nbsp;";
      }
      if ( ! ( pObj instanceof String ) )
      {
         return toStr( pObj );
      }
      final String tStr = (String) pObj ;
      if ( empty( tStr ) )
      {
         return "&nbsp;";
      }
      return tStr;
   }

   /**
    * regulrer Ausdruck nachempunden
    * als Jokerzeichen dient '?' (CP/M)
    *
    * @param pStrValue
    * @param pStrMask
    * @return
    */
   public static boolean match(//-
      final String pStrValue, //-
      final String pStrMask)
   {
      if (pStrValue== null || pStrMask == null)
      {
         return false;
      }
      if (pStrValue.length() != pStrMask.length() )
      {
         return false;
      }
      for (int i = 0; i < pStrMask.length(); i++)
      {
         if ( pStrMask.charAt( i ) == '?')
         {
            continue;
         }
         if ( pStrMask.charAt( i ) != pStrValue.charAt( i ))
         {
            return false;
         }
      }
      return true;
   }

   /**
    * Check the specified String for Digits
    * or other characters.
    *
    * @param pStr String to check
    * @return whether only digits or not
    */
   public static boolean onlyDigits(final String pStr)
   {
       if (pStr == null)
       {
           return false;
       }
       for (int i = 0; i < pStr.length(); i++)
       {
           if ( ! Character.isDigit( pStr.charAt(i) ) )
           {
               return false;
           }
       }
       return true;
   }

   /**
    * Formatierung {@link Date} oder
    * {@link java.sql.Timestamp}.
    *
    * @param pObject
    * @return
    */
    public static String toFullStr(Date pDate) {
       if (pDate == null){
        return null;
       }
       if (pDate instanceof Timestamp){
          // Konvertieren Nanos in Millisekunden
          // siehe Javadoc of java.sql.Timestamp
           pDate = new Date( ((Timestamp) pDate ).getTime()
                   + ( ((Timestamp) pDate ).getNanos() / 1000000 ) );
       }
       //Apache-Commons: final FastDateFormat dateFormat = FastDateFormat.getInstance( "yyyy-MM-dd HH:mm:ss.SSS" );
       final DateFormat dateFormat = new SimpleDateFormat( "yyyy-MM-dd HH:mm:ss.SSS" );
       final String dateStr = dateFormat.format( pDate );
       //TimeZone timeZone = pDate.g; TODO Ausgabe TimeZone
        return dateStr;
    }

    /**
     * Entfernen umliegende " oder ' vom bergebenen String.
     *
     * @param pSource
     * @return
     * @author Heiner Kcker
     * @since Aug 2, 2005
     */
    public static String stripStrEnc(final String pStr)
    {
        if ( pStr == null )
        {
            return null;
        }
        if ( pStr.startsWith( "\"" ) )
        {
            return cutLeftRight( pStr, "\"", "\"");
        }
        if ( pStr.startsWith( "'" ) )
        {
            return cutLeftRight( pStr, "'", "'");
        }
        return pStr; // evtl. Exception werfen
    }

    /**
     * Abfrage eines Zeichen von einem String mit Abfangung
     * Exception bei falscher Position.
     *
     * @param pStr abzufragender String
     * @param pPos Zeichenposition
     * @return abgefragtes Zeichen oder chr(0), wenn Position nicht ok
     */
    public static char charAt(String pStr , int pPos)
    {
        try
        {
            return pStr.charAt(pPos);
        }
        catch (Exception e)
        {
            return (char) 0;
        }
    }

    /**
     *
     *
     * @param pStr
     * @return
     * @author Heiner Kcker
     * @since Aug 5, 2005
     */
    public static String encodeToJavaStrLiteral( //--
            final String pStr )
    {
        if ( pStr == null )
        {
            return null;
        }
        final StringBuffer strBuff = new StringBuffer( (int) ( pStr.length() * 1.2f ) );
        synchronized (strBuff )
        {
            strBuff.append( '"' );
            for (int i = 0; i < pStr.length(); i++)
            {
                // TODO Einbau Oktal-Encoding
                final char chr = pStr.charAt( i );
                switch (chr)
                {
                    case '\b' :
                        strBuff.append( "\\b" );
                        break;

                    case '\t' :
                        strBuff.append( "\\t" );
                        break;

                    case '\n' :
                        strBuff.append( "\\n" );
                        break;

                    case '\f' :
                        strBuff.append( "\\f" );
                        break;

                    case '\r' :
                        strBuff.append( "\\r" );
                        break;

                    case '\"' :
                        strBuff.append( "\\\"" );
                        break;

                    case '\'' :
                        strBuff.append( "\\'" );
                        break;

                    case '\\' :
                        strBuff.append( "\\\\" );
                        break;

                    default :
                        strBuff.append( chr );
                        break;
                }
            }
            strBuff.append( '"' );
        }
        return strBuff.toString();
    }

    /**
     *
     *
     * @param pStr
     * @return
     * @author Heiner Kcker
     * @since Aug 5, 2005
     */
    public static String decodeFromJavaStrLiteral( //--
            final String pStr )
    {
        if ( pStr == null )
        {
            return null;
        }
        if ( pStr.length() < 2 )
        {
            throw new RuntimeException( "minimum length 2 expected: " + pStr );
        }
        if ( charAt( pStr , 0 ) != '\"' )
        {
            throw new RuntimeException( "start with quote \" expected: " + pStr );
        }
        if ( charAt( pStr , pStr.length() - 1 ) != '\"' )
        {
            throw new RuntimeException( "end with quote \" expected: " + pStr );
        }
        final StringBuffer strBuff = new StringBuffer( pStr.length() );
        synchronized (strBuff )
        {
            for (int i = 1; i < pStr.length() - 1; i++)
            {
                if ( i < pStr.length() - 1 //--
                        && pStr.charAt( i ) == '\\' )
                {
                    final char chr = pStr.charAt( i + 1 );
                    strBuff.append( getDecodedChr(chr) );
                    i++;
                }
                else
                {
                    strBuff.append( pStr.charAt( i ) );
                }
            }
        }
        return strBuff.toString();
    }

    /**
     * Get decoded char for given encoded second char.
     *
     * @param strBuff
     * @param chr
     * @author Heiner Kcker
     * @since Aug 5, 2005
     */
    public static char getDecodedChr(//--
            final char chr)
    {
        // TODO Einbau Oktal-Encoding
        switch (chr)
        {
            case 'b' :
                return '\b';
                //break;

            case 't' :
                return '\t';
                //break;

            case 'n' :
                return '\n';
                //break;

            case 'f' :
                return '\f';
                //break;

            case 'r' :
                return '\r';
                //break;

            case '"' :
                return '\"';
                //break;

            case '\'' :
                return '\'';
                //break;

            case '\\' :
                return '\\';
                //break;

            default :
                //return chr;
                //break;
                throw new RuntimeException( "illegal encoded character \\" + chr );
        }
    }

    /**
     * bergebenen String am Anfang jedes Elementes
     * des bergebenen String-Arrays Einfgen.
     *
     * @param pStrArr
     * @param pString
     * @return clone zum Vermeiden von ungewollten Seiten-Effekten
     */
    public static final String[] insertForAllLines(
            final String[] pStrArr,
            final String pString)
    {
        if ( pStrArr == null )
        {
            return null;
        }
        final String[] retStrArr = new String[ pStrArr.length ];
        for (int i = 0; i < pStrArr.length; i++)
        {
            final String str = pStrArr[i];
            if ( str == null )
            {
                retStrArr[ i ] = pString;
            }
            else
            {
                retStrArr[ i ] = pString + str;
            }
        }
        return retStrArr;
    }

    /**
     * Einen in einem String-Array verpackten Text so
     * weit wie mglich nach links verschieben, so
     * dass die Struktur erhalten bleibt.
     *
     * @param pStrArr
     * @return clone zum Vermeiden von ungewollten Seiten-Effekten
     * @exception RuntimeException wenn ein verbotenes weisses Zeichen auftaucht (alles ausser Leerzeichen)
     */
    public static final String[] removeIndentation(
            final String[] pStrArr)
    {
        if ( pStrArr == null )
        {
            return null;
        }
        // Ermitteln der maximalen Einrckung
        int indentSize = -1;
        for (int i = 0; i < pStrArr.length; i++)
        {
            final String str = pStrArr[i];
            if ( empty( str ) )
            {
                continue;
            }

            final int leftSpace = getLeftSpaceCount( str );

            if ( indentSize < 0 )
                // indentSize wurde noch nie gesetzt
            {
                indentSize = leftSpace;
            }
            else
            {
                indentSize = Math.min( indentSize , leftSpace );
            }
            if ( leftSpace == 0 )
                // es gibt keinen Platz am Rand
            {
                break;
            }
        }
        // Umkopieren der Zeilen
        final String[] retStrArr = new String[ pStrArr.length ];
        for (int i = 0; i < pStrArr.length; i++)
        {
            final String str = pStrArr[i];
            if ( str == null )
            {
                retStrArr[ i ] = null;
            }
            else if ( str.length() > indentSize )
            {
                retStrArr[ i ] = str.substring( indentSize );
            }
            else
            {
                retStrArr[ i ] = str;
            }
        }
        return retStrArr;

    }

    /**
     * Ermitteln der Anzahl Leerzeichen (Space) links am bergebenen
     * String. Falls der gesamte String aus weissen Zeichen besteht,
     * wird numerisch Null zurckgegeben.
     *
     * @param pStr
     * @return Anzahl Leerzeichen am linken Rand des Strings.
     * @exception RuntimeException wenn ein verbotenes weisses Zeichen auftaucht (alles ausser Leerzeichen)
     */
    private static int getLeftSpaceCount(
            final String pStr)
    {
        if ( empty( pStr ) )
        {
            return 0;
        }
        for (int i = 0; i < pStr.length(); i++)
        {
            final char runChr = pStr.charAt( i );
            if ( Character.isWhitespace( runChr ) //--
                    && runChr != ' ' )
            {
                throw new RuntimeException( "forbidden white char #" + ( (int) runChr ) + " at pos " + i );
            }
            if ( runChr != ' ' )
            {
                return i;
            }
        }
        // keine weissen Zeichen am linken Rand
        return 0;
    }


    /**
     * bergebenen String am Ende jedes Elementes
     * des bergebenen String-Arrays anhngen.
     *
     * @param pStrArr
     * @param pString
     * @return clone zum Vermeiden von ungewollten Seiten-Effekten
     */
    public static final String[] arrAllLinesAdd(
            final String[] pStrArr,
            final String pString)
    {
        if ( pStrArr == null )
        {
            return null;
        }
        final String[] retStrArr = new String[ pStrArr.length ];
        for (int i = 0; i < pStrArr.length; i++)
        {
            final String str = pStrArr[i];
            if ( str == null )
            {
                retStrArr[ i ] = pString;
            }
            else
            {
                retStrArr[ i ] = str + pString;
            }
        }
        return retStrArr;
    }

    /**
     * Jedes Elementes des bergebenen String-Arrays
     * auf die gewnschte Lnge ausrichten.
     *
     * @param pStrArr auszurichtendes String-Array
     * @param pLen gewnschte Lnge fr jedes Element
     * @return clone zum Vermeiden von ungewollten Seiten-Effekten
     */
    public static final String[] arrAllLinesPad(
            final String[] pStrArr,
            final int pLen)
    {
        if ( pStrArr == null )
        {
            return null;
        }
        final String[] retStrArr = new String[ pStrArr.length ];
        for (int i = 0; i < pStrArr.length; i++)
        {
            final String str = pStrArr[i];
            if ( str == null )
            {
                retStrArr[ i ] = pad( "" , pLen );
            }
            else
            {
                retStrArr[ i ] = pad( str , pLen );
            }
        }
        return retStrArr;
    }

    /**
     * Ermittele den lngsten String im
     * bergebenen String-Array.
     *
     * @param pStrArr
     * @return
     */
    public static final int getArrMaxStrLen(
            final String[] pStrArr)
    {
        if ( pStrArr == null )
        {
            return 0;
        }
        int maxLen = 0;
        for (int i = 0; i < pStrArr.length; i++)
        {
            final String str = pStrArr[i];
            if ( str != null )
            {
                maxLen = Math.max( maxLen , str.length() );
            }
        }
        return maxLen;
    }

    /**
     * Erzeugen eines string-Arrays in der gewnschten Lnge
     * und befllen jedes Eintrages mit dem bergebenen String.
     *
     * @param pLen gewnschte Lge des zu erzeugenden String-Arrays
     * @param pStr auf jeden Platz des String-Arrays zu kopierender String
     * @return erzeugtes String-Array
     */
    public static String[] fillStrArr(
            final int pLen,
            final String pStr)
    {
        final String[] retStrArr = new String[ pLen ];
        Arrays.fill( retStrArr , pStr );
        return retStrArr;
    }

} // end class StringUtil
