package de.heinerkuecker.xmlbuilder;

/**
 * Util class for operations like string operations
 * on byte arrays to avoid problems with locale
 * encoding and memory blowing.
 *
 * @author Heiner K&uuml;cker
 */
public class Str8
{
    /**
     * Concat the specified byte arrays.
     *
     * @param pFirstArr first array
     * @param pAddArr   second array
     * @return result byte[]
     */
    public static byte[] add(//--
    final byte[] pFirstArr, //-
    final byte[] pAddArr)
    {
        if (pAddArr == null)
        {
            //eventuelly cone
            return pFirstArr;
        }
        final byte[] retArr = new byte[pFirstArr.length + pAddArr.length];
        System.arraycopy(pFirstArr, 0, retArr, 0, pFirstArr.length);
        System.arraycopy(pAddArr, 0, retArr, pFirstArr.length, pAddArr.length);
        return retArr;
    }
    /**
     *
     * @param pArr
     * @param pSearchArr
     * @return
     */
    public static final boolean startsWith(//--
    final byte[] pArr, //--
    final byte[] pSearchArr)
    {
//System.out.println("startsWith");
        final int searArrLen = pSearchArr.length;
        if (searArrLen < 1)
        {
            return true;
        }
        if (pArr.length < searArrLen)
        {
            return false;
        }
        for (int i = 0; i < searArrLen ; i++)
        {
            final byte runByte = pSearchArr[i];
            if ( runByte != pArr[i] )
            {
                return false;
            }
        }
        return true;
    }
    /**
     *
     * @param pArr
     * @param pSearchArr
     * @param pPos
     * @return
     */
    public static final boolean startsWith(//--
    final byte[] pArr, //--
    final byte[] pSearchArr, //--
    final int pPos)
    {
//System.out.println("startsWith " + pPos);
        if (pSearchArr.length < 1)
        {
            return true;
        }
        if (pArr.length < pSearchArr.length + pPos)
        {
            return false;
        }
        final int endPos = pPos + pSearchArr.length;
        for (int i = pPos, j = 0; i < endPos ; i++)
        {
            final byte runByte = pSearchArr[j];
            if ( runByte != pArr[i] )
            {
                return false;
            }
            j++;
        }
        return true;
    }

    /**
     * Return the index of the first occurence within the specified
     * byte array.
     *
     * @param pArr
     * @param pSearchArr
     * @return
     */
    public static final int indexOf(//--
    final byte[] pArr, //--
    final byte[] pSearchArr)
    {
        return indexOf(pArr, pSearchArr, 0);
    }
    /**
     * Return the index of the first occurence within the specified
     * byte array.
     *
     * @param pArr
     * @param pSearchArr
     * @return
     */
    public static final int indexOf(//--
    final byte[] pArr, //--
    final byte[] pSearchArr, //--
    final int pFromPos)
    {
        return indexOf(pArr, pSearchArr, pFromPos, pArr.length);
    }
    /**
     * Return the index of the first occurence within the specified
     * byte array.
     *
     * @param pArr
     * @param pSearchArr
     * @return
     */
    public static final int indexOf(//--
    final byte[] pArr, //--
    final byte[] pSearchArr, //--
    final int pFromPos, //--
    final int pToPos)
    {
//System.out.println("indexOf");
        if (pToPos > pArr.length)
        {
            throw new ArrayIndexOutOfBoundsException(pToPos);
        }
        final int searchArrLen = pSearchArr.length;
        if (searchArrLen < 1)
        {
            return 0;
        }
        //if (pArr.length < pSearchArr.length + pFromPos)
        if (pToPos < searchArrLen + pFromPos)
        {
            return -1;
        }
//System.out.println("pArr.length: " + pArr.length);
        //final int searchLength = pArr.length - pSearchArr.length + 1;
        final int searchLength = pToPos - searchArrLen + 1;
        for (int i = pFromPos; i < searchLength; i++)
        {
//System.out.println("i: " + i);
            final byte startByte = pArr[i];
            if (startByte == pSearchArr[0])
            {
                boolean isMatch = true;
                for (int j = 0; j < searchArrLen //--
                    //&& (i + j) < pArr.length
                    ; j++)
                {
//System.out.println("j: " + j);
                    final byte runByte = pSearchArr[j];
//System.out.println("runByte: " + runByte );
//System.out.println("pArr[i + j]: " + pArr[i + j] );
                    if (runByte != pArr[i + j])
                    {
                        isMatch = false;
                        break;
                    }
                }
                if (isMatch)
                {
//System.out.println("Str8#indexOf " + i);
                    return i;
                }
            }
        }
//System.out.println("Str8#indexOf -1");
        return -1;
    }

    /**
     * Return the last index of the first occurence within the specified
     * byte array.
     *
     * @param pArr
     * @param pSearchArr
     * @return
     */
    public static final int lastIndexOf(//--
    final byte[] pArr, //--
    final byte[] pSearchArr)
    {
        return lastIndexOf(pArr, pSearchArr, 0);
    }
    /**
     * Return the last index of the first occurence within the specified
     * byte array.
     *
     * @param pArr
     * @param pSearchArr
     * @return
     */
    public static final int lastIndexOf(//--
    final byte[] pArr, //--
    final byte[] pSearchArr, //--
    final int pFromPos)
    {
        return lastIndexOf(pArr, pSearchArr, pFromPos, pArr.length);
    }
    /**
     * Return the last index of the first occurence within the specified
     * byte array.
     *
     * @param pArr
     * @param pSearchArr
     * @return
     */
    public static final int lastIndexOf(//--
    final byte[] pArr, //--
    final byte[] pSearchArr, //--
    final int pFromPos, //--
    final int pToPos)
    {
//System.out.println("indexOf");
        if (pToPos > pArr.length)
        {
            throw new ArrayIndexOutOfBoundsException(pToPos);
        }
        final int searchArrLen = pSearchArr.length;
        if (searchArrLen < 1)
        {
            return 0;
        }
        //if (pArr.length < pSearchArr.length + pFromPos)
        if (pToPos < searchArrLen + pFromPos)
        {
            return -1;
        }
//System.out.println("pArr.length: " + pArr.length);
        //final int searchLength = pArr.length - pSearchArr.length + 1;
        final int searchLength = pToPos - searchArrLen + 1;
        for (int i = searchLength - 1; i >= pFromPos ; i--)
        {
//System.out.println("i: " + i);
            final byte startByte = pArr[i];
            if (startByte == pSearchArr[0])
            {
                boolean isMatch = true;
                for (int j = 0; j < searchArrLen //--
                    //&& (i + j) < pArr.length
                    ; j++)
                {
//System.out.println("j: " + j);
                    final byte runByte = pSearchArr[j];
//System.out.println("runByte: " + runByte );
//System.out.println("pArr[i + j]: " + pArr[i + j] );
                    if (runByte != pArr[i + j])
                    {
                        isMatch = false;
                        break;
                    }
                }
                if (isMatch)
                {
//System.out.println("Str8#indexOf " + i);
                    return i;
                }
            }
        }
//System.out.println("Str8#indexOf -1");
        return -1;
    }

    /**
     * like java.lang.String#substring.
     *
     * @param pArr
     * @param pFromPos
     * @return
     */
    public static final byte[] sub(//--
    final byte[] pArr, //--
    final int pFromPos)
    {
//System.out.println("sub(1)");
        final int copyLength = pArr.length - pFromPos;
        final byte[] retArr = new byte[copyLength];
        System.arraycopy(pArr, pFromPos, retArr, 0, copyLength);
        return retArr;
    }

    /**
     * like java.lang.String#substring.
     *
     * @param pArr
     * @param pFromPos
     * @return
     */
    public static final byte[] sub(//--
    final byte[] pArr, //--
    final int pFromPos, //--
    final int pToPos)
    {
//System.out.println("sub(2)");
        final int copyLength = pToPos - pFromPos;
        final byte[] retArr = new byte[copyLength];
        System.arraycopy(pArr, pFromPos, retArr, 0, copyLength);
        return retArr;
    }

    /**
     * like java.lang.String#substring.
     *
     * @param pArr
     * @param pFromPos
     * @return
     */
    public static final byte[] left(//--
    final byte[] pArr, //--
    final int pToPos)
    {
        final int copyLength = pToPos;
        final byte[] retArr = new byte[copyLength];
        System.arraycopy(pArr, 0, retArr, 0, copyLength);
        return retArr;
    }

    /**
     * get the specified number of left bytes from the
     * specified byte array.
     *
     * @param pArr
     * @param pToPos
     * @return
     */
    public static byte[] leftLenient( //--
    final byte[] pArr, //--
    final int pToPos)
    {
        if ( pToPos <= pArr.length)
        {
            return left(pArr, pToPos);
        }
        return pArr;
    }

    /**
     * @param pBs
     * @param pI
     * @param pI2
     * @return
     */
    public static byte[] subLenient(
    final byte[] pArr, //--
    final int pFromPos, //--
    final int pToPos)
    {
//System.out.println("subLenient");
        final int copyLength = Math.max( Math.min( pToPos , pArr.length - 1 ) - Math.max( pFromPos , 0 ) , 0 );
        final byte[] retArr = new byte[copyLength];
        System.arraycopy(pArr, pFromPos, retArr, 0, copyLength);
        return retArr;
    }

    /**
     * @param bArr
     * @param pArr
     * @param size
     * @return
     */
    public static boolean endsWith( //--
        final byte[] pArr, //--
        final byte[] pSearchArr)
    {
        return endsWith(pArr, pSearchArr, pArr.length);
    }
    /**
     * @param bArr
     * @param pArr
     * @param size
     * @return
     */
    public static boolean endsWith( //--
        final byte[] pArr, //--
        final byte[] pSearchArr, //--
        final int pEndPos)
    {
        if (pSearchArr.length > pEndPos)
        {
            return false;
        }
        int j = 0;
        for (int i = pEndPos - pSearchArr.length; i < pEndPos; i++)
        {
            final byte b1 = pArr[i];
            final byte b2 = pSearchArr[j];
            if ( b1 != b2 )
            {
                return false;
            }
            j++;
        }
        return true;
    }

    /**
     * convert byte to int.
     * Beachtung Vorzeichen.
     *
     * @param b
     * @return
     */
    public static int byteToInt( byte b )
    {
      return b & 0xff;
    }

    /**
     * Test.
     * @param args
     */
    public static void main(String[] args)
    {
        final byte[] abc = "abc".getBytes();
        final byte[] abcabc = add(abc, abc);
        System.out.println(new String(abcabc));

        boolean startsWith = startsWith(abcabc, abc);
        System.out.println("startsWith: " + startsWith);

        startsWith = startsWith(abcabc, abc, 3);
        System.out.println("startsWith: " + startsWith);

        boolean endsWith = endsWith(abcabc, abc);
        System.out.println("endsWith: " + endsWith);

        endsWith = endsWith(abcabc, abc, 3);
        System.out.println("endsWith: " + endsWith);

        int index = indexOf(abcabc, "bc".getBytes());
        System.out.println("index: " + index);

        byte[] subArr = sub( abcabc, 3 );
        System.out.println(new String( subArr ));

        subArr = sub( abcabc, 3 , 5 );
        System.out.println(new String( subArr ));

        byte[] a1 = "abcde\r\n.\r\n".getBytes();
        index = indexOf(a1, "\r\n.\r\n".getBytes() );
        System.out.println("index: " + index);
    }
}
